"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.range = exports.mapAsync = exports.isSubset = exports.enumerate = exports.concatUnique = exports.assertRemoveAtIndex = exports.assertReplaceFirstInstance = exports.assertRemoveFirstInstance = exports.assertReplaceAtIndex = exports.assertNonZeroLength = exports.assertIncludes = exports.assertGet = exports.all = void 0;
const _1 = require(".");
function all(collection, predicate = (value) => Boolean(value)) {
    for (const element of collection) {
        if (!predicate(element)) {
            return false;
        }
    }
    return true;
}
exports.all = all;
function assertGet(collection, index, message, details) {
    return _1.Assert.asDefined(collection[index], message, details);
}
exports.assertGet = assertGet;
function assertIncludes(collection, element, message, details) {
    _1.Assert.isTrue(collection.includes(element), message !== null && message !== void 0 ? message : "collection.includes(element) failed", details !== null && details !== void 0 ? details : { collection, element });
}
exports.assertIncludes = assertIncludes;
function assertNonZeroLength(collection, message, details) {
    _1.Assert.isTrue(collection.length > 0, message !== null && message !== void 0 ? message : `collection should have at least one element in it`, details !== null && details !== void 0 ? details : {
        collectionLength: collection.length,
    });
}
exports.assertNonZeroLength = assertNonZeroLength;
function assertReplaceAtIndex(collection, value, index) {
    _1.Assert.isFalse(index < 0 || index >= collection.length, "index < 0 || index >= collection.length", {
        index,
        collectionLength: collection.length,
    });
    return [...collection.slice(0, index), value, ...collection.slice(index + 1)];
}
exports.assertReplaceAtIndex = assertReplaceAtIndex;
function assertRemoveFirstInstance(collection, element) {
    return assertRemoveAtIndex(collection, collection.indexOf(element));
}
exports.assertRemoveFirstInstance = assertRemoveFirstInstance;
function assertReplaceFirstInstance(collection, oldValue, newValue) {
    return assertReplaceAtIndex(collection, newValue, collection.indexOf(oldValue));
}
exports.assertReplaceFirstInstance = assertReplaceFirstInstance;
function assertRemoveAtIndex(collection, index) {
    _1.Assert.isFalse(index < 0 || index >= collection.length, "index < 0 || index >= collection.length", {
        index,
        collectionLength: collection.length,
    });
    return [...collection.slice(0, index), ...collection.slice(index + 1)];
}
exports.assertRemoveAtIndex = assertRemoveAtIndex;
function concatUnique(left, right) {
    const partial = [...left];
    for (const element of right) {
        if (partial.indexOf(element) === -1) {
            partial.push(element);
        }
    }
    return partial;
}
exports.concatUnique = concatUnique;
function enumerate(collection) {
    return collection.map((value, index) => [value, index]);
}
exports.enumerate = enumerate;
function isSubset(largerCollection, smallerCollection, comparer = (left, right) => left === right) {
    if (smallerCollection.length > largerCollection.length) {
        return false;
    }
    for (const smallerCollectionValue of smallerCollection) {
        let foundMatch = false;
        for (const largerCollectionValue of largerCollection) {
            if (comparer(smallerCollectionValue, largerCollectionValue)) {
                foundMatch = true;
                break;
            }
        }
        if (!foundMatch) {
            return false;
        }
    }
    return true;
}
exports.isSubset = isSubset;
function mapAsync(collection, map) {
    return __awaiter(this, void 0, void 0, function* () {
        const tasks = collection.map(map);
        return yield Promise.all(tasks);
    });
}
exports.mapAsync = mapAsync;
function range(size, startAt = 0) {
    // tslint:disable-next-line: prefer-array-literal
    return [...Array(size).keys()].map((index) => index + startAt);
}
exports.range = range;
//# sourceMappingURL=arrayUtils.js.map