"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.NoOpCancellationToken = exports.ExpiredCancellationToken = exports.TimedCancellationToken = exports.CounterCancellationToken = void 0;
const __1 = require("..");
// Cancelled after X calls are made to isCancelled.
// Not really useful other than as an example of how to create your own cancellation token.
class CounterCancellationToken {
    constructor(cancellationThreshold) {
        this.cancellationThreshold = cancellationThreshold;
        this.wasForceCancelled = false;
        this.counter = 0;
    }
    isCancelled() {
        this.counter += 1;
        return this.wasForceCancelled || this.counter >= this.cancellationThreshold;
    }
    throwIfCancelled() {
        var _a;
        if (this.isCancelled()) {
            throw new __1.CommonError.CancellationError(this, (_a = this.cancelReason) !== null && _a !== void 0 ? _a : `Exceeded ${this.cancellationThreshold} calls`);
        }
    }
    cancel(reason) {
        this.wasForceCancelled = true;
        this.cancelReason = reason;
    }
}
exports.CounterCancellationToken = CounterCancellationToken;
// Cancelled after X milliseconds.
class TimedCancellationToken {
    constructor(milliseconds) {
        this.milliseconds = milliseconds;
        this.threshold = Date.now() + milliseconds;
        this.wasForceCancelled = false;
    }
    throwIfCancelled() {
        var _a;
        if (this.isCancelled()) {
            throw new __1.CommonError.CancellationError(this, (_a = this.cancelReason) !== null && _a !== void 0 ? _a : `Exceeded ${this.milliseconds}ms`);
        }
    }
    isCancelled() {
        return this.wasForceCancelled || Date.now() >= this.threshold;
    }
    cancel(reason) {
        this.wasForceCancelled = true;
        this.cancelReason = reason;
    }
}
exports.TimedCancellationToken = TimedCancellationToken;
exports.ExpiredCancellationToken = new TimedCancellationToken(0);
// In case you need to provide a cancellation token but don't want to support cancellation.
exports.NoOpCancellationToken = {
    isCancelled: () => false,
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    throwIfCancelled: () => { },
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    cancel: () => { },
};
//# sourceMappingURL=cancellationToken.js.map