"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.ensureCommonError = exports.throwIfCancellationError = exports.isTInnerCommonError = exports.isCancellationError = exports.isCommonError = exports.assertIsCommonError = exports.UnknownError = exports.InvariantError = exports.CancellationError = exports.CommonError = void 0;
const localization_1 = require("../localization");
const _1 = require(".");
class CommonError extends Error {
    constructor(innerError) {
        super(innerError.message);
        this.innerError = innerError;
        Object.setPrototypeOf(this, CommonError.prototype);
    }
}
exports.CommonError = CommonError;
class CancellationError extends Error {
    constructor(cancellationToken, reason) {
        super(localization_1.Localization.error_common_cancellationError(localization_1.Templates.DefaultTemplates, reason));
        this.cancellationToken = cancellationToken;
        this.reason = reason;
        Object.setPrototypeOf(this, CancellationError.prototype);
    }
}
exports.CancellationError = CancellationError;
class InvariantError extends Error {
    constructor(invariantBroken, details) {
        super(localization_1.Localization.error_common_invariantError(localization_1.Templates.DefaultTemplates, invariantBroken, details));
        this.invariantBroken = invariantBroken;
        this.details = details;
        Object.setPrototypeOf(this, InvariantError.prototype);
    }
}
exports.InvariantError = InvariantError;
class UnknownError extends Error {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    constructor(innerError, locale) {
        super(localization_1.Localization.error_common_unknown(localization_1.LocalizationUtils.getLocalizationTemplates(locale), innerError));
        this.innerError = innerError;
        Object.setPrototypeOf(this, UnknownError.prototype);
    }
}
exports.UnknownError = UnknownError;
function assertIsCommonError(error) {
    _1.Assert.isTrue(isCommonError(error), "isCommonError(error)");
    return true;
}
exports.assertIsCommonError = assertIsCommonError;
function isCommonError(error) {
    return error instanceof CommonError;
}
exports.isCommonError = isCommonError;
function isCancellationError(error) {
    return error instanceof CancellationError;
}
exports.isCancellationError = isCancellationError;
function isTInnerCommonError(error) {
    return error instanceof CancellationError || error instanceof InvariantError || error instanceof UnknownError;
}
exports.isTInnerCommonError = isTInnerCommonError;
function throwIfCancellationError(error) {
    if (isCancellationError(error)) {
        throw error;
    }
}
exports.throwIfCancellationError = throwIfCancellationError;
function ensureCommonError(error, locale) {
    if (error instanceof CommonError) {
        return error;
    }
    else if (isTInnerCommonError(error)) {
        return new CommonError(error);
    }
    else {
        return new CommonError(new UnknownError(error, locale));
    }
}
exports.ensureCommonError = ensureCommonError;
//# sourceMappingURL=error.js.map