"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.pick = exports.isSubsetMap = exports.isEqualMap = exports.hasKeys = exports.filter = exports.assertNotIn = exports.assertHas = exports.assertGet = exports.assertDelete = void 0;
const _1 = require(".");
function assertDelete(map, key, message, details) {
    _1.Assert.isTrue(map.delete(key), message !== null && message !== void 0 ? message : `failed to delete, key is absent`, details !== null && details !== void 0 ? details : { key });
}
exports.assertDelete = assertDelete;
function assertGet(map, key, message, details) {
    return _1.Assert.asDefined(map.get(key), message !== null && message !== void 0 ? message : `key not found in given map`, details !== null && details !== void 0 ? details : { key });
}
exports.assertGet = assertGet;
function assertHas(map, key, message) {
    _1.Assert.isTrue(map.has(key), message !== null && message !== void 0 ? message : `key is absent`, { key });
}
exports.assertHas = assertHas;
function assertNotIn(map, key, message) {
    _1.Assert.isFalse(map.has(key), message !== null && message !== void 0 ? message : `key is present`, { key });
}
exports.assertNotIn = assertNotIn;
function filter(map, predicate) {
    const filtered = new Map();
    for (const [key, value] of map.entries()) {
        if (predicate(key, value)) {
            filtered.set(key, value);
        }
    }
    return filtered;
}
exports.filter = filter;
function hasKeys(map, keys) {
    for (const key of keys) {
        if (!map.has(key)) {
            return false;
        }
    }
    return true;
}
exports.hasKeys = hasKeys;
function isEqualMap(left, right, comparer) {
    if (left.size !== right.size) {
        return false;
    }
    for (const [leftKey, leftValue] of left.entries()) {
        const value = right.get(leftKey);
        if (value === undefined || !comparer(leftValue, value)) {
            return false;
        }
    }
    return true;
}
exports.isEqualMap = isEqualMap;
function isSubsetMap(left, right, comparer) {
    if (left.size > right.size) {
        return false;
    }
    for (const [key, leftType] of left.entries()) {
        const rightType = right.get(key);
        if (rightType === undefined || !comparer(leftType, rightType)) {
            return false;
        }
    }
    return true;
}
exports.isSubsetMap = isSubsetMap;
function pick(map, keys) {
    const newMap = new Map();
    for (const key of keys) {
        newMap.set(key, _1.Assert.asDefined(map.get(key), `key from keys is not found in map`, { key }));
    }
    return newMap;
}
exports.pick = pick;
//# sourceMappingURL=mapUtils.js.map