"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.OrderedMap = void 0;
const _1 = require(".");
class OrderedMap {
    constructor(entries) {
        if (!entries) {
            this.map = new Map();
            this.order = [];
            this.size = 0;
        }
        else {
            this.map = new Map(entries);
            if (entries instanceof Map) {
                this.order = [...entries.keys()];
                this.size = entries.size;
            }
            else {
                this.order = entries.map((pair) => pair[0]);
                this.size = entries.length;
            }
        }
    }
    [(Symbol.toStringTag, Symbol.iterator)]() {
        return this.entries();
    }
    clear() {
        this.map.clear();
        this.order = [];
    }
    delete(key) {
        if (this.map.delete(key)) {
            this.order = _1.ArrayUtils.assertRemoveFirstInstance(this.order, key);
            return true;
        }
        else {
            return false;
        }
    }
    *entries() {
        for (const key of this.order) {
            yield [key, _1.Assert.asDefined(this.map.get(key))];
        }
    }
    forEach(callback) {
        for (const [key, value] of this.entries()) {
            callback(value, key, this.map);
        }
    }
    get(key) {
        return this.map.get(key);
    }
    has(key) {
        return this.map.has(key);
    }
    keys() {
        return this.map.keys();
    }
    set(key, value, maintainIndex) {
        if (this.has(key)) {
            if (!maintainIndex) {
                this.order = [..._1.ArrayUtils.assertRemoveFirstInstance(this.order, key), key];
            }
        }
        else {
            this.order = [...this.order, key];
        }
        this.map.set(key, value);
        return this;
    }
    values() {
        return this.map.values();
    }
}
exports.OrderedMap = OrderedMap;
//# sourceMappingURL=orderedMap.js.map