"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.okOrDefault = exports.isError = exports.isOk = exports.ensureResultAsync = exports.ensureResult = exports.error = exports.ok = exports.assertError = exports.assertOk = exports.assertIsError = exports.assertIsOk = void 0;
const __1 = require("..");
const result_1 = require("./result");
function assertIsOk(result) {
    if (!isOk(result)) {
        throw new __1.CommonError.InvariantError(`assert failed, result expected to be an Ok`, {
            error: result.error,
        });
    }
}
exports.assertIsOk = assertIsOk;
function assertIsError(result) {
    if (!isError(result)) {
        throw new __1.CommonError.InvariantError(`assert failed, result expected to be an Error`);
    }
}
exports.assertIsError = assertIsError;
function assertOk(result) {
    assertIsOk(result);
    return result.value;
}
exports.assertOk = assertOk;
function assertError(result) {
    assertIsError(result);
    return result.error;
}
exports.assertError = assertError;
function ok(value) {
    return {
        kind: result_1.ResultKind.Ok,
        value,
    };
}
exports.ok = ok;
function error(error) {
    return {
        kind: result_1.ResultKind.Error,
        error,
    };
}
exports.error = error;
function ensureResult(callback, locale) {
    try {
        return ok(callback());
    }
    catch (caught) {
        __1.Assert.isInstanceofError(caught);
        return error(__1.CommonError.ensureCommonError(caught, locale));
    }
}
exports.ensureResult = ensureResult;
function ensureResultAsync(callback, locale) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            return ok(yield callback());
        }
        catch (caught) {
            __1.Assert.isInstanceofError(caught);
            return error(__1.CommonError.ensureCommonError(caught, locale));
        }
    });
}
exports.ensureResultAsync = ensureResultAsync;
function isOk(result) {
    return result.kind === result_1.ResultKind.Ok;
}
exports.isOk = isOk;
function isError(result) {
    return result.kind === result_1.ResultKind.Error;
}
exports.isError = isError;
function okOrDefault(result, defaultValue) {
    return isOk(result) ? result.value : defaultValue;
}
exports.okOrDefault = okOrDefault;
//# sourceMappingURL=resultUtils.js.map