import { CommonError, Result } from ".";
import { NodeIdMap, TXorNode } from "../parser";
import { Ast } from "../language";
import { Settings } from "..";
export declare type TriedTraverse<ResultType> = Result<ResultType, CommonError.CommonError>;
export declare type TVisitNodeFn<State extends ITraversalState<ResultType>, ResultType, Node, Return> = (state: State, node: Node, correlationId: number | undefined) => Promise<Return>;
export declare type TEarlyExitFn<State extends ITraversalState<ResultType>, ResultType, Node> = TVisitNodeFn<State, ResultType, Node, boolean>;
export declare type TExpandNodesFn<State extends ITraversalState<ResultType>, ResultType, Node, NodesById> = (state: State, node: Node, collection: NodesById) => Promise<ReadonlyArray<Node>>;
export declare enum VisitNodeStrategy {
    BreadthFirst = "BreadthFirst",
    DepthFirst = "DepthFirst"
}
export interface ITraversalState<T> extends Pick<Settings, "locale" | "cancellationToken" | "traceManager"> {
    readonly initialCorrelationId: number | undefined;
    result: T;
}
export declare function tryTraverse<State extends ITraversalState<ResultType>, ResultType, Node, NodesById>(state: State, nodesById: NodesById, root: Node, strategy: VisitNodeStrategy, visitNodeFn: TVisitNodeFn<State, ResultType, Node, void>, expandNodesFn: TExpandNodesFn<State, ResultType, Node, NodesById>, earlyExitFn: TEarlyExitFn<State, ResultType, Node> | undefined): Promise<TriedTraverse<ResultType>>;
export declare function tryTraverseAst<State extends ITraversalState<ResultType>, ResultType>(state: State, nodeIdMapCollection: NodeIdMap.Collection, root: Ast.TNode, strategy: VisitNodeStrategy, visitNodeFn: TVisitNodeFn<State, ResultType, Ast.TNode, void>, expandNodesFn: TExpandNodesFn<State, ResultType, Ast.TNode, NodeIdMap.Collection>, earlyExitFn: TEarlyExitFn<State, ResultType, Ast.TNode> | undefined): Promise<TriedTraverse<ResultType>>;
export declare function tryTraverseXor<State extends ITraversalState<ResultType>, ResultType>(state: State, nodeIdMapCollection: NodeIdMap.Collection, root: TXorNode, strategy: VisitNodeStrategy, visitNodeFn: TVisitNodeFn<State, ResultType, TXorNode, void>, expandNodesFn: TExpandNodesFn<State, ResultType, TXorNode, NodeIdMap.Collection>, earlyExitFn: TEarlyExitFn<State, ResultType, TXorNode> | undefined): Promise<TriedTraverse<ResultType>>;
export declare function assertGetAllAstChildren<State extends ITraversalState<ResultType>, ResultType>(_state: State, astNode: Ast.TNode, nodeIdMapCollection: NodeIdMap.Collection): Promise<ReadonlyArray<Ast.TNode>>;
export declare function assertGetAllXorChildren<State extends ITraversalState<ResultType>, ResultType>(_state: State, xorNode: TXorNode, nodeIdMapCollection: NodeIdMap.Collection): Promise<ReadonlyArray<TXorNode>>;
export declare function expandXorParent<T>(_state: T, xorNode: TXorNode, nodeIdMapCollection: NodeIdMap.Collection): Promise<ReadonlyArray<TXorNode>>;
