"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.expandXorParent = exports.assertGetAllXorChildren = exports.assertGetAllAstChildren = exports.tryTraverseXor = exports.tryTraverseAst = exports.tryTraverse = exports.VisitNodeStrategy = void 0;
const parser_1 = require("../parser");
const result_1 = require("./result");
var VisitNodeStrategy;
(function (VisitNodeStrategy) {
    VisitNodeStrategy["BreadthFirst"] = "BreadthFirst";
    VisitNodeStrategy["DepthFirst"] = "DepthFirst";
})(VisitNodeStrategy = exports.VisitNodeStrategy || (exports.VisitNodeStrategy = {}));
// Most commonly used by visiting all nodes (either Ast or TXorNode) with the visitNodeFn,
// It usually expands all children nodes but can expand the list of nodes to visit through other logic.
function tryTraverse(state, nodesById, root, strategy, visitNodeFn, expandNodesFn, earlyExitFn) {
    return result_1.ResultUtils.ensureResultAsync(() => __awaiter(this, void 0, void 0, function* () {
        yield traverseRecursion(state, nodesById, root, strategy, visitNodeFn, expandNodesFn, earlyExitFn, state.initialCorrelationId);
        return state.result;
    }), state.locale);
}
exports.tryTraverse = tryTraverse;
// sets the tryTraverse type parameters: Node and NodesById
function tryTraverseAst(state, nodeIdMapCollection, root, strategy, visitNodeFn, expandNodesFn, earlyExitFn) {
    return tryTraverse(state, nodeIdMapCollection, root, strategy, visitNodeFn, expandNodesFn, earlyExitFn);
}
exports.tryTraverseAst = tryTraverseAst;
// sets the tryTraverse type parameters: Node and NodesById
function tryTraverseXor(state, nodeIdMapCollection, root, strategy, visitNodeFn, expandNodesFn, earlyExitFn) {
    return tryTraverse(state, nodeIdMapCollection, root, strategy, visitNodeFn, expandNodesFn, earlyExitFn);
}
exports.tryTraverseXor = tryTraverseXor;
// a TExpandNodesFn usable by tryTraverseAst which visits all Ast nodes.
// eslint-disable-next-line require-await
function assertGetAllAstChildren(_state, astNode, nodeIdMapCollection) {
    return __awaiter(this, void 0, void 0, function* () {
        const childIds = nodeIdMapCollection.childIdsById.get(astNode.id);
        return childIds ? childIds.map((nodeId) => parser_1.NodeIdMapUtils.assertAst(nodeIdMapCollection, nodeId)) : [];
    });
}
exports.assertGetAllAstChildren = assertGetAllAstChildren;
// a TExpandNodesFn usable by tryTraverseXor which visits all nodes.
// eslint-disable-next-line require-await
function assertGetAllXorChildren(_state, xorNode, nodeIdMapCollection) {
    return __awaiter(this, void 0, void 0, function* () {
        return parser_1.NodeIdMapIterator.assertIterChildrenXor(nodeIdMapCollection, xorNode.node.id);
    });
}
exports.assertGetAllXorChildren = assertGetAllXorChildren;
// Returns the TXorNode's parent if one exists.
function expandXorParent(_state, xorNode, nodeIdMapCollection) {
    const parent = parser_1.NodeIdMapUtils.parentXor(nodeIdMapCollection, xorNode.node.id);
    return Promise.resolve(parent !== undefined ? [parent] : []);
}
exports.expandXorParent = expandXorParent;
// The core logic of a traversal.
function traverseRecursion(state, nodesById, node, strategy, visitNode, visitOrderQueue, isEarlyExit, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Traversal" /* Traversal */, traverseRecursion.name, correlationId);
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        if (isEarlyExit && (yield isEarlyExit(state, node, trace.id))) {
            return;
        }
        else if (strategy === VisitNodeStrategy.BreadthFirst) {
            yield visitNode(state, node, trace.id);
        }
        for (const child of yield visitOrderQueue(state, node, nodesById)) {
            // eslint-disable-next-line no-await-in-loop
            yield traverseRecursion(state, nodesById, child, strategy, visitNode, visitOrderQueue, isEarlyExit, trace.id);
        }
        if (strategy === VisitNodeStrategy.DepthFirst) {
            yield visitNode(state, node, trace.id);
        }
        trace.exit();
    });
}
//# sourceMappingURL=traversal.js.map