"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertIsTPairedConstant = exports.assertIsTKeyValuePair = exports.assertIsTBinOpExpression = exports.assertIsNodeKind = exports.assertAsTPairedConstant = exports.assertAsTKeyValuePair = exports.assertAsTBinOpExpression = exports.assertAsNodeKind = void 0;
const TypeGuards = require("./typeGuards");
const __1 = require("..");
const common_1 = require("../../../common");
function assertAsNodeKind(node, expectedNodeKinds) {
    assertIsNodeKind(node, expectedNodeKinds);
    return node;
}
exports.assertAsNodeKind = assertAsNodeKind;
function assertAsTBinOpExpression(node) {
    return assertAsNodeKind(node, [
        __1.Ast.NodeKind.ArithmeticExpression,
        __1.Ast.NodeKind.AsExpression,
        __1.Ast.NodeKind.EqualityExpression,
        __1.Ast.NodeKind.IsExpression,
        __1.Ast.NodeKind.LogicalExpression,
        __1.Ast.NodeKind.MetadataExpression,
        __1.Ast.NodeKind.RelationalExpression,
    ]);
}
exports.assertAsTBinOpExpression = assertAsTBinOpExpression;
function assertAsTKeyValuePair(node) {
    return assertAsNodeKind(node, [
        __1.Ast.NodeKind.GeneralizedIdentifierPairedAnyLiteral,
        __1.Ast.NodeKind.GeneralizedIdentifierPairedExpression,
        __1.Ast.NodeKind.IdentifierPairedExpression,
    ]);
}
exports.assertAsTKeyValuePair = assertAsTKeyValuePair;
function assertAsTPairedConstant(node) {
    return assertAsNodeKind(node, [
        __1.Ast.NodeKind.AsNullablePrimitiveType,
        __1.Ast.NodeKind.AsType,
        __1.Ast.NodeKind.ErrorRaisingExpression,
        __1.Ast.NodeKind.IsNullablePrimitiveType,
        __1.Ast.NodeKind.NullablePrimitiveType,
        __1.Ast.NodeKind.NullableType,
        __1.Ast.NodeKind.OtherwiseExpression,
        __1.Ast.NodeKind.TypePrimaryType,
    ]);
}
exports.assertAsTPairedConstant = assertAsTPairedConstant;
function assertIsNodeKind(node, expectedNodeKinds) {
    if (!TypeGuards.isNodeKind(node, expectedNodeKinds)) {
        throw new common_1.CommonError.InvariantError(`unexpected node kind`, {
            nodeId: node.id,
            nodeKind: node.kind,
            expectedNodeKinds,
        });
    }
}
exports.assertIsNodeKind = assertIsNodeKind;
function assertIsTBinOpExpression(node) {
    assertIsNodeKind(node, [
        __1.Ast.NodeKind.ArithmeticExpression,
        __1.Ast.NodeKind.AsExpression,
        __1.Ast.NodeKind.EqualityExpression,
        __1.Ast.NodeKind.IsExpression,
        __1.Ast.NodeKind.LogicalExpression,
        __1.Ast.NodeKind.MetadataExpression,
        __1.Ast.NodeKind.RelationalExpression,
    ]);
}
exports.assertIsTBinOpExpression = assertIsTBinOpExpression;
function assertIsTKeyValuePair(node) {
    assertIsNodeKind(node, [
        __1.Ast.NodeKind.GeneralizedIdentifierPairedAnyLiteral,
        __1.Ast.NodeKind.GeneralizedIdentifierPairedExpression,
        __1.Ast.NodeKind.IdentifierPairedExpression,
    ]);
}
exports.assertIsTKeyValuePair = assertIsTKeyValuePair;
function assertIsTPairedConstant(node) {
    assertIsNodeKind(node, [
        __1.Ast.NodeKind.AsNullablePrimitiveType,
        __1.Ast.NodeKind.AsType,
        __1.Ast.NodeKind.ErrorRaisingExpression,
        __1.Ast.NodeKind.IsNullablePrimitiveType,
        __1.Ast.NodeKind.NullablePrimitiveType,
        __1.Ast.NodeKind.NullableType,
        __1.Ast.NodeKind.OtherwiseExpression,
        __1.Ast.NodeKind.TypePrimaryType,
    ]);
}
exports.assertIsTPairedConstant = assertIsTPairedConstant;
//# sourceMappingURL=assert.js.map