"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.primitiveTypeConstantKindFrom = exports.simplifyAsNullablePrimitiveType = exports.nodeKindFromTBinOpExpressionOperator = exports.simplifyType = exports.literalKindFrom = exports.isTUnaryExpression = exports.isTTypeExpression = exports.isTType = exports.isTRelationalExpression = exports.isTRecursivePrimaryExpression = exports.isTPrimaryType = exports.isTPrimaryExpression = exports.isTNullCoalescingExpression = exports.isTNullablePrimitiveType = exports.isTMetadataExpression = exports.isTLogicalExpression = exports.isTListItem = exports.isTLeaf = exports.isTIsExpression = exports.isTFieldAccessExpression = exports.isTExpression = exports.isTEqualityExpression = exports.isTAsExpression = exports.isTArithmeticExpression = exports.isTAnyLiteral = exports.getIdentifierLiteral = exports.assertIsTNullablePrimitiveType = exports.assertIsTUnaryExpression = exports.assertIsLeaf = void 0;
const common_1 = require("../../../common");
const __1 = require("..");
const constant_1 = require("../../constant");
const token_1 = require("../../token");
function assertIsLeaf(node) {
    common_1.Assert.isTrue(node.isLeaf, "Assert(node.isLeaf)", { nodeId: node.id, nodeKind: node.kind });
}
exports.assertIsLeaf = assertIsLeaf;
function assertIsTUnaryExpression(node) {
    if (!isTUnaryExpression(node)) {
        throw new common_1.CommonError.InvariantError("assertIsTUnaryExpression failed", {
            nodeId: node.id,
            nodeKind: node.kind,
        });
    }
}
exports.assertIsTUnaryExpression = assertIsTUnaryExpression;
function assertIsTNullablePrimitiveType(node) {
    if (!isTNullablePrimitiveType(node)) {
        throw new common_1.CommonError.InvariantError("assertIsTNullablePrimitiveType failed", {
            nodeId: node.id,
            nodeKind: node.kind,
        });
    }
}
exports.assertIsTNullablePrimitiveType = assertIsTNullablePrimitiveType;
function getIdentifierLiteral(node) {
    switch (node.kind) {
        case __1.Ast.NodeKind.Identifier:
            return node.literal;
        case __1.Ast.NodeKind.IdentifierExpression:
            return node.inclusiveConstant ? `@${node.identifier.literal}` : node.identifier.literal;
        default:
            throw common_1.Assert.isNever(node);
    }
}
exports.getIdentifierLiteral = getIdentifierLiteral;
function isTAnyLiteral(node) {
    return __1.Ast.NodeKindsForTAnyLiteral.has(node.kind);
}
exports.isTAnyLiteral = isTAnyLiteral;
function isTArithmeticExpression(node) {
    return __1.Ast.NodeKindsForTArithmeticExpression.has(node.kind);
}
exports.isTArithmeticExpression = isTArithmeticExpression;
function isTAsExpression(node) {
    return __1.Ast.NodeKindsForTAsExpression.has(node.kind);
}
exports.isTAsExpression = isTAsExpression;
function isTEqualityExpression(node) {
    return __1.Ast.NodeKindsForTEqualityExpression.has(node.kind);
}
exports.isTEqualityExpression = isTEqualityExpression;
function isTExpression(node) {
    return __1.Ast.NodeKindsForTExpression.has(node.kind);
}
exports.isTExpression = isTExpression;
function isTFieldAccessExpression(node) {
    return __1.Ast.NodeKindsForTFieldAccessExpression.has(node.kind);
}
exports.isTFieldAccessExpression = isTFieldAccessExpression;
function isTIsExpression(node) {
    return __1.Ast.NodeKindsForTIsExpression.has(node.kind);
}
exports.isTIsExpression = isTIsExpression;
function isTLeaf(node) {
    return node.isLeaf;
}
exports.isTLeaf = isTLeaf;
function isTListItem(node) {
    return __1.Ast.NodeKindsForTListItem.has(node.kind);
}
exports.isTListItem = isTListItem;
function isTLogicalExpression(node) {
    return __1.Ast.NodeKindsForTLogicalExpression.has(node.kind);
}
exports.isTLogicalExpression = isTLogicalExpression;
function isTMetadataExpression(node) {
    return __1.Ast.NodeKindsForTMetadataExpression.has(node.kind);
}
exports.isTMetadataExpression = isTMetadataExpression;
function isTNullablePrimitiveType(node) {
    return __1.Ast.NodeKindsForTNullablePrimitiveType.has(node.kind);
}
exports.isTNullablePrimitiveType = isTNullablePrimitiveType;
function isTNullCoalescingExpression(node) {
    return __1.Ast.NodeKindsForTNullCoalescingExpression.has(node.kind);
}
exports.isTNullCoalescingExpression = isTNullCoalescingExpression;
function isTPrimaryExpression(node) {
    return __1.Ast.NodeKindsForTPrimaryExpression.has(node.kind);
}
exports.isTPrimaryExpression = isTPrimaryExpression;
function isTPrimaryType(node) {
    return __1.Ast.NodeKindsForTPrimaryType.has(node.kind);
}
exports.isTPrimaryType = isTPrimaryType;
function isTRecursivePrimaryExpression(node) {
    return __1.Ast.NodeKindsForTRecursivePrimaryExpression.has(node.kind);
}
exports.isTRecursivePrimaryExpression = isTRecursivePrimaryExpression;
function isTRelationalExpression(node) {
    return __1.Ast.NodeKindsForTRelationalExpression.has(node.kind);
}
exports.isTRelationalExpression = isTRelationalExpression;
function isTType(node) {
    return __1.Ast.NodeKindsForTType.has(node.kind);
}
exports.isTType = isTType;
function isTTypeExpression(node) {
    return __1.Ast.NodeKindsForTTypeExpression.has(node.kind);
}
exports.isTTypeExpression = isTTypeExpression;
function isTUnaryExpression(node) {
    return __1.Ast.NodeKindsForTUnaryExpression.has(node.kind);
}
exports.isTUnaryExpression = isTUnaryExpression;
function literalKindFrom(tokenKind) {
    switch (tokenKind) {
        case token_1.TokenKind.HexLiteral:
        case token_1.TokenKind.KeywordHashNan:
        case token_1.TokenKind.KeywordHashInfinity:
        case token_1.TokenKind.NumericLiteral:
            return __1.Ast.LiteralKind.Numeric;
        case token_1.TokenKind.KeywordFalse:
        case token_1.TokenKind.KeywordTrue:
            return __1.Ast.LiteralKind.Logical;
        case token_1.TokenKind.NullLiteral:
            return __1.Ast.LiteralKind.Null;
        case token_1.TokenKind.TextLiteral:
            return __1.Ast.LiteralKind.Text;
        default:
            return undefined;
    }
}
exports.literalKindFrom = literalKindFrom;
function simplifyType(type) {
    let isNullable;
    let primitiveTypeConstantKind;
    switch (type.kind) {
        case __1.Ast.NodeKind.PrimitiveType:
            isNullable = false;
            primitiveTypeConstantKind = type.primitiveTypeKind;
            break;
        case __1.Ast.NodeKind.NullableType:
            isNullable = true;
            primitiveTypeConstantKind = simplifyType(type.paired).primitiveTypeConstantKind;
            break;
        case __1.Ast.NodeKind.FunctionType:
            isNullable = false;
            primitiveTypeConstantKind = constant_1.Constant.PrimitiveTypeConstant.Function;
            break;
        case __1.Ast.NodeKind.ListType:
            isNullable = false;
            primitiveTypeConstantKind = constant_1.Constant.PrimitiveTypeConstant.List;
            break;
        case __1.Ast.NodeKind.RecordType:
            isNullable = false;
            primitiveTypeConstantKind = constant_1.Constant.PrimitiveTypeConstant.Record;
            break;
        case __1.Ast.NodeKind.TableType:
            isNullable = false;
            primitiveTypeConstantKind = constant_1.Constant.PrimitiveTypeConstant.Table;
            break;
        default:
            throw new common_1.CommonError.InvariantError("this should never be reached", {
                nodeId: type.id,
                nodeKind: type.kind,
            });
    }
    return {
        isNullable,
        primitiveTypeConstantKind,
    };
}
exports.simplifyType = simplifyType;
function nodeKindFromTBinOpExpressionOperator(constantKind) {
    switch (constantKind) {
        case constant_1.Constant.ArithmeticOperator.Multiplication:
        case constant_1.Constant.ArithmeticOperator.Division:
        case constant_1.Constant.ArithmeticOperator.Addition:
        case constant_1.Constant.ArithmeticOperator.Subtraction:
        case constant_1.Constant.ArithmeticOperator.And:
            return __1.Ast.NodeKind.ArithmeticExpression;
        case constant_1.Constant.EqualityOperator.EqualTo:
        case constant_1.Constant.EqualityOperator.NotEqualTo:
            return __1.Ast.NodeKind.EqualityExpression;
        case constant_1.Constant.LogicalOperator.And:
        case constant_1.Constant.LogicalOperator.Or:
            return __1.Ast.NodeKind.LogicalExpression;
        case constant_1.Constant.RelationalOperator.LessThan:
        case constant_1.Constant.RelationalOperator.LessThanEqualTo:
        case constant_1.Constant.RelationalOperator.GreaterThan:
        case constant_1.Constant.RelationalOperator.GreaterThanEqualTo:
            return __1.Ast.NodeKind.RelationalExpression;
        case constant_1.Constant.KeywordConstant.As:
            return __1.Ast.NodeKind.AsExpression;
        case constant_1.Constant.KeywordConstant.Is:
            return __1.Ast.NodeKind.IsExpression;
        case constant_1.Constant.KeywordConstant.Meta:
            return __1.Ast.NodeKind.MetadataExpression;
        case constant_1.Constant.MiscConstant.NullCoalescingOperator:
            return __1.Ast.NodeKind.NullCoalescingExpression;
        default:
            throw common_1.Assert.isNever(constantKind);
    }
}
exports.nodeKindFromTBinOpExpressionOperator = nodeKindFromTBinOpExpressionOperator;
function simplifyAsNullablePrimitiveType(node) {
    let isNullable;
    let primitiveTypeConstantKind;
    const nullablePrimitiveType = node.paired;
    switch (nullablePrimitiveType.kind) {
        case __1.Ast.NodeKind.NullablePrimitiveType:
            isNullable = true;
            primitiveTypeConstantKind = nullablePrimitiveType.paired.primitiveTypeKind;
            break;
        case __1.Ast.NodeKind.PrimitiveType:
            isNullable = false;
            primitiveTypeConstantKind = nullablePrimitiveType.primitiveTypeKind;
            break;
        default:
            throw common_1.Assert.isNever(nullablePrimitiveType);
    }
    return {
        primitiveTypeConstantKind,
        isNullable,
    };
}
exports.simplifyAsNullablePrimitiveType = simplifyAsNullablePrimitiveType;
function primitiveTypeConstantKindFrom(node) {
    switch (node.kind) {
        case __1.Ast.NodeKind.AsNullablePrimitiveType:
            return primitiveTypeConstantKindFrom(node.paired);
        case __1.Ast.NodeKind.NullablePrimitiveType:
            return node.paired.primitiveTypeKind;
        case __1.Ast.NodeKind.PrimitiveType:
            return node.primitiveTypeKind;
        default:
            throw common_1.Assert.isNever(node);
    }
}
exports.primitiveTypeConstantKindFrom = primitiveTypeConstantKindFrom;
//# sourceMappingURL=astUtils.js.map