"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.isTPairedConstant = exports.isTKeyValuePair = exports.isTBinOpExpressionKind = exports.isTBinOpExpression = exports.isNodeKind = void 0;
const __1 = require("..");
function isNodeKind(node, expectedNodeKinds) {
    return Array.isArray(expectedNodeKinds) ? expectedNodeKinds.includes(node.kind) : node.kind === expectedNodeKinds;
}
exports.isNodeKind = isNodeKind;
function isTBinOpExpression(node) {
    return isNodeKind(node, [
        __1.Ast.NodeKind.ArithmeticExpression,
        __1.Ast.NodeKind.AsExpression,
        __1.Ast.NodeKind.EqualityExpression,
        __1.Ast.NodeKind.IsExpression,
        __1.Ast.NodeKind.LogicalExpression,
        __1.Ast.NodeKind.NullCoalescingExpression,
        __1.Ast.NodeKind.MetadataExpression,
        __1.Ast.NodeKind.RelationalExpression,
    ]);
}
exports.isTBinOpExpression = isTBinOpExpression;
function isTBinOpExpressionKind(nodeKind) {
    switch (nodeKind) {
        case __1.Ast.NodeKind.ArithmeticExpression:
        case __1.Ast.NodeKind.AsExpression:
        case __1.Ast.NodeKind.EqualityExpression:
        case __1.Ast.NodeKind.IsExpression:
        case __1.Ast.NodeKind.LogicalExpression:
        case __1.Ast.NodeKind.NullCoalescingExpression:
        case __1.Ast.NodeKind.MetadataExpression:
        case __1.Ast.NodeKind.RelationalExpression:
            return true;
        default:
            return false;
    }
}
exports.isTBinOpExpressionKind = isTBinOpExpressionKind;
function isTKeyValuePair(node) {
    switch (node.kind) {
        case __1.Ast.NodeKind.GeneralizedIdentifierPairedAnyLiteral:
        case __1.Ast.NodeKind.GeneralizedIdentifierPairedExpression:
        case __1.Ast.NodeKind.IdentifierPairedExpression:
            return true;
        default:
            return false;
    }
}
exports.isTKeyValuePair = isTKeyValuePair;
function isTPairedConstant(node) {
    switch (node.kind) {
        case __1.Ast.NodeKind.AsNullablePrimitiveType:
        case __1.Ast.NodeKind.AsType:
        case __1.Ast.NodeKind.ErrorRaisingExpression:
        case __1.Ast.NodeKind.IsNullablePrimitiveType:
        case __1.Ast.NodeKind.NullablePrimitiveType:
        case __1.Ast.NodeKind.NullableType:
        case __1.Ast.NodeKind.OtherwiseExpression:
        case __1.Ast.NodeKind.TypePrimaryType:
            return true;
        default:
            return false;
    }
}
exports.isTPairedConstant = isTPairedConstant;
//# sourceMappingURL=typeGuards.js.map