"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isPairedWrapperConstantKinds = exports.isWrapperConstant = exports.isUnaryOperator = exports.isRelationalOperator = exports.isPrimitiveTypeConstant = exports.isMiscConstant = exports.isLogicalOperator = exports.isLanguageConstant = exports.isKeywordConstant = exports.isEqualityOperator = exports.isArithmeticOperator = exports.isTConstant = exports.isBinOpExpressionOperator = exports.binOpExpressionOperatorPrecedence = exports.binOpExpressionOperatorKindFrom = exports.relationalOperatorKindFrom = exports.logicalOperatorKindFrom = exports.equalityOperatorKindFrom = exports.arithmeticOperatorKindFrom = exports.unaryOperatorKindFrom = void 0;
const common_1 = require("../../common");
const _1 = require(".");
const token_1 = require("../token");
function unaryOperatorKindFrom(tokenKind) {
    switch (tokenKind) {
        case token_1.TokenKind.Plus:
            return _1.Constant.UnaryOperator.Positive;
        case token_1.TokenKind.Minus:
            return _1.Constant.UnaryOperator.Negative;
        case token_1.TokenKind.KeywordNot:
            return _1.Constant.UnaryOperator.Not;
        default:
            return undefined;
    }
}
exports.unaryOperatorKindFrom = unaryOperatorKindFrom;
function arithmeticOperatorKindFrom(tokenKind) {
    switch (tokenKind) {
        case token_1.TokenKind.Asterisk:
            return _1.Constant.ArithmeticOperator.Multiplication;
        case token_1.TokenKind.Division:
            return _1.Constant.ArithmeticOperator.Division;
        case token_1.TokenKind.Plus:
            return _1.Constant.ArithmeticOperator.Addition;
        case token_1.TokenKind.Minus:
            return _1.Constant.ArithmeticOperator.Subtraction;
        case token_1.TokenKind.Ampersand:
            return _1.Constant.ArithmeticOperator.And;
        default:
            return undefined;
    }
}
exports.arithmeticOperatorKindFrom = arithmeticOperatorKindFrom;
function equalityOperatorKindFrom(tokenKind) {
    switch (tokenKind) {
        case token_1.TokenKind.Equal:
            return _1.Constant.EqualityOperator.EqualTo;
        case token_1.TokenKind.NotEqual:
            return _1.Constant.EqualityOperator.NotEqualTo;
        default:
            return undefined;
    }
}
exports.equalityOperatorKindFrom = equalityOperatorKindFrom;
function logicalOperatorKindFrom(tokenKind) {
    switch (tokenKind) {
        case token_1.TokenKind.KeywordAnd:
            return _1.Constant.LogicalOperator.And;
        case token_1.TokenKind.KeywordOr:
            return _1.Constant.LogicalOperator.Or;
        default:
            return undefined;
    }
}
exports.logicalOperatorKindFrom = logicalOperatorKindFrom;
function relationalOperatorKindFrom(tokenKind) {
    switch (tokenKind) {
        case token_1.TokenKind.LessThan:
            return _1.Constant.RelationalOperator.LessThan;
        case token_1.TokenKind.LessThanEqualTo:
            return _1.Constant.RelationalOperator.LessThanEqualTo;
        case token_1.TokenKind.GreaterThan:
            return _1.Constant.RelationalOperator.GreaterThan;
        case token_1.TokenKind.GreaterThanEqualTo:
            return _1.Constant.RelationalOperator.GreaterThanEqualTo;
        default:
            return undefined;
    }
}
exports.relationalOperatorKindFrom = relationalOperatorKindFrom;
function binOpExpressionOperatorKindFrom(tokenKind) {
    switch (tokenKind) {
        // ArithmeticOperator
        case token_1.TokenKind.Asterisk:
            return _1.Constant.ArithmeticOperator.Multiplication;
        case token_1.TokenKind.Division:
            return _1.Constant.ArithmeticOperator.Division;
        case token_1.TokenKind.Plus:
            return _1.Constant.ArithmeticOperator.Addition;
        case token_1.TokenKind.Minus:
            return _1.Constant.ArithmeticOperator.Subtraction;
        case token_1.TokenKind.Ampersand:
            return _1.Constant.ArithmeticOperator.And;
        // EqualityOperator
        case token_1.TokenKind.Equal:
            return _1.Constant.EqualityOperator.EqualTo;
        case token_1.TokenKind.NotEqual:
            return _1.Constant.EqualityOperator.NotEqualTo;
        // LogicalOperator
        case token_1.TokenKind.KeywordAnd:
            return _1.Constant.LogicalOperator.And;
        case token_1.TokenKind.KeywordOr:
            return _1.Constant.LogicalOperator.Or;
        // RelationalOperator
        case token_1.TokenKind.LessThan:
            return _1.Constant.RelationalOperator.LessThan;
        case token_1.TokenKind.LessThanEqualTo:
            return _1.Constant.RelationalOperator.LessThanEqualTo;
        case token_1.TokenKind.GreaterThan:
            return _1.Constant.RelationalOperator.GreaterThan;
        case token_1.TokenKind.GreaterThanEqualTo:
            return _1.Constant.RelationalOperator.GreaterThanEqualTo;
        // Keyword operator
        case token_1.TokenKind.KeywordAs:
            return _1.Constant.KeywordConstant.As;
        case token_1.TokenKind.KeywordIs:
            return _1.Constant.KeywordConstant.Is;
        case token_1.TokenKind.KeywordMeta:
            return _1.Constant.KeywordConstant.Meta;
        case token_1.TokenKind.NullCoalescingOperator:
            return _1.Constant.MiscConstant.NullCoalescingOperator;
        default:
            return undefined;
    }
}
exports.binOpExpressionOperatorKindFrom = binOpExpressionOperatorKindFrom;
function binOpExpressionOperatorPrecedence(operator) {
    switch (operator) {
        case _1.Constant.KeywordConstant.Meta:
            return 110;
        case _1.Constant.ArithmeticOperator.Multiplication:
        case _1.Constant.ArithmeticOperator.Division:
            return 100;
        case _1.Constant.ArithmeticOperator.Addition:
        case _1.Constant.ArithmeticOperator.Subtraction:
        case _1.Constant.ArithmeticOperator.And:
            return 90;
        case _1.Constant.RelationalOperator.GreaterThan:
        case _1.Constant.RelationalOperator.GreaterThanEqualTo:
        case _1.Constant.RelationalOperator.LessThan:
        case _1.Constant.RelationalOperator.LessThanEqualTo:
            return 80;
        case _1.Constant.EqualityOperator.EqualTo:
        case _1.Constant.EqualityOperator.NotEqualTo:
            return 70;
        case _1.Constant.KeywordConstant.As:
            return 60;
        case _1.Constant.KeywordConstant.Is:
            return 50;
        case _1.Constant.LogicalOperator.And:
            return 40;
        case _1.Constant.LogicalOperator.Or:
            return 30;
        case _1.Constant.MiscConstant.NullCoalescingOperator:
            return 20;
        default:
            throw common_1.Assert.isNever(operator);
    }
}
exports.binOpExpressionOperatorPrecedence = binOpExpressionOperatorPrecedence;
function isBinOpExpressionOperator(text) {
    return (isArithmeticOperator(text) ||
        isEqualityOperator(text) ||
        isLogicalOperator(text) ||
        isRelationalOperator(text) ||
        text === _1.Constant.MiscConstant.NullCoalescingOperator ||
        text === _1.Constant.KeywordConstant.As ||
        text === _1.Constant.KeywordConstant.Is ||
        text === _1.Constant.KeywordConstant.Meta);
}
exports.isBinOpExpressionOperator = isBinOpExpressionOperator;
function isTConstant(text) {
    return (isArithmeticOperator(text) ||
        isEqualityOperator(text) ||
        isKeywordConstant(text) ||
        isLanguageConstant(text) ||
        isLogicalOperator(text) ||
        isMiscConstant(text) ||
        isPrimitiveTypeConstant(text) ||
        isRelationalOperator(text) ||
        isUnaryOperator(text) ||
        isWrapperConstant(text));
}
exports.isTConstant = isTConstant;
function isArithmeticOperator(text) {
    return _1.Constant.ArithmeticOperators.includes(text);
}
exports.isArithmeticOperator = isArithmeticOperator;
function isEqualityOperator(text) {
    return _1.Constant.EqualityOperators.includes(text);
}
exports.isEqualityOperator = isEqualityOperator;
function isKeywordConstant(text) {
    return _1.Constant.KeywordConstants.includes(text);
}
exports.isKeywordConstant = isKeywordConstant;
function isLanguageConstant(text) {
    return _1.Constant.LanguageConstants.includes(text);
}
exports.isLanguageConstant = isLanguageConstant;
function isLogicalOperator(text) {
    return _1.Constant.LogicalOperators.includes(text);
}
exports.isLogicalOperator = isLogicalOperator;
function isMiscConstant(text) {
    return _1.Constant.MiscConstants.includes(text);
}
exports.isMiscConstant = isMiscConstant;
function isPrimitiveTypeConstant(text) {
    return _1.Constant.PrimitiveTypeConstants.includes(text);
}
exports.isPrimitiveTypeConstant = isPrimitiveTypeConstant;
function isRelationalOperator(text) {
    return _1.Constant.RelationalOperators.includes(text);
}
exports.isRelationalOperator = isRelationalOperator;
function isUnaryOperator(text) {
    return _1.Constant.UnaryOperators.includes(text);
}
exports.isUnaryOperator = isUnaryOperator;
function isWrapperConstant(text) {
    return _1.Constant.WrapperConstants.includes(text);
}
exports.isWrapperConstant = isWrapperConstant;
function isPairedWrapperConstantKinds(left, right) {
    return ((left === _1.Constant.WrapperConstant.LeftBrace && right === _1.Constant.WrapperConstant.RightBrace) ||
        (left === _1.Constant.WrapperConstant.LeftBracket && right === _1.Constant.WrapperConstant.RightBracket) ||
        (left === _1.Constant.WrapperConstant.LeftParenthesis && right === _1.Constant.WrapperConstant.RightParenthesis));
}
exports.isPairedWrapperConstantKinds = isPairedWrapperConstantKinds;
//# sourceMappingURL=constantUtils.js.map