import { OrderedMap } from "../../common";
export declare type TPowerQueryType = TPrimitiveType | TExtendedType;
export declare type TExtendedType = AnyUnion | DefinedFunction | DefinedList | DefinedListType | DefinedRecord | DefinedTable | FunctionType | ListType | LogicalLiteral | NumberLiteral | PrimaryPrimitiveType | RecordType | TableType | TableTypePrimaryExpression | TextLiteral;
export declare type TExtendedTypeKind = TypeKind.Any | TypeKind.Function | TypeKind.List | TypeKind.Logical | TypeKind.Number | TypeKind.Record | TypeKind.Table | TypeKind.Text | TypeKind.Type;
export declare type TLiteral = LogicalLiteral | NumberLiteral | TextLiteral;
export declare type TLiteralKind = ExtendedTypeKind.LogicalLiteral | ExtendedTypeKind.NumberLiteral | ExtendedTypeKind.TextLiteral;
export declare type TAny = Any | AnyUnion;
export declare type TList = List | DefinedList;
export declare type TLogical = Logical | LogicalLiteral;
export declare type TFunction = Function | DefinedFunction;
export declare type TNumber = Number | NumberLiteral;
export declare type TRecord = Record | DefinedRecord;
export declare type TTable = Table | DefinedTable;
export declare type TText = Text | TextLiteral;
export declare type TType = Type | DefinedListType | FunctionType | ListType | PrimaryPrimitiveType | RecordType | TableType | TableTypePrimaryExpression;
export declare type Action = IPrimitiveType<TypeKind.Action>;
export declare type Any = IPrimitiveType<TypeKind.Any>;
export declare type AnyNonNull = IPrimitiveType<TypeKind.AnyNonNull>;
export declare type Binary = IPrimitiveType<TypeKind.Binary>;
export declare type Date = IPrimitiveType<TypeKind.Date>;
export declare type DateTime = IPrimitiveType<TypeKind.DateTime>;
export declare type DateTimeZone = IPrimitiveType<TypeKind.DateTimeZone>;
export declare type Duration = IPrimitiveType<TypeKind.Duration>;
export declare type Function = IPrimitiveType<TypeKind.Function>;
export declare type List = IPrimitiveType<TypeKind.List>;
export declare type Logical = IPrimitiveType<TypeKind.Logical>;
export declare type None = IPrimitiveType<TypeKind.None>;
export declare type NotApplicable = IPrimitiveType<TypeKind.NotApplicable>;
export declare type Null = IPrimitiveType<TypeKind.Null>;
export declare type Number = IPrimitiveType<TypeKind.Number>;
export declare type Record = IPrimitiveType<TypeKind.Record>;
export declare type Table = IPrimitiveType<TypeKind.Table>;
export declare type Text = IPrimitiveType<TypeKind.Text>;
export declare type Time = IPrimitiveType<TypeKind.Time>;
export declare type Type = IPrimitiveType<TypeKind.Type>;
export declare type Unknown = IPrimitiveType<TypeKind.Unknown>;
export declare type TPrimitiveType = Action | Any | AnyNonNull | Binary | Date | DateTime | DateTimeZone | Duration | Function | List | Logical | None | NotApplicable | Null | Number | Record | Table | Text | Time | Type | Unknown;
export declare type TFieldSpecificationList = FieldSpecificationList<TFields>;
export declare type TFields = OrderedFields | UnorderedFields;
export declare type OrderedFields = OrderedMap<string, TPowerQueryType>;
export declare type UnorderedFields = Map<string, TPowerQueryType>;
export declare enum TypeKind {
    Any = "Any",
    AnyNonNull = "AnyNonNull",
    Binary = "Binary",
    Date = "Date",
    DateTime = "DateTime",
    DateTimeZone = "DateTimeZone",
    Duration = "Duration",
    Function = "Function",
    List = "List",
    Logical = "Logical",
    None = "None",
    Null = "Null",
    Number = "Number",
    Record = "Record",
    Table = "Table",
    Text = "Text",
    Type = "Type",
    Action = "Action",
    Time = "Time",
    NotApplicable = "NotApplicable",
    Unknown = "Unknown"
}
export declare const TypeKinds: ReadonlyArray<TypeKind>;
export declare enum ExtendedTypeKind {
    AnyUnion = "AnyUnion",
    DefinedFunction = "DefinedFunction",
    DefinedList = "DefinedList",
    DefinedListType = "DefinedListType",
    DefinedRecord = "DefinedRecord",
    DefinedTable = "DefinedTable",
    FunctionType = "FunctionType",
    ListType = "ListType",
    LogicalLiteral = "LogicalLiteral",
    NumberLiteral = "NumberLiteral",
    PrimaryPrimitiveType = "PrimaryPrimitiveType",
    RecordType = "RecordType",
    TextLiteral = "TextLiteral",
    TableType = "TableType",
    TableTypePrimaryExpression = "TableTypePrimaryExpression"
}
export interface IType<T extends TypeKind = TypeKind> {
    readonly kind: T;
    readonly extendedKind: ExtendedTypeKind | undefined;
    readonly isNullable: boolean;
}
export interface IExtendedType extends IType {
    readonly kind: TExtendedTypeKind;
    readonly extendedKind: ExtendedTypeKind;
}
export interface IPrimitiveLiteral<T> extends IExtendedType {
    readonly literal: string;
    readonly normalizedLiteral: T;
}
export interface IPrimitiveType<T extends TypeKind = TypeKind> extends IType<T> {
    readonly extendedKind: undefined;
}
export interface FieldSpecificationList<T extends TFields> {
    readonly fields: T;
    readonly isOpen: boolean;
}
export interface FunctionSignature {
    readonly parameters: ReadonlyArray<FunctionParameter>;
    readonly returnType: TPowerQueryType;
}
export interface SimplifiedNullablePrimitiveType {
    readonly typeKind: TypeKind;
    readonly isNullable: boolean;
}
export interface FunctionParameter {
    readonly nameLiteral: string;
    readonly isOptional: boolean;
    readonly isNullable: boolean;
    readonly type: TypeKind | undefined;
}
export interface AnyUnion extends IExtendedType {
    readonly kind: TypeKind.Any;
    readonly extendedKind: ExtendedTypeKind.AnyUnion;
    readonly unionedTypePairs: ReadonlyArray<TPowerQueryType>;
}
export declare type DefinedFunction = IExtendedType & FunctionSignature & {
    readonly kind: TypeKind.Function;
    readonly extendedKind: ExtendedTypeKind.DefinedFunction;
};
export interface DefinedList extends IExtendedType {
    readonly kind: TypeKind.List;
    readonly extendedKind: ExtendedTypeKind.DefinedList;
    readonly elements: ReadonlyArray<TPowerQueryType>;
}
export interface DefinedListType extends IExtendedType {
    readonly kind: TypeKind.Type;
    readonly extendedKind: ExtendedTypeKind.DefinedListType;
    readonly itemTypes: ReadonlyArray<TPowerQueryType>;
}
export declare type DefinedRecord = IExtendedType & FieldSpecificationList<UnorderedFields> & {
    readonly kind: TypeKind.Record;
    readonly extendedKind: ExtendedTypeKind.DefinedRecord;
};
export declare type DefinedTable = IExtendedType & FieldSpecificationList<OrderedFields> & {
    readonly kind: TypeKind.Table;
    readonly extendedKind: ExtendedTypeKind.DefinedTable;
};
export declare type FunctionType = IExtendedType & FunctionSignature & {
    readonly kind: TypeKind.Type;
    readonly extendedKind: ExtendedTypeKind.FunctionType;
};
export interface ListType extends IExtendedType {
    readonly kind: TypeKind.Type;
    readonly extendedKind: ExtendedTypeKind.ListType;
    readonly itemType: TPowerQueryType;
}
export interface LogicalLiteral extends IPrimitiveLiteral<boolean> {
    readonly kind: TypeKind.Logical;
    readonly extendedKind: ExtendedTypeKind.LogicalLiteral;
}
export interface NumberLiteral extends IPrimitiveLiteral<number> {
    readonly kind: TypeKind.Number;
    readonly extendedKind: ExtendedTypeKind.NumberLiteral;
}
export interface PrimaryPrimitiveType extends IExtendedType {
    readonly kind: TypeKind.Type;
    readonly extendedKind: ExtendedTypeKind.PrimaryPrimitiveType;
    readonly primitiveType: TPrimitiveType;
}
export declare type RecordType = IExtendedType & FieldSpecificationList<UnorderedFields> & {
    readonly kind: TypeKind.Type;
    readonly extendedKind: ExtendedTypeKind.RecordType;
};
export declare type TableType = IExtendedType & FieldSpecificationList<UnorderedFields> & {
    readonly kind: TypeKind.Type;
    readonly extendedKind: ExtendedTypeKind.TableType;
};
export interface TableTypePrimaryExpression extends IExtendedType {
    readonly kind: TypeKind.Type;
    readonly extendedKind: ExtendedTypeKind.TableTypePrimaryExpression;
    readonly primaryExpression: TPowerQueryType;
}
export interface TextLiteral extends IPrimitiveLiteral<string> {
    readonly kind: TypeKind.Text;
    readonly extendedKind: ExtendedTypeKind.TextLiteral;
}
export declare const ActionInstance: IPrimitiveType<TypeKind.Action>;
export declare const AnyInstance: IPrimitiveType<TypeKind.Any>;
export declare const AnyNonNullInstance: IPrimitiveType<TypeKind.AnyNonNull>;
export declare const BinaryInstance: IPrimitiveType<TypeKind.Binary>;
export declare const DateInstance: IPrimitiveType<TypeKind.Date>;
export declare const DateTimeInstance: IPrimitiveType<TypeKind.DateTime>;
export declare const DateTimeZoneInstance: IPrimitiveType<TypeKind.DateTimeZone>;
export declare const DurationInstance: IPrimitiveType<TypeKind.Duration>;
export declare const FunctionInstance: IPrimitiveType<TypeKind.Function>;
export declare const ListInstance: IPrimitiveType<TypeKind.List>;
export declare const LogicalInstance: IPrimitiveType<TypeKind.Logical>;
export declare const NoneInstance: IPrimitiveType<TypeKind.None>;
export declare const NotApplicableInstance: IPrimitiveType<TypeKind.NotApplicable>;
export declare const NullInstance: IPrimitiveType<TypeKind.Null>;
export declare const NumberInstance: IPrimitiveType<TypeKind.Number>;
export declare const RecordInstance: IPrimitiveType<TypeKind.Record>;
export declare const TableInstance: IPrimitiveType<TypeKind.Table>;
export declare const TextInstance: IPrimitiveType<TypeKind.Text>;
export declare const TimeInstance: IPrimitiveType<TypeKind.Time>;
export declare const TypePrimitiveInstance: IPrimitiveType<TypeKind.Type>;
export declare const UnknownInstance: IPrimitiveType<TypeKind.Unknown>;
export declare const NullableActionInstance: IPrimitiveType<TypeKind.Action>;
export declare const NullableAnyInstance: IPrimitiveType<TypeKind.Any>;
export declare const NullableBinaryInstance: IPrimitiveType<TypeKind.Binary>;
export declare const NullableDateInstance: IPrimitiveType<TypeKind.Date>;
export declare const NullableDateTimeInstance: IPrimitiveType<TypeKind.DateTime>;
export declare const NullableDateTimeZoneInstance: IPrimitiveType<TypeKind.DateTimeZone>;
export declare const NullableDurationInstance: IPrimitiveType<TypeKind.Duration>;
export declare const NullableFunctionInstance: IPrimitiveType<TypeKind.Function>;
export declare const NullableListInstance: IPrimitiveType<TypeKind.List>;
export declare const NullableLogicalInstance: IPrimitiveType<TypeKind.Logical>;
export declare const NullableNoneInstance: IPrimitiveType<TypeKind.None>;
export declare const NullableNotApplicableInstance: IPrimitiveType<TypeKind.NotApplicable>;
export declare const NullableNumberInstance: IPrimitiveType<TypeKind.Number>;
export declare const NullableRecordInstance: IPrimitiveType<TypeKind.Record>;
export declare const NullableTableInstance: IPrimitiveType<TypeKind.Table>;
export declare const NullableTextInstance: IPrimitiveType<TypeKind.Text>;
export declare const NullableTimeInstance: IPrimitiveType<TypeKind.Time>;
export declare const NullableTypeInstance: IPrimitiveType<TypeKind.Type>;
export declare const NullableUnknownInstance: IPrimitiveType<TypeKind.Unknown>;
export declare const FalseInstance: LogicalLiteral;
export declare const TrueInstance: LogicalLiteral;
export declare const NullableFalseInstance: LogicalLiteral;
export declare const NullableTrueInstance: LogicalLiteral;
export declare const PrimitiveInstance: AnyUnion;
export declare const NullablePrimitiveInstance: AnyUnion;
export declare const ExpressionInstance: AnyUnion;
export declare const LiteralExpressionInstance: AnyUnion;
export declare const PrimaryExpressionInstance: AnyUnion;
export declare const PrimaryTypeInstance: AnyUnion;
export declare const TypeProductionInstance: AnyUnion;
export declare const TypeExpressionInstance: AnyUnion;
export declare const AnyLiteralInstance: AnyUnion;
