"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExpressionInstance = exports.NullablePrimitiveInstance = exports.PrimitiveInstance = exports.NullableTrueInstance = exports.NullableFalseInstance = exports.TrueInstance = exports.FalseInstance = exports.NullableUnknownInstance = exports.NullableTypeInstance = exports.NullableTimeInstance = exports.NullableTextInstance = exports.NullableTableInstance = exports.NullableRecordInstance = exports.NullableNumberInstance = exports.NullableNotApplicableInstance = exports.NullableNoneInstance = exports.NullableLogicalInstance = exports.NullableListInstance = exports.NullableFunctionInstance = exports.NullableDurationInstance = exports.NullableDateTimeZoneInstance = exports.NullableDateTimeInstance = exports.NullableDateInstance = exports.NullableBinaryInstance = exports.NullableAnyInstance = exports.NullableActionInstance = exports.UnknownInstance = exports.TypePrimitiveInstance = exports.TimeInstance = exports.TextInstance = exports.TableInstance = exports.RecordInstance = exports.NumberInstance = exports.NullInstance = exports.NotApplicableInstance = exports.NoneInstance = exports.LogicalInstance = exports.ListInstance = exports.FunctionInstance = exports.DurationInstance = exports.DateTimeZoneInstance = exports.DateTimeInstance = exports.DateInstance = exports.BinaryInstance = exports.AnyNonNullInstance = exports.AnyInstance = exports.ActionInstance = exports.ExtendedTypeKind = exports.TypeKinds = exports.TypeKind = void 0;
exports.AnyLiteralInstance = exports.TypeExpressionInstance = exports.TypeProductionInstance = exports.PrimaryTypeInstance = exports.PrimaryExpressionInstance = exports.LiteralExpressionInstance = void 0;
var TypeKind;
(function (TypeKind) {
    TypeKind["Any"] = "Any";
    TypeKind["AnyNonNull"] = "AnyNonNull";
    TypeKind["Binary"] = "Binary";
    TypeKind["Date"] = "Date";
    TypeKind["DateTime"] = "DateTime";
    TypeKind["DateTimeZone"] = "DateTimeZone";
    TypeKind["Duration"] = "Duration";
    TypeKind["Function"] = "Function";
    TypeKind["List"] = "List";
    TypeKind["Logical"] = "Logical";
    TypeKind["None"] = "None";
    TypeKind["Null"] = "Null";
    TypeKind["Number"] = "Number";
    TypeKind["Record"] = "Record";
    TypeKind["Table"] = "Table";
    TypeKind["Text"] = "Text";
    TypeKind["Type"] = "Type";
    // Types that are not defined in the standard.
    TypeKind["Action"] = "Action";
    TypeKind["Time"] = "Time";
    // Some NodeKinds are non-typeable, such as ArrayWrapper.
    // There can be nodes which are typable but contain non-typable children, such as RecordExpressions.
    TypeKind["NotApplicable"] = "NotApplicable";
    // Something that can't be typed due to a lack of information.
    // Eg. '[', a RecordExpression where the user hasn't entered any fields for.
    TypeKind["Unknown"] = "Unknown";
})(TypeKind = exports.TypeKind || (exports.TypeKind = {}));
exports.TypeKinds = [
    TypeKind.Action,
    TypeKind.Any,
    TypeKind.AnyNonNull,
    TypeKind.Binary,
    TypeKind.Date,
    TypeKind.DateTime,
    TypeKind.DateTimeZone,
    TypeKind.Duration,
    TypeKind.Function,
    TypeKind.List,
    TypeKind.Logical,
    TypeKind.Null,
    TypeKind.Number,
    TypeKind.Record,
    TypeKind.Table,
    TypeKind.Text,
    TypeKind.Time,
    TypeKind.Type,
];
var ExtendedTypeKind;
(function (ExtendedTypeKind) {
    // In Power Query if you want a union type, such as when a function returns either a `text` OR a `number`,
    // then you're only option is to give it the 'any' type.
    // This is a narrower definition which restricts that typing to the union `text | number`.
    ExtendedTypeKind["AnyUnion"] = "AnyUnion";
    // A function with known paramaters and a known return type.
    ExtendedTypeKind["DefinedFunction"] = "DefinedFunction";
    // A list of known size and a typing for each element in the list.
    // Eg. `{1, "foo"}
    ExtendedTypeKind["DefinedList"] = "DefinedList";
    // A narrower typing for ListType, used to validate a DefinedList.
    ExtendedTypeKind["DefinedListType"] = "DefinedListType";
    // A list of known fields and typing for each field.
    // An open record: `[a=1, b=2, ...]`
    // A closed record: `[a=1, b=2]`
    ExtendedTypeKind["DefinedRecord"] = "DefinedRecord";
    // See details on DefinedRecord. They are functionally the same.
    ExtendedTypeKind["DefinedTable"] = "DefinedTable";
    // `type function (x as number, optional y) => number`
    ExtendedTypeKind["FunctionType"] = "FunctionType";
    // `type list { number }`
    ExtendedTypeKind["ListType"] = "ListType";
    // true
    ExtendedTypeKind["LogicalLiteral"] = "LogicalLiteral";
    // `1`
    ExtendedTypeKind["NumberLiteral"] = "NumberLiteral";
    // `type number`
    ExtendedTypeKind["PrimaryPrimitiveType"] = "PrimaryPrimitiveType";
    // `type record [ a, b = number, ...]`
    ExtendedTypeKind["RecordType"] = "RecordType";
    // '"foo"`
    ExtendedTypeKind["TextLiteral"] = "TextLiteral";
    // `type table [a, b = text]
    ExtendedTypeKind["TableType"] = "TableType";
    // The docs say `table-type` only accepts a `field-specification-list` for `row-type`,
    // but the parser also accepts primary-expression. This handles that edge case.
    ExtendedTypeKind["TableTypePrimaryExpression"] = "TableTypePrimaryExpression";
})(ExtendedTypeKind = exports.ExtendedTypeKind || (exports.ExtendedTypeKind = {}));
// -------------------------------------------------------
// ---------- Non-nullable primitive singletons ----------
// -------------------------------------------------------
exports.ActionInstance = createPrimitiveType(TypeKind.Action, false);
exports.AnyInstance = createPrimitiveType(TypeKind.Any, false);
exports.AnyNonNullInstance = createPrimitiveType(TypeKind.AnyNonNull, false);
exports.BinaryInstance = createPrimitiveType(TypeKind.Binary, false);
exports.DateInstance = createPrimitiveType(TypeKind.Date, false);
exports.DateTimeInstance = createPrimitiveType(TypeKind.DateTime, false);
exports.DateTimeZoneInstance = createPrimitiveType(TypeKind.DateTimeZone, false);
exports.DurationInstance = createPrimitiveType(TypeKind.Duration, false);
exports.FunctionInstance = createPrimitiveType(TypeKind.Function, false);
exports.ListInstance = createPrimitiveType(TypeKind.List, false);
exports.LogicalInstance = createPrimitiveType(TypeKind.Logical, false);
exports.NoneInstance = createPrimitiveType(TypeKind.None, false);
exports.NotApplicableInstance = createPrimitiveType(TypeKind.NotApplicable, false);
exports.NullInstance = createPrimitiveType(TypeKind.Null, true);
exports.NumberInstance = createPrimitiveType(TypeKind.Number, false);
exports.RecordInstance = createPrimitiveType(TypeKind.Record, false);
exports.TableInstance = createPrimitiveType(TypeKind.Table, false);
exports.TextInstance = createPrimitiveType(TypeKind.Text, false);
exports.TimeInstance = createPrimitiveType(TypeKind.Time, false);
exports.TypePrimitiveInstance = createPrimitiveType(TypeKind.Type, false);
exports.UnknownInstance = createPrimitiveType(TypeKind.Unknown, false);
// ---------------------------------------------------
// ---------- Nullable primitive singletons ----------
// ---------------------------------------------------
exports.NullableActionInstance = createPrimitiveType(TypeKind.Action, true);
exports.NullableAnyInstance = createPrimitiveType(TypeKind.Any, true);
exports.NullableBinaryInstance = createPrimitiveType(TypeKind.Binary, true);
exports.NullableDateInstance = createPrimitiveType(TypeKind.Date, true);
exports.NullableDateTimeInstance = createPrimitiveType(TypeKind.DateTime, true);
exports.NullableDateTimeZoneInstance = createPrimitiveType(TypeKind.DateTimeZone, true);
exports.NullableDurationInstance = createPrimitiveType(TypeKind.Duration, true);
exports.NullableFunctionInstance = createPrimitiveType(TypeKind.Function, true);
exports.NullableListInstance = createPrimitiveType(TypeKind.List, true);
exports.NullableLogicalInstance = createPrimitiveType(TypeKind.Logical, true);
exports.NullableNoneInstance = createPrimitiveType(TypeKind.None, true);
exports.NullableNotApplicableInstance = createPrimitiveType(TypeKind.NotApplicable, true);
exports.NullableNumberInstance = createPrimitiveType(TypeKind.Number, true);
exports.NullableRecordInstance = createPrimitiveType(TypeKind.Record, true);
exports.NullableTableInstance = createPrimitiveType(TypeKind.Table, true);
exports.NullableTextInstance = createPrimitiveType(TypeKind.Text, true);
exports.NullableTimeInstance = createPrimitiveType(TypeKind.Time, true);
exports.NullableTypeInstance = createPrimitiveType(TypeKind.Type, true);
exports.NullableUnknownInstance = createPrimitiveType(TypeKind.Unknown, true);
// ----------------------------------------------
// ---------- Non-primitive singletons ----------
// ----------------------------------------------
exports.FalseInstance = createLogicalLiteral(false, false);
exports.TrueInstance = createLogicalLiteral(false, true);
exports.NullableFalseInstance = createLogicalLiteral(true, false);
exports.NullableTrueInstance = createLogicalLiteral(true, true);
exports.PrimitiveInstance = {
    kind: TypeKind.Any,
    extendedKind: ExtendedTypeKind.AnyUnion,
    isNullable: false,
    unionedTypePairs: [
        exports.ActionInstance,
        exports.AnyInstance,
        exports.AnyNonNullInstance,
        exports.BinaryInstance,
        exports.DateInstance,
        exports.DateTimeInstance,
        exports.DateTimeZoneInstance,
        exports.DurationInstance,
        exports.FunctionInstance,
        exports.ListInstance,
        exports.LogicalInstance,
        exports.NoneInstance,
        exports.NotApplicableInstance,
        exports.NullInstance,
        exports.NumberInstance,
        exports.RecordInstance,
        exports.TableInstance,
        exports.TextInstance,
        exports.TimeInstance,
        exports.TypePrimitiveInstance,
    ],
};
exports.NullablePrimitiveInstance = {
    kind: TypeKind.Any,
    extendedKind: ExtendedTypeKind.AnyUnion,
    isNullable: true,
    unionedTypePairs: [
        exports.NullableActionInstance,
        exports.NullableAnyInstance,
        exports.NullableBinaryInstance,
        exports.NullableDateInstance,
        exports.NullableDateTimeInstance,
        exports.NullableDateTimeZoneInstance,
        exports.NullableDurationInstance,
        exports.NullableFunctionInstance,
        exports.NullableListInstance,
        exports.NullableLogicalInstance,
        exports.NullableNoneInstance,
        exports.NullableNotApplicableInstance,
        exports.NullableNumberInstance,
        exports.NullableRecordInstance,
        exports.NullableTableInstance,
        exports.NullableTextInstance,
        exports.NullableTimeInstance,
        exports.NullableTypeInstance,
    ],
};
exports.ExpressionInstance = {
    kind: TypeKind.Any,
    extendedKind: ExtendedTypeKind.AnyUnion,
    isNullable: exports.PrimitiveInstance.isNullable || exports.NullablePrimitiveInstance.isNullable,
    unionedTypePairs: [...exports.PrimitiveInstance.unionedTypePairs, ...exports.NullablePrimitiveInstance.unionedTypePairs],
};
exports.LiteralExpressionInstance = {
    kind: TypeKind.Any,
    extendedKind: ExtendedTypeKind.AnyUnion,
    isNullable: exports.LogicalInstance.isNullable || exports.NumberInstance.isNullable || exports.TextInstance.isNullable || exports.NullInstance.isNullable,
    unionedTypePairs: [exports.LogicalInstance, exports.NumberInstance, exports.TextInstance, exports.NullInstance],
};
exports.PrimaryExpressionInstance = {
    kind: TypeKind.Any,
    extendedKind: ExtendedTypeKind.AnyUnion,
    isNullable: exports.LiteralExpressionInstance.isNullable || exports.ListInstance.isNullable || exports.RecordInstance.isNullable,
    unionedTypePairs: [...exports.LiteralExpressionInstance.unionedTypePairs, exports.ListInstance, exports.RecordInstance],
};
exports.PrimaryTypeInstance = {
    kind: TypeKind.Any,
    extendedKind: ExtendedTypeKind.AnyUnion,
    isNullable: true,
    unionedTypePairs: [...exports.PrimitiveInstance.unionedTypePairs, ...exports.NullablePrimitiveInstance.unionedTypePairs],
};
exports.TypeProductionInstance = {
    kind: TypeKind.Any,
    extendedKind: ExtendedTypeKind.AnyUnion,
    isNullable: exports.ExpressionInstance.isNullable || exports.PrimaryTypeInstance.isNullable,
    unionedTypePairs: [...exports.ExpressionInstance.unionedTypePairs, ...exports.PrimaryTypeInstance.unionedTypePairs],
};
exports.TypeExpressionInstance = {
    kind: TypeKind.Any,
    extendedKind: ExtendedTypeKind.AnyUnion,
    isNullable: exports.PrimaryExpressionInstance.isNullable || exports.PrimaryTypeInstance.isNullable,
    unionedTypePairs: [exports.PrimaryExpressionInstance, exports.PrimaryTypeInstance],
};
exports.AnyLiteralInstance = {
    kind: TypeKind.Any,
    extendedKind: ExtendedTypeKind.AnyUnion,
    isNullable: exports.RecordInstance.isNullable ||
        exports.ListInstance.isNullable ||
        exports.LogicalInstance.isNullable ||
        exports.NumberInstance.isNullable ||
        exports.TextInstance.isNullable ||
        exports.NullInstance.isNullable,
    unionedTypePairs: [exports.RecordInstance, exports.ListInstance, exports.LogicalInstance, exports.NumberInstance, exports.TextInstance, exports.NullInstance],
};
// --------------------------------------
// ---------- Helper functions ----------
// --------------------------------------
// Creates IPrimitiveType<T> singleton instances.
function createPrimitiveType(typeKind, isNullable) {
    return {
        kind: typeKind,
        extendedKind: undefined,
        isNullable,
    };
}
function createLogicalLiteral(isNullable, normalizedLiteral) {
    return {
        isNullable,
        kind: TypeKind.Logical,
        extendedKind: ExtendedTypeKind.LogicalLiteral,
        literal: normalizedLiteral ? "true" : "false",
        normalizedLiteral,
    };
}
//# sourceMappingURL=type.js.map