import { ImmutableSet } from "../../../common";
import { TraceManager } from "../../../common/trace";
import { Type } from "..";
export declare type TCategory = TExtendedCategory | NonExtendedCategory;
export declare type TExtendedCategory = AnyCategory | FunctionCategory | ListCategory | LogicalCategory | NumberCategory | RecordCategory | TableCategory | TextCategory | TypeCategory;
export declare type NonExtendedCategory = ActionCategory | AnyNonNullCategory | BinaryCategory | DateCategory | DateTimeCategory | DateTimeZoneCategory | DurationCategory | NoneCategory | NotApplicableCategory | NullCategory | TimeCategory | UnknownCategory;
export interface CategorizedPowerQueryTypes {
    readonly actions: ActionCategory | undefined;
    readonly anyNonNulls: AnyNonNullCategory | undefined;
    readonly anys: AnyCategory | undefined;
    readonly binaries: BinaryCategory | undefined;
    readonly dates: DateCategory | undefined;
    readonly dateTimes: DateTimeCategory | undefined;
    readonly dateTimeZones: DateTimeZoneCategory | undefined;
    readonly durations: DurationCategory | undefined;
    readonly functions: FunctionCategory | undefined;
    readonly lists: ListCategory | undefined;
    readonly logicals: LogicalCategory | undefined;
    readonly nones: NoneCategory | undefined;
    readonly notApplicables: NotApplicableCategory | undefined;
    readonly numbers: NumberCategory | undefined;
    readonly nulls: NullCategory | undefined;
    readonly records: RecordCategory | undefined;
    readonly tables: TableCategory | undefined;
    readonly texts: TextCategory | undefined;
    readonly times: TimeCategory | undefined;
    readonly types: TypeCategory | undefined;
    readonly unknowns: UnknownCategory | undefined;
}
export declare type ActionCategory = ITypeKindCategory<Type.Action>;
export declare type AnyNonNullCategory = ITypeKindCategory<Type.AnyNonNull>;
export declare type BinaryCategory = ITypeKindCategory<Type.Binary>;
export declare type DateCategory = ITypeKindCategory<Type.Date>;
export declare type DateTimeCategory = ITypeKindCategory<Type.DateTime>;
export declare type DateTimeZoneCategory = ITypeKindCategory<Type.DateTimeZone>;
export declare type DurationCategory = ITypeKindCategory<Type.Duration>;
export declare type NoneCategory = ITypeKindCategory<Type.None>;
export declare type NotApplicableCategory = ITypeKindCategory<Type.NotApplicable>;
export declare type NullCategory = ITypeKindCategory<Type.Null>;
export declare type TimeCategory = ITypeKindCategory<Type.Time>;
export declare type UnknownCategory = ITypeKindCategory<Type.Unknown>;
export interface AnyCategory extends ITypeKindCategory<Type.Any> {
    readonly anyUnions: ImmutableSet<Type.AnyUnion>;
    readonly flattenedAnyUnions: ImmutableSet<Type.TPowerQueryType>;
}
export interface FunctionCategory extends ITypeKindCategory<Type.Function> {
    readonly definedFunctions: ImmutableSet<Type.DefinedFunction>;
}
export interface ListCategory extends ITypeKindCategory<Type.List> {
    readonly definedLists: ImmutableSet<Type.DefinedList>;
}
export interface LogicalCategory extends ITypeKindCategory<Type.Logical> {
    readonly hasFalsyNonNullableLiteral: boolean;
    readonly hasFalsyNullableLiteral: boolean;
    readonly hasTruthyNonNullableLiteral: boolean;
    readonly hasTruthyNullableLiteral: boolean;
}
export interface NumberCategory extends ITypeKindCategory<Type.Number> {
    readonly literals: ImmutableSet<Type.NumberLiteral>;
}
export interface RecordCategory extends ITypeKindCategory<Type.Record> {
    readonly definedRecords: ImmutableSet<Type.DefinedRecord>;
}
export interface TableCategory extends ITypeKindCategory<Type.Table> {
    readonly definedTables: ImmutableSet<Type.DefinedTable>;
}
export interface TextCategory extends ITypeKindCategory<Type.Text> {
    readonly literals: ImmutableSet<Type.TextLiteral>;
}
export interface TypeCategory extends ITypeKindCategory<Type.Type> {
    readonly definedListTypes: ImmutableSet<Type.DefinedListType>;
    readonly functionTypes: ImmutableSet<Type.FunctionType>;
    readonly listTypes: ImmutableSet<Type.ListType>;
    readonly primaryPrimitiveTypes: ImmutableSet<Type.PrimaryPrimitiveType>;
    readonly recordTypes: ImmutableSet<Type.RecordType>;
    readonly tableTypes: ImmutableSet<Type.TableType>;
    readonly tablePrimaryExpressionTypes: ImmutableSet<Type.TableTypePrimaryExpression>;
}
export declare function categorize(types: ReadonlyArray<Type.TPowerQueryType>, traceManager: TraceManager, correlationId: number | undefined): CategorizedPowerQueryTypes;
interface ITypeKindCategory<T extends Type.TPowerQueryType> {
    readonly kind: T["kind"];
    readonly primitives: ImmutableSet<T>;
}
export {};
