"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.textLiteral = exports.tableTypePrimary = exports.tableType = exports.recordType = exports.primaryPrimitiveType = exports.numberLiteral = exports.listType = exports.logicalLiteral = exports.functionType = exports.definedTable = exports.definedRecord = exports.definedListType = exports.definedList = exports.definedFunction = exports.anyUnion = exports.primitiveType = void 0;
const common_1 = require("../../../common");
const primitive_1 = require("./primitive");
const simplify_1 = require("./simplify");
const __1 = require("..");
const typeTraceConstant_1 = require("./typeTraceConstant");
function primitiveType(isNullable, typeKind) {
    const key = (0, primitive_1.primitiveTypeMapKey)(isNullable, typeKind);
    return common_1.Assert.asDefined(primitive_1.PrimitiveTypeConstantMap.get(key), `unknown key for PrimitiveTypeConstantMap`, {
        typeKind,
        isNullable,
    });
}
exports.primitiveType = primitiveType;
// If the given types can be simplified/deduped down to a single type then that is returned instead.
// Otherwise returns an instance of `Type.AnyUnion`.
function anyUnion(unionedTypePairs, traceManager, correlationId) {
    const trace = traceManager.entry(typeTraceConstant_1.TypeUtilsTraceConstant.AnyUnion, anyUnion.name, correlationId);
    const simplified = (0, simplify_1.simplify)(unionedTypePairs, traceManager, trace.id);
    if (simplified.length === 1) {
        trace.exit();
        return simplified[0];
    }
    const result = {
        kind: __1.Type.TypeKind.Any,
        extendedKind: __1.Type.ExtendedTypeKind.AnyUnion,
        isNullable: unionedTypePairs.find((ttype) => ttype.isNullable) !== undefined,
        unionedTypePairs: simplified,
    };
    trace.exit();
    return result;
}
exports.anyUnion = anyUnion;
function definedFunction(isNullable, parameters, returnType) {
    return {
        kind: __1.Type.TypeKind.Function,
        extendedKind: __1.Type.ExtendedTypeKind.DefinedFunction,
        isNullable,
        parameters,
        returnType,
    };
}
exports.definedFunction = definedFunction;
function definedList(isNullable, elements) {
    return {
        kind: __1.Type.TypeKind.List,
        extendedKind: __1.Type.ExtendedTypeKind.DefinedList,
        isNullable,
        elements,
    };
}
exports.definedList = definedList;
function definedListType(isNullable, itemTypes) {
    return {
        kind: __1.Type.TypeKind.Type,
        extendedKind: __1.Type.ExtendedTypeKind.DefinedListType,
        isNullable,
        itemTypes,
    };
}
exports.definedListType = definedListType;
function definedRecord(isNullable, fields, isOpen) {
    return {
        kind: __1.Type.TypeKind.Record,
        extendedKind: __1.Type.ExtendedTypeKind.DefinedRecord,
        isNullable,
        fields,
        isOpen,
    };
}
exports.definedRecord = definedRecord;
function definedTable(isNullable, fields, isOpen) {
    return {
        kind: __1.Type.TypeKind.Table,
        extendedKind: __1.Type.ExtendedTypeKind.DefinedTable,
        isNullable,
        fields,
        isOpen,
    };
}
exports.definedTable = definedTable;
function functionType(isNullable, parameters, returnType) {
    return {
        kind: __1.Type.TypeKind.Type,
        extendedKind: __1.Type.ExtendedTypeKind.FunctionType,
        isNullable,
        parameters,
        returnType,
    };
}
exports.functionType = functionType;
function logicalLiteral(isNullable, literal) {
    let parsedLiteral;
    let normalizedLiteral;
    if (literal === true || literal === "true") {
        parsedLiteral = "true";
        normalizedLiteral = true;
    }
    else if (literal === false || literal === "false") {
        parsedLiteral = "false";
        normalizedLiteral = false;
    }
    else {
        throw new common_1.CommonError.InvariantError(`invalid boolean string`);
    }
    return {
        isNullable,
        kind: __1.Type.TypeKind.Logical,
        extendedKind: __1.Type.ExtendedTypeKind.LogicalLiteral,
        literal: parsedLiteral,
        normalizedLiteral,
    };
}
exports.logicalLiteral = logicalLiteral;
function listType(isNullable, itemType) {
    return {
        kind: __1.Type.TypeKind.Type,
        extendedKind: __1.Type.ExtendedTypeKind.ListType,
        isNullable,
        itemType,
    };
}
exports.listType = listType;
function numberLiteral(isNullable, literal) {
    let parsedLiteral;
    let normalizedLiteral;
    if (typeof literal === "number") {
        parsedLiteral = literal.toString();
        normalizedLiteral = literal;
    }
    else {
        parsedLiteral = literal;
        normalizedLiteral = Number.parseFloat(common_1.Assert.asDefined(common_1.StringUtils.normalizeNumber(literal)));
    }
    return {
        isNullable,
        kind: __1.Type.TypeKind.Number,
        extendedKind: __1.Type.ExtendedTypeKind.NumberLiteral,
        literal: parsedLiteral,
        normalizedLiteral,
    };
}
exports.numberLiteral = numberLiteral;
function primaryPrimitiveType(isNullable, primitiveType) {
    return {
        kind: __1.Type.TypeKind.Type,
        extendedKind: __1.Type.ExtendedTypeKind.PrimaryPrimitiveType,
        isNullable,
        primitiveType,
    };
}
exports.primaryPrimitiveType = primaryPrimitiveType;
function recordType(isNullable, fields, isOpen) {
    return {
        kind: __1.Type.TypeKind.Type,
        extendedKind: __1.Type.ExtendedTypeKind.RecordType,
        isNullable,
        fields,
        isOpen,
    };
}
exports.recordType = recordType;
function tableType(isNullable, fields, isOpen) {
    return {
        kind: __1.Type.TypeKind.Type,
        extendedKind: __1.Type.ExtendedTypeKind.TableType,
        isNullable,
        fields,
        isOpen,
    };
}
exports.tableType = tableType;
function tableTypePrimary(isNullable, primaryExpression) {
    return {
        kind: __1.Type.TypeKind.Type,
        extendedKind: __1.Type.ExtendedTypeKind.TableTypePrimaryExpression,
        isNullable,
        primaryExpression,
    };
}
exports.tableTypePrimary = tableTypePrimary;
function textLiteral(isNullable, literal) {
    literal = common_1.StringUtils.ensureQuoted(literal);
    return {
        isNullable,
        kind: __1.Type.TypeKind.Text,
        extendedKind: __1.Type.ExtendedTypeKind.TextLiteral,
        literal,
        normalizedLiteral: literal,
    };
}
exports.textLiteral = textLiteral;
//# sourceMappingURL=factories.js.map