"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.isEqualTextLiteral = exports.isEqualTableTypePrimaryExpression = exports.isEqualTableType = exports.isEqualRecordType = exports.isEqualPrimitiveType = exports.isEqualNumberLiteral = exports.isEqualLogicalLiteral = exports.isEqualListType = exports.isEqualFunctionType = exports.isEqualFunctionParameters = exports.isEqualFieldSpecificationList = exports.isEqualPrimaryPrimitiveType = exports.isEqualDefinedTable = exports.isEqualDefinedRecord = exports.isEqualDefinedListType = exports.isEqualDefinedList = exports.isEqualDefinedFunction = exports.isEqualAnyUnion = exports.isEqualExtendedTypes = exports.isEqualTypes = exports.isEqualFunctionSignature = exports.isEqualFunctionParameter = exports.isEqualType = void 0;
const common_1 = require("../../../common");
const typeUtils_1 = require("./typeUtils");
const __1 = require("..");
function isEqualType(left, right) {
    if (left === right) {
        return true;
    }
    else if (left.kind !== right.kind ||
        left.extendedKind !== right.extendedKind ||
        left.isNullable !== right.isNullable) {
        return false;
    }
    else if (left.extendedKind !== undefined && right.extendedKind !== undefined) {
        return isEqualExtendedTypes(left, right);
    }
    else {
        return true;
    }
}
exports.isEqualType = isEqualType;
function isEqualFunctionParameter(left, right) {
    return left.isNullable !== right.isNullable || left.isOptional !== right.isOptional || left.type !== right.type;
}
exports.isEqualFunctionParameter = isEqualFunctionParameter;
function isEqualFunctionSignature(left, right) {
    return (left === right ||
        (left.isNullable === right.isNullable &&
            isEqualType(left.returnType, right.returnType) &&
            isEqualFunctionParameters(left.parameters, right.parameters)));
}
exports.isEqualFunctionSignature = isEqualFunctionSignature;
// Does not care about ordering.
function isEqualTypes(leftTypes, rightTypes) {
    if (leftTypes === rightTypes) {
        return true;
    }
    else if (leftTypes.length !== rightTypes.length) {
        return false;
    }
    const numTypes = leftTypes.length;
    for (let index = 0; index < numTypes; index += 1) {
        if (!(0, typeUtils_1.isTypeInArray)(leftTypes, rightTypes[index])) {
            return false;
        }
    }
    return true;
}
exports.isEqualTypes = isEqualTypes;
function isEqualExtendedTypes(left, right) {
    if (left === right) {
        return true;
    }
    else if (left.extendedKind !== right.extendedKind) {
        return false;
    }
    switch (left.extendedKind) {
        case __1.Type.ExtendedTypeKind.AnyUnion:
            return isEqualAnyUnion(left, right);
        case __1.Type.ExtendedTypeKind.DefinedFunction:
            return isEqualDefinedFunction(left, right);
        case __1.Type.ExtendedTypeKind.DefinedList:
            return isEqualDefinedList(left, right);
        case __1.Type.ExtendedTypeKind.DefinedListType:
            return isEqualDefinedListType(left, right);
        case __1.Type.ExtendedTypeKind.DefinedRecord:
            return isEqualDefinedRecord(left, right);
        case __1.Type.ExtendedTypeKind.DefinedTable:
            return isEqualDefinedTable(left, right);
        case __1.Type.ExtendedTypeKind.FunctionType:
            return isEqualFunctionType(left, right);
        case __1.Type.ExtendedTypeKind.ListType:
            return isEqualListType(left, right);
        case __1.Type.ExtendedTypeKind.LogicalLiteral:
            return isEqualLogicalLiteral(left, right);
        case __1.Type.ExtendedTypeKind.NumberLiteral:
            return isEqualNumberLiteral(left, right);
        case __1.Type.ExtendedTypeKind.PrimaryPrimitiveType:
            return isEqualPrimaryPrimitiveType(left, right);
        case __1.Type.ExtendedTypeKind.RecordType:
            return isEqualRecordType(left, right);
        case __1.Type.ExtendedTypeKind.TableType:
            return isEqualTableType(left, right);
        case __1.Type.ExtendedTypeKind.TableTypePrimaryExpression:
            return isEqualTableTypePrimaryExpression(left, right);
        case __1.Type.ExtendedTypeKind.TextLiteral:
            return isEqualTextLiteral(left, right);
        default:
            throw common_1.Assert.isNever(left);
    }
}
exports.isEqualExtendedTypes = isEqualExtendedTypes;
function isEqualAnyUnion(left, right) {
    return (left === right ||
        (left.isNullable === right.isNullable && isEqualTypes(left.unionedTypePairs, right.unionedTypePairs)));
}
exports.isEqualAnyUnion = isEqualAnyUnion;
function isEqualDefinedFunction(left, right) {
    return isEqualFunctionSignature(left, right);
}
exports.isEqualDefinedFunction = isEqualDefinedFunction;
function isEqualDefinedList(left, right) {
    if (left === right) {
        return true;
    }
    else if (left.elements.length !== right.elements.length || left.isNullable !== right.isNullable) {
        return false;
    }
    const rightElements = right.elements;
    return common_1.ArrayUtils.all(left.elements.map((leftType, index) => isEqualType(leftType, rightElements[index])));
}
exports.isEqualDefinedList = isEqualDefinedList;
function isEqualDefinedListType(left, right) {
    if (left === right) {
        return true;
    }
    else if (left.itemTypes.length !== right.itemTypes.length || left.isNullable !== right.isNullable) {
        return false;
    }
    const rightElements = right.itemTypes;
    return common_1.ArrayUtils.all(left.itemTypes.map((leftType, index) => isEqualType(leftType, rightElements[index])));
}
exports.isEqualDefinedListType = isEqualDefinedListType;
function isEqualDefinedRecord(left, right) {
    return (left === right ||
        (left.isNullable === right.isNullable &&
            common_1.MapUtils.isEqualMap(left.fields, right.fields, isEqualType)));
}
exports.isEqualDefinedRecord = isEqualDefinedRecord;
function isEqualDefinedTable(left, right) {
    return (left === right ||
        (left.isNullable === right.isNullable &&
            common_1.MapUtils.isEqualMap(left.fields, right.fields, isEqualType)));
}
exports.isEqualDefinedTable = isEqualDefinedTable;
function isEqualPrimaryPrimitiveType(left, right) {
    return (left === right || (left.isNullable === right.isNullable && isEqualType(left.primitiveType, right.primitiveType)));
}
exports.isEqualPrimaryPrimitiveType = isEqualPrimaryPrimitiveType;
function isEqualFieldSpecificationList(left, right) {
    if (left === right) {
        return true;
    }
    else if (left.isOpen !== right.isOpen || left.fields.size !== right.fields.size) {
        return false;
    }
    for (const [key, leftValue] of left.fields.entries()) {
        const rightValue = right.fields.get(key);
        if (rightValue === undefined || !isEqualType(leftValue, rightValue)) {
            return false;
        }
    }
    return true;
}
exports.isEqualFieldSpecificationList = isEqualFieldSpecificationList;
function isEqualFunctionParameters(left, right) {
    if (left === right) {
        return true;
    }
    else if (left.length !== right.length) {
        return false;
    }
    const numParameters = left.length;
    for (let index = 0; index < numParameters; index += 1) {
        const nthLeft = left[index];
        const nthRight = right[index];
        if (!isEqualFunctionParameter(nthLeft, nthRight)) {
            return false;
        }
    }
    return true;
}
exports.isEqualFunctionParameters = isEqualFunctionParameters;
function isEqualFunctionType(left, right) {
    return isEqualFunctionSignature(left, right);
}
exports.isEqualFunctionType = isEqualFunctionType;
function isEqualListType(left, right) {
    return left === right || (left.isNullable === right.isNullable && isEqualType(left.itemType, right.itemType));
}
exports.isEqualListType = isEqualListType;
function isEqualLogicalLiteral(left, right) {
    return (left === right || (left.isNullable === right.isNullable && left.normalizedLiteral === right.normalizedLiteral));
}
exports.isEqualLogicalLiteral = isEqualLogicalLiteral;
function isEqualNumberLiteral(left, right) {
    return (left === right || (left.isNullable === right.isNullable && left.normalizedLiteral === right.normalizedLiteral));
}
exports.isEqualNumberLiteral = isEqualNumberLiteral;
function isEqualPrimitiveType(left, right) {
    return left === right || (left.kind === right.kind && left.isNullable === right.isNullable);
}
exports.isEqualPrimitiveType = isEqualPrimitiveType;
function isEqualRecordType(left, right) {
    return left.isNullable === right.isNullable && isEqualFieldSpecificationList(left, right);
}
exports.isEqualRecordType = isEqualRecordType;
function isEqualTableType(left, right) {
    return left.isNullable === right.isNullable && isEqualFieldSpecificationList(left, right);
}
exports.isEqualTableType = isEqualTableType;
function isEqualTableTypePrimaryExpression(left, right) {
    return left === right || isEqualType(left.primaryExpression, right.primaryExpression);
}
exports.isEqualTableTypePrimaryExpression = isEqualTableTypePrimaryExpression;
function isEqualTextLiteral(left, right) {
    return left === right || (left.isNullable === right.isNullable && left.literal === right.literal);
}
exports.isEqualTextLiteral = isEqualTextLiteral;
//# sourceMappingURL=isEqualType.js.map