"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.simplify = void 0;
const categorize_1 = require("./categorize");
const isEqualType_1 = require("./isEqualType");
const __1 = require("..");
const typeTraceConstant_1 = require("./typeTraceConstant");
function simplify(types, traceManager, correlationId) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z;
    const trace = traceManager.entry(typeTraceConstant_1.TypeUtilsTraceConstant.Simplify, simplify.name, correlationId);
    const categorized = (0, categorize_1.categorize)(types, traceManager, trace.id);
    // If an `any` exists then that's as simplified as we can make it.
    const any = findAnyPrimitive(categorized);
    if (any) {
        return [any];
    }
    const partial = [
        ...((_b = (_a = categorized.actions) === null || _a === void 0 ? void 0 : _a.primitives.values()) !== null && _b !== void 0 ? _b : []),
        ...((_d = (_c = categorized.anyNonNulls) === null || _c === void 0 ? void 0 : _c.primitives.values()) !== null && _d !== void 0 ? _d : []),
        ...((_f = (_e = categorized.binaries) === null || _e === void 0 ? void 0 : _e.primitives.values()) !== null && _f !== void 0 ? _f : []),
        ...((_h = (_g = categorized.dates) === null || _g === void 0 ? void 0 : _g.primitives.values()) !== null && _h !== void 0 ? _h : []),
        ...((_k = (_j = categorized.dateTimes) === null || _j === void 0 ? void 0 : _j.primitives.values()) !== null && _k !== void 0 ? _k : []),
        ...((_m = (_l = categorized.dateTimeZones) === null || _l === void 0 ? void 0 : _l.primitives.values()) !== null && _m !== void 0 ? _m : []),
        ...((_p = (_o = categorized.durations) === null || _o === void 0 ? void 0 : _o.primitives.values()) !== null && _p !== void 0 ? _p : []),
        ...((_r = (_q = categorized.nones) === null || _q === void 0 ? void 0 : _q.primitives.values()) !== null && _r !== void 0 ? _r : []),
        ...((_t = (_s = categorized.notApplicables) === null || _s === void 0 ? void 0 : _s.primitives.values()) !== null && _t !== void 0 ? _t : []),
        ...((_v = (_u = categorized.nulls) === null || _u === void 0 ? void 0 : _u.primitives.values()) !== null && _v !== void 0 ? _v : []),
        ...((_x = (_w = categorized.times) === null || _w === void 0 ? void 0 : _w.primitives.values()) !== null && _x !== void 0 ? _x : []),
        ...((_z = (_y = categorized.unknowns) === null || _y === void 0 ? void 0 : _y.primitives.values()) !== null && _z !== void 0 ? _z : []),
        ...simplifyFunctionCategory(categorized.functions),
        ...simplifyListCategory(categorized.lists),
        ...simplifyLogicalCategory(categorized.logicals),
        ...simplifyNumberCategory(categorized.numbers),
        ...simplifyRecordCategory(categorized.records),
        ...simplifyTableCategory(categorized.tables),
        ...simplifyTextCategory(categorized.texts),
        ...simplifyTypeCategory(categorized.types),
    ];
    for (const flattenedValue of simplifyAnyCategory(categorized.anys)) {
        if (!partial.find((item) => (0, isEqualType_1.isEqualType)(item, flattenedValue))) {
            partial.push(flattenedValue);
        }
    }
    trace.exit();
    return partial;
}
exports.simplify = simplify;
// Returns the first nullable instance if one exists,
// otherwise returns the first element in the collection.
function firstNullableElseFirst(immutableSet) {
    const setValues = [...immutableSet.values()];
    for (const item of setValues) {
        if (item.isNullable) {
            return item;
        }
    }
    return setValues[0];
}
function findAnyPrimitive(categorized) {
    var _a;
    const anySet = (_a = categorized.anys) === null || _a === void 0 ? void 0 : _a.primitives;
    return anySet ? firstNullableElseFirst(anySet) : undefined;
}
function simplifyAnyCategory(category) {
    if (!(category === null || category === void 0 ? void 0 : category.flattenedAnyUnions)) {
        return [];
    }
    else {
        const flattnedAnyUnions = category === null || category === void 0 ? void 0 : category.flattenedAnyUnions;
        return [...flattnedAnyUnions.values()];
    }
}
function simplifyFunctionCategory(category) {
    return category ? simplifyExtendedType(category.primitives, category.definedFunctions) : [];
}
function simplifyListCategory(category) {
    return category ? simplifyExtendedType(category.primitives, category.definedLists) : [];
}
function simplifyLogicalCategory(category) {
    if (category === undefined) {
        return [];
    }
    else if ((category.hasFalsyNullableLiteral || category.hasFalsyNonNullableLiteral) &&
        (category.hasTruthyNullableLiteral || category.hasTruthyNonNullableLiteral)) {
        return category.hasFalsyNullableLiteral || category.hasTruthyNullableLiteral
            ? [__1.Type.NullableLogicalInstance]
            : [__1.Type.LogicalInstance];
    }
    else {
        const type = firstNullableElseFirst(category.primitives);
        return type ? [type] : [];
    }
}
function simplifyNumberCategory(category) {
    return category ? simplifyExtendedType(category.primitives, category.literals) : [];
}
function simplifyRecordCategory(category) {
    return category ? simplifyExtendedType(category.primitives, category.definedRecords) : [];
}
function simplifyTableCategory(category) {
    return category ? simplifyExtendedType(category.primitives, category.definedTables) : [];
}
function simplifyTextCategory(category) {
    return category ? simplifyExtendedType(category.primitives, category.literals) : [];
}
function simplifyTypeCategory(category) {
    if (category === undefined) {
        return [];
    }
    else if (category.primitives.size) {
        const typeType = firstNullableElseFirst(category.primitives);
        return typeType ? [typeType] : [];
    }
    else {
        return [
            ...category.definedListTypes.values(),
            ...category.functionTypes.values(),
            ...category.listTypes.values(),
            ...category.primaryPrimitiveTypes.values(),
            ...category.recordTypes.values(),
            ...category.tablePrimaryExpressionTypes.values(),
            ...category.tableTypes.values(),
        ];
    }
}
function simplifyExtendedType(primitives, literals) {
    if (primitives.size) {
        const type = firstNullableElseFirst(primitives);
        return type ? [type] : [];
    }
    else if (literals.size) {
        return [...literals.values()];
    }
    else {
        return [];
    }
}
//# sourceMappingURL=simplify.js.map