import { TraceManager } from "../../../common/trace";
import { Type } from "..";
export declare type TChecked = CheckedDefinedFunction | CheckedDefinedList | CheckedDefinedRecord | CheckedDefinedTable | CheckedFunctionSignature | CheckedInvocation;
export interface IChecked<Key, Mismatch> {
    readonly valid: ReadonlyArray<Key>;
    readonly invalid: Map<Key, Mismatch>;
    readonly extraneous: ReadonlyArray<Key>;
    readonly missing: ReadonlyArray<Key>;
}
export interface CheckedDefinedFunction extends IChecked<number, DefinedFunctionMismatch> {
    readonly isReturnTypeCompatible: boolean;
}
export declare type CheckedDefinedList = IChecked<number, DefinedListMismatch>;
export declare type CheckedDefinedRecord = IChecked<string, DefinedRecordMismatch>;
export declare type CheckedDefinedTable = IChecked<string, DefinedTableMismatch>;
export declare type CheckedFunctionSignature = IChecked<number, FunctionSignatureMismatch>;
export declare type CheckedInvocation = IChecked<number, InvocationMismatch>;
export declare type TMismatch = DefinedFunctionMismatch | DefinedListMismatch | DefinedRecordMismatch | DefinedTableMismatch | FunctionSignatureMismatch | InvocationMismatch;
export interface IMismatch<Actual, Expected> {
    readonly actual: Actual;
    readonly expected: Expected;
}
export declare type DefinedFunctionMismatch = IMismatch<Type.FunctionParameter, Type.FunctionParameter>;
export declare type DefinedListMismatch = IMismatch<Type.TPowerQueryType, Type.TPowerQueryType>;
export declare type DefinedRecordMismatch = IMismatch<Type.TPowerQueryType, Type.TPowerQueryType>;
export declare type DefinedTableMismatch = IMismatch<Type.TPowerQueryType, Type.TPowerQueryType>;
export declare type FunctionSignatureMismatch = IMismatch<Type.FunctionParameter, Type.FunctionParameter>;
export declare type InvocationMismatch = IMismatch<Type.TPowerQueryType | undefined, Type.FunctionParameter>;
export declare function typeCheckFunction(valueType: Type.DefinedFunction, schemaType: Type.FunctionType, traceManager: TraceManager, correlationId: number | undefined): CheckedDefinedFunction;
export declare function typeCheckFunctionSignature(valueType: Type.FunctionSignature, schemaType: Type.FunctionSignature, traceManager: TraceManager, correlationId: number | undefined): CheckedFunctionSignature;
export declare function typeCheckInvocation(args: ReadonlyArray<Type.TPowerQueryType>, definedFunction: Type.DefinedFunction, traceManager: TraceManager, correlationId: number | undefined): CheckedInvocation;
export declare function typeCheckListWithListType(valueType: Type.DefinedList, schemaType: Type.ListType, traceManager: TraceManager, correlationId: number | undefined): CheckedDefinedList;
export declare function typeCheckListWithDefinedListType(valueType: Type.DefinedList, schemaType: Type.DefinedListType, traceManager: TraceManager, correlationId: number | undefined): CheckedDefinedList;
export declare function typeCheckRecord(valueType: Type.DefinedRecord, schemaType: Type.RecordType, traceManager: TraceManager, correlationId: number | undefined): CheckedDefinedRecord;
export declare function typeCheckTable(valueType: Type.DefinedTable, schemaType: Type.TableType, traceManager: TraceManager, correlationId: number | undefined): CheckedDefinedTable;
