"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.isTInnerLexError = exports.isTLexError = exports.isLexError = exports.UnterminatedMultilineTokenError = exports.UnexpectedReadError = exports.UnexpectedEofError = exports.ExpectedError = exports.EndOfStreamError = exports.ErrorLineMapError = exports.BadStateError = exports.BadRangeError = exports.BadLineNumberError = exports.LexError = exports.UnterminatedMultilineTokenKind = exports.ExpectedKind = exports.BadRangeKind = exports.BadLineNumberKind = void 0;
const common_1 = require("../common");
const localization_1 = require("../localization");
var BadLineNumberKind;
(function (BadLineNumberKind) {
    BadLineNumberKind["LessThanZero"] = "LessThanZero";
    BadLineNumberKind["GreaterThanNumLines"] = "GreaterThanNumLines";
})(BadLineNumberKind = exports.BadLineNumberKind || (exports.BadLineNumberKind = {}));
// do not these sort variants,
// they are in order that logical checks are made, which in turn help create logical variants
var BadRangeKind;
(function (BadRangeKind) {
    BadRangeKind["SameLine_LineCodeUnitStart_Higher"] = "SameLine_LineCodeUnitStart_Higher";
    BadRangeKind["LineNumberStart_GreaterThan_LineNumberEnd"] = "LineNumberStart_GreaterThan_LineNumberEnd";
    BadRangeKind["LineNumberStart_LessThan_Zero"] = "LineNumberStart_LessThan_Zero";
    BadRangeKind["LineNumberStart_GreaterThan_NumLines"] = "LineNumberStart_GreaterThan_NumLines";
    BadRangeKind["LineNumberEnd_GreaterThan_NumLines"] = "LineNumberEnd_GreaterThan_NumLines";
    BadRangeKind["LineCodeUnitStart_GreaterThan_LineLength"] = "LineCodeUnitStart_GreaterThan_LineLength";
    BadRangeKind["LineCodeUnitEnd_GreaterThan_LineLength"] = "LineCodeUnitEnd_GreaterThan_LineLength";
})(BadRangeKind = exports.BadRangeKind || (exports.BadRangeKind = {}));
var ExpectedKind;
(function (ExpectedKind) {
    ExpectedKind["HexLiteral"] = "HexLiteral";
    ExpectedKind["KeywordOrIdentifier"] = "KeywordOrIdentifier";
    ExpectedKind["Numeric"] = "Numeric";
})(ExpectedKind = exports.ExpectedKind || (exports.ExpectedKind = {}));
var UnterminatedMultilineTokenKind;
(function (UnterminatedMultilineTokenKind) {
    UnterminatedMultilineTokenKind["MultilineComment"] = "MultilineComment";
    UnterminatedMultilineTokenKind["QuotedIdentifier"] = "QuotedIdentifier";
    UnterminatedMultilineTokenKind["Text"] = "Text";
})(UnterminatedMultilineTokenKind = exports.UnterminatedMultilineTokenKind || (exports.UnterminatedMultilineTokenKind = {}));
class LexError extends Error {
    constructor(innerError) {
        super(innerError.message);
        this.innerError = innerError;
        Object.setPrototypeOf(this, LexError);
    }
}
exports.LexError = LexError;
class BadLineNumberError extends Error {
    constructor(kind, lineNumber, numLines, locale) {
        super(localization_1.Localization.error_lex_badLineNumber(localization_1.LocalizationUtils.getLocalizationTemplates(locale), kind));
        this.kind = kind;
        this.lineNumber = lineNumber;
        this.numLines = numLines;
        Object.setPrototypeOf(this, BadLineNumberError.prototype);
    }
}
exports.BadLineNumberError = BadLineNumberError;
class BadRangeError extends Error {
    constructor(range, kind, locale) {
        super(localization_1.Localization.error_lex_badRange(localization_1.LocalizationUtils.getLocalizationTemplates(locale), kind));
        this.range = range;
        this.kind = kind;
        Object.setPrototypeOf(this, BadRangeError.prototype);
    }
}
exports.BadRangeError = BadRangeError;
class BadStateError extends Error {
    constructor(innerError, locale) {
        super(localization_1.Localization.error_lex_badState(localization_1.LocalizationUtils.getLocalizationTemplates(locale)));
        this.innerError = innerError;
        Object.setPrototypeOf(this, BadStateError.prototype);
    }
}
exports.BadStateError = BadStateError;
class ErrorLineMapError extends Error {
    constructor(errorLineMap, locale) {
        super(localization_1.Localization.error_lex_lineMap(localization_1.LocalizationUtils.getLocalizationTemplates(locale), errorLineMap));
        this.errorLineMap = errorLineMap;
        Object.setPrototypeOf(this, ErrorLineMapError.prototype);
    }
}
exports.ErrorLineMapError = ErrorLineMapError;
class EndOfStreamError extends Error {
    constructor(locale) {
        super(localization_1.Localization.error_lex_endOfStream(localization_1.LocalizationUtils.getLocalizationTemplates(locale)));
        Object.setPrototypeOf(this, EndOfStreamError.prototype);
    }
}
exports.EndOfStreamError = EndOfStreamError;
class ExpectedError extends Error {
    constructor(graphemePosition, kind, locale) {
        super(localization_1.Localization.error_lex_expectedKind(localization_1.LocalizationUtils.getLocalizationTemplates(locale), kind));
        this.graphemePosition = graphemePosition;
        this.kind = kind;
        Object.setPrototypeOf(this, ExpectedError.prototype);
    }
}
exports.ExpectedError = ExpectedError;
class UnexpectedEofError extends Error {
    constructor(graphemePosition, locale) {
        super(localization_1.Localization.error_lex_endOfStreamPartwayRead(localization_1.LocalizationUtils.getLocalizationTemplates(locale)));
        this.graphemePosition = graphemePosition;
        Object.setPrototypeOf(this, UnexpectedEofError.prototype);
    }
}
exports.UnexpectedEofError = UnexpectedEofError;
class UnexpectedReadError extends Error {
    constructor(graphemePosition, locale) {
        super(localization_1.Localization.error_lex_unexpectedRead(localization_1.LocalizationUtils.getLocalizationTemplates(locale)));
        this.graphemePosition = graphemePosition;
        Object.setPrototypeOf(this, UnexpectedReadError.prototype);
    }
}
exports.UnexpectedReadError = UnexpectedReadError;
class UnterminatedMultilineTokenError extends Error {
    constructor(locale, graphemePosition, kind) {
        super(localization_1.Localization.error_lex_unterminatedMultilineToken(localization_1.LocalizationUtils.getLocalizationTemplates(locale), kind));
        this.graphemePosition = graphemePosition;
        this.kind = kind;
        Object.setPrototypeOf(this, UnterminatedMultilineTokenError.prototype);
    }
}
exports.UnterminatedMultilineTokenError = UnterminatedMultilineTokenError;
function isLexError(error) {
    return error instanceof LexError;
}
exports.isLexError = isLexError;
function isTLexError(error) {
    return error instanceof LexError || error instanceof common_1.CommonError.CommonError;
}
exports.isTLexError = isTLexError;
function isTInnerLexError(error) {
    return (error instanceof BadLineNumberError ||
        error instanceof BadRangeError ||
        error instanceof BadStateError ||
        error instanceof EndOfStreamError ||
        error instanceof ErrorLineMapError ||
        error instanceof ExpectedError ||
        error instanceof UnexpectedEofError ||
        error instanceof UnexpectedReadError ||
        error instanceof UnterminatedMultilineTokenError);
}
exports.isTInnerLexError = isTInnerLexError;
//# sourceMappingURL=error.js.map