"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.Localization = exports.localizeTokenKind = void 0;
const common_1 = require("../common");
const error_1 = require("../parser/error");
const lexer_1 = require("../lexer");
const parser_1 = require("../parser");
const language_1 = require("../language");
function localizeTokenKind(localizationTemplates, tokenKind) {
    switch (tokenKind) {
        case language_1.Token.TokenKind.Ampersand:
            return localizationTemplates.tokenKind_ampersand;
        case language_1.Token.TokenKind.Asterisk:
            return localizationTemplates.tokenKind_asterisk;
        case language_1.Token.TokenKind.AtSign:
            return localizationTemplates.tokenKind_atSign;
        case language_1.Token.TokenKind.Bang:
            return localizationTemplates.tokenKind_bang;
        case language_1.Token.TokenKind.Comma:
            return localizationTemplates.tokenKind_comma;
        case language_1.Token.TokenKind.Division:
            return localizationTemplates.tokenKind_division;
        case language_1.Token.TokenKind.DotDot:
            return localizationTemplates.tokenKind_dotDot;
        case language_1.Token.TokenKind.Ellipsis:
            return localizationTemplates.tokenKind_ellipsis;
        case language_1.Token.TokenKind.Equal:
            return localizationTemplates.tokenKind_equal;
        case language_1.Token.TokenKind.FatArrow:
            return localizationTemplates.tokenKind_fatArrow;
        case language_1.Token.TokenKind.GreaterThan:
            return localizationTemplates.tokenKind_greaterThan;
        case language_1.Token.TokenKind.GreaterThanEqualTo:
            return localizationTemplates.tokenKind_greaterThanEqualTo;
        case language_1.Token.TokenKind.HexLiteral:
            return localizationTemplates.tokenKind_hexLiteral;
        case language_1.Token.TokenKind.Identifier:
            return localizationTemplates.tokenKind_identifier;
        case language_1.Token.TokenKind.KeywordAnd:
            return localizationTemplates.tokenKind_keywordAnd;
        case language_1.Token.TokenKind.KeywordAs:
            return localizationTemplates.tokenKind_keywordAs;
        case language_1.Token.TokenKind.KeywordEach:
            return localizationTemplates.tokenKind_keywordEach;
        case language_1.Token.TokenKind.KeywordElse:
            return localizationTemplates.tokenKind_keywordElse;
        case language_1.Token.TokenKind.KeywordError:
            return localizationTemplates.tokenKind_keywordError;
        case language_1.Token.TokenKind.KeywordFalse:
            return localizationTemplates.tokenKind_keywordFalse;
        case language_1.Token.TokenKind.KeywordHashBinary:
            return localizationTemplates.tokenKind_keywordHashBinary;
        case language_1.Token.TokenKind.KeywordHashDate:
            return localizationTemplates.tokenKind_keywordHashDate;
        case language_1.Token.TokenKind.KeywordHashDateTime:
            return localizationTemplates.tokenKind_keywordHashDateTime;
        case language_1.Token.TokenKind.KeywordHashDateTimeZone:
            return localizationTemplates.tokenKind_keywordHashDateTimeZone;
        case language_1.Token.TokenKind.KeywordHashDuration:
            return localizationTemplates.tokenKind_keywordHashDuration;
        case language_1.Token.TokenKind.KeywordHashInfinity:
            return localizationTemplates.tokenKind_keywordHashInfinity;
        case language_1.Token.TokenKind.KeywordHashNan:
            return localizationTemplates.tokenKind_keywordHashNan;
        case language_1.Token.TokenKind.KeywordHashSections:
            return localizationTemplates.tokenKind_keywordHashSections;
        case language_1.Token.TokenKind.KeywordHashShared:
            return localizationTemplates.tokenKind_keywordShared;
        case language_1.Token.TokenKind.KeywordHashTable:
            return localizationTemplates.tokenKind_keywordHashTable;
        case language_1.Token.TokenKind.KeywordHashTime:
            return localizationTemplates.tokenKind_keywordHashTime;
        case language_1.Token.TokenKind.KeywordIf:
            return localizationTemplates.tokenKind_keywordIf;
        case language_1.Token.TokenKind.KeywordIn:
            return localizationTemplates.tokenKind_keywordIn;
        case language_1.Token.TokenKind.KeywordIs:
            return localizationTemplates.tokenKind_keywordIs;
        case language_1.Token.TokenKind.KeywordLet:
            return localizationTemplates.tokenKind_keywordLet;
        case language_1.Token.TokenKind.KeywordMeta:
            return localizationTemplates.tokenKind_keywordMeta;
        case language_1.Token.TokenKind.KeywordNot:
            return localizationTemplates.tokenKind_notEqual;
        case language_1.Token.TokenKind.KeywordOr:
            return localizationTemplates.tokenKind_keywordOr;
        case language_1.Token.TokenKind.KeywordOtherwise:
            return localizationTemplates.tokenKind_keywordOtherwise;
        case language_1.Token.TokenKind.KeywordSection:
            return localizationTemplates.tokenKind_keywordSection;
        case language_1.Token.TokenKind.KeywordShared:
            return localizationTemplates.tokenKind_keywordShared;
        case language_1.Token.TokenKind.KeywordThen:
            return localizationTemplates.tokenKind_keywordThen;
        case language_1.Token.TokenKind.KeywordTrue:
            return localizationTemplates.tokenKind_keywordTrue;
        case language_1.Token.TokenKind.KeywordTry:
            return localizationTemplates.tokenKind_keywordTry;
        case language_1.Token.TokenKind.KeywordType:
            return localizationTemplates.tokenKind_keywordType;
        case language_1.Token.TokenKind.LeftBrace:
            return localizationTemplates.tokenKind_leftBrace;
        case language_1.Token.TokenKind.LeftBracket:
            return localizationTemplates.tokenKind_leftBracket;
        case language_1.Token.TokenKind.LeftParenthesis:
            return localizationTemplates.tokenKind_leftParenthesis;
        case language_1.Token.TokenKind.LessThan:
            return localizationTemplates.tokenKind_lessThan;
        case language_1.Token.TokenKind.LessThanEqualTo:
            return localizationTemplates.tokenKind_lessThanEqualTo;
        case language_1.Token.TokenKind.Minus:
            return localizationTemplates.tokenKind_minus;
        case language_1.Token.TokenKind.NotEqual:
            return localizationTemplates.tokenKind_notEqual;
        case language_1.Token.TokenKind.NullCoalescingOperator:
            return localizationTemplates.tokenKind_nullCoalescingOperator;
        case language_1.Token.TokenKind.NullLiteral:
            return localizationTemplates.tokenKind_nullLiteral;
        case language_1.Token.TokenKind.NumericLiteral:
            return localizationTemplates.tokenKind_numericLiteral;
        case language_1.Token.TokenKind.Plus:
            return localizationTemplates.tokenKind_plus;
        case language_1.Token.TokenKind.QuestionMark:
            return localizationTemplates.tokenKind_questionMark;
        case language_1.Token.TokenKind.RightBrace:
            return localizationTemplates.tokenKind_rightBrace;
        case language_1.Token.TokenKind.RightBracket:
            return localizationTemplates.tokenKind_rightBracket;
        case language_1.Token.TokenKind.RightParenthesis:
            return localizationTemplates.tokenKind_rightParenthesis;
        case language_1.Token.TokenKind.Semicolon:
            return localizationTemplates.tokenKind_semicolon;
        case language_1.Token.TokenKind.TextLiteral:
            return localizationTemplates.tokenKind_textLiteral;
        default:
            throw common_1.Assert.isNever(tokenKind);
    }
}
exports.localizeTokenKind = localizeTokenKind;
exports.Localization = {
    error_common_cancellationError: (templates, reason) => common_1.StringUtils.assertGetFormatted(templates.error_common_cancellationError, new Map([["reason", reason]])),
    error_common_invariantError: (templates, invariantBroken, jsonDetails) => {
        if (jsonDetails !== undefined) {
            return common_1.StringUtils.assertGetFormatted(templates.error_common_invariantError_1_details, new Map([
                ["invariantBroken", invariantBroken],
                ["details", JSON.stringify(jsonDetails, undefined, 4)],
            ]));
        }
        else {
            return common_1.StringUtils.assertGetFormatted(templates.error_common_invariantError_2_noDetails, new Map([["invariantBroken", invariantBroken]]));
        }
    },
    error_common_unknown: (templates, innerError) => common_1.StringUtils.assertGetFormatted(templates.error_common_unknown, new Map([["innerError", innerError]])),
    error_lex_badLineNumber: (templates, kind) => {
        switch (kind) {
            case lexer_1.LexError.BadLineNumberKind.GreaterThanNumLines:
                return templates.error_lex_badLineNumber_1_greaterThanNumLines;
            case lexer_1.LexError.BadLineNumberKind.LessThanZero:
                return templates.error_lex_badLineNumber_2_lessThanZero;
            default:
                throw common_1.Assert.isNever(kind);
        }
    },
    error_lex_badRange: (templates, kind) => {
        switch (kind) {
            case lexer_1.LexError.BadRangeKind.SameLine_LineCodeUnitStart_Higher:
                return templates.error_lex_badRange_7_sameLine_codeUnitStartGreaterThanCodeUnitEnd;
            case lexer_1.LexError.BadRangeKind.LineNumberStart_GreaterThan_LineNumberEnd:
                return templates.error_lex_badRange_4_lineNumberStart_greaterThanLineNumberEnd;
            case lexer_1.LexError.BadRangeKind.LineNumberStart_LessThan_Zero:
                return templates.error_lex_badRange_6_lineNumberStart_lessThanZero;
            case lexer_1.LexError.BadRangeKind.LineNumberStart_GreaterThan_NumLines:
                return templates.error_lex_badRange_5_lineNumberStart_greaterThanNumLines;
            case lexer_1.LexError.BadRangeKind.LineNumberEnd_GreaterThan_NumLines:
                return templates.error_lex_badRange_2_lineNumberEnd_greaterThanLineNumbers;
            case lexer_1.LexError.BadRangeKind.LineCodeUnitStart_GreaterThan_LineLength:
                return templates.error_lex_badRange_3_lineNumberStart_greaterThanLineLength;
            case lexer_1.LexError.BadRangeKind.LineCodeUnitEnd_GreaterThan_LineLength:
                return templates.error_lex_badRange_1_lineNumberEnd_greaterThanLineLength;
            default:
                throw common_1.Assert.isNever(kind);
        }
    },
    error_lex_badState: (templates) => templates.error_lex_badState,
    error_lex_endOfStream: (templates) => templates.error_lex_endOfStream,
    error_lex_endOfStreamPartwayRead: (templates) => templates.error_lex_endOfStreamPartwayRead,
    error_lex_expectedKind: (templates, kind) => {
        switch (kind) {
            case lexer_1.LexError.ExpectedKind.HexLiteral:
                return templates.error_lex_expectedKind_1_hex;
            case lexer_1.LexError.ExpectedKind.KeywordOrIdentifier:
                return templates.error_lex_expectedKind_2_keywordOrIdentifier;
            case lexer_1.LexError.ExpectedKind.Numeric:
                return templates.error_lex_expectedKind_3_numeric;
            default:
                throw common_1.Assert.isNever(kind);
        }
    },
    error_lex_lineMap: (templates, errorLineMap) => {
        const lineNumbers = [...errorLineMap.keys()]
            .map((lineNumber) => lineNumber.toString())
            .join(",");
        return common_1.StringUtils.assertGetFormatted(templates.error_lex_lineMap, new Map([["lineNumbers", lineNumbers]]));
    },
    error_lex_unexpectedRead: (templates) => templates.error_lex_unexpectedRead,
    error_lex_unterminatedMultilineToken: (templates, kind) => {
        switch (kind) {
            case lexer_1.LexError.UnterminatedMultilineTokenKind.MultilineComment:
                return templates.error_lex_unterminatedMultilineToken_1_comment;
            case lexer_1.LexError.UnterminatedMultilineTokenKind.QuotedIdentifier:
                return templates.error_lex_unterminatedMultilineToken_2_quotedIdentifier;
            case lexer_1.LexError.UnterminatedMultilineTokenKind.Text:
                return templates.error_lex_unterminatedMultilineToken_3_string;
            default:
                throw common_1.Assert.isNever(kind);
        }
    },
    error_parse_csvContinuation: (templates, kind) => {
        switch (kind) {
            case parser_1.ParseError.CsvContinuationKind.DanglingComma:
                return templates.error_parse_csvContinuation_1_danglingComma;
            case parser_1.ParseError.CsvContinuationKind.LetExpression:
                return templates.error_parse_csvContinuation_2_letExpression;
            default:
                throw common_1.Assert.isNever(kind);
        }
    },
    error_parse_expectAnyTokenKind: (templates, expectedAnyTokenKinds, foundToken) => {
        const localizedExpectedAnyTokenKinds = expectedAnyTokenKinds
            .map((tokenKind) => localizeTokenKind(templates, tokenKind))
            .join(", ");
        if (foundToken !== undefined) {
            return common_1.StringUtils.assertGetFormatted(templates.error_parse_expectAnyTokenKind_1_other, new Map([
                ["foundTokenKind", localizeTokenKind(templates, foundToken.token.kind)],
                ["expectedAnyTokenKinds", localizedExpectedAnyTokenKinds],
            ]));
        }
        else {
            return common_1.StringUtils.assertGetFormatted(templates.error_parse_expectAnyTokenKind_2_endOfStream, new Map([["expectedAnyTokenKinds", localizedExpectedAnyTokenKinds]]));
        }
    },
    error_parse_expectCommaOrKind: (templates, expectedTokenKind) => common_1.StringUtils.assertGetFormatted(templates.error_parse_expectedCommaOrTokenKind, new Map([
        ["localizedComma", localizeTokenKind(templates, language_1.Token.TokenKind.Comma)],
        ["localizedAlternative", localizeTokenKind(templates, expectedTokenKind)],
    ])),
    error_parse_expectGeneralizedIdentifier: (templates, foundToken) => {
        if (foundToken !== undefined) {
            return templates.error_parse_expectGeneralizedIdentifier_1_other;
        }
        else {
            return templates.error_parse_expectGeneralizedIdentifier_2_endOfStream;
        }
    },
    error_parse_expectTokenKind: (templates, expectedTokenKind, foundToken) => {
        const localizedExpectedTokenKind = localizeTokenKind(templates, expectedTokenKind);
        if (foundToken !== undefined) {
            return common_1.StringUtils.assertGetFormatted(templates.error_parse_expectTokenKind_1_other, new Map([
                ["expectedTokenKind", localizedExpectedTokenKind],
                ["foundTokenKind", localizeTokenKind(templates, foundToken.token.kind)],
            ]));
        }
        else {
            return common_1.StringUtils.assertGetFormatted(templates.error_parse_expectTokenKind_2_endOfStream, new Map([["expectedTokenKind", localizedExpectedTokenKind]]));
        }
    },
    error_parse_invalidCatchFunction: (templates) => templates.error_parse_invalidCatchFunction,
    error_parse_invalidPrimitiveType: (templates, token) => common_1.StringUtils.assertGetFormatted(templates.error_parse_invalidPrimitiveType, new Map([["foundTokenKind", localizeTokenKind(templates, token.kind)]])),
    error_parse_requiredParameterAfterOptional: (templates) => templates.error_parse_requiredParameterAfterOptional,
    error_parse_unterminated_sequence: (templates, sequenceKind) => {
        switch (sequenceKind) {
            case error_1.SequenceKind.Bracket:
                return templates.error_parse_unterminated_sequence_bracket;
            case error_1.SequenceKind.Parenthesis:
                return templates.error_parse_unterminated_sequence_parenthesis;
            default:
                throw common_1.Assert.isNever(sequenceKind);
        }
    },
    error_parse_unusedTokens: (templates) => templates.error_parse_unusedTokens,
};
//# sourceMappingURL=localization.js.map