"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.tokenFrom = exports.isTInnerParseError = exports.isTParseError = exports.isParseError = exports.assertIsParseError = exports.UnusedTokensRemainError = exports.UnterminatedSequence = exports.RequiredParameterAfterOptionalParameterError = exports.InvalidPrimitiveTypeError = exports.InvalidCatchFunctionError = exports.ExpectedGeneralizedIdentifierError = exports.ExpectedTokenKindError = exports.ExpectedClosingTokenKind = exports.ExpectedAnyTokenKindError = exports.ExpectedCsvContinuationError = exports.ParseError = exports.SequenceKind = exports.CsvContinuationKind = void 0;
const common_1 = require("../common");
const localization_1 = require("../localization");
var CsvContinuationKind;
(function (CsvContinuationKind) {
    CsvContinuationKind["DanglingComma"] = "DanglingComma";
    CsvContinuationKind["LetExpression"] = "LetExpression";
})(CsvContinuationKind = exports.CsvContinuationKind || (exports.CsvContinuationKind = {}));
var SequenceKind;
(function (SequenceKind) {
    SequenceKind["Bracket"] = "Bracket";
    SequenceKind["Parenthesis"] = "Parenthesis";
})(SequenceKind = exports.SequenceKind || (exports.SequenceKind = {}));
class ParseError extends Error {
    constructor(innerError, state) {
        super(innerError.message);
        this.innerError = innerError;
        this.state = state;
        Object.setPrototypeOf(this, ParseError.prototype);
    }
}
exports.ParseError = ParseError;
class ExpectedCsvContinuationError extends Error {
    constructor(kind, foundToken, locale) {
        super(localization_1.Localization.error_parse_csvContinuation(localization_1.LocalizationUtils.getLocalizationTemplates(locale), kind));
        this.kind = kind;
        this.foundToken = foundToken;
        Object.setPrototypeOf(this, ExpectedCsvContinuationError.prototype);
    }
}
exports.ExpectedCsvContinuationError = ExpectedCsvContinuationError;
class ExpectedAnyTokenKindError extends Error {
    constructor(expectedAnyTokenKinds, foundToken, locale) {
        super(localization_1.Localization.error_parse_expectAnyTokenKind(localization_1.LocalizationUtils.getLocalizationTemplates(locale), expectedAnyTokenKinds, foundToken));
        this.expectedAnyTokenKinds = expectedAnyTokenKinds;
        this.foundToken = foundToken;
        Object.setPrototypeOf(this, ExpectedAnyTokenKindError.prototype);
    }
}
exports.ExpectedAnyTokenKindError = ExpectedAnyTokenKindError;
class ExpectedClosingTokenKind extends Error {
    constructor(expectedTokenKind, foundToken, locale) {
        super(localization_1.Localization.error_parse_expectCommaOrKind(localization_1.LocalizationUtils.getLocalizationTemplates(locale), expectedTokenKind));
        this.expectedTokenKind = expectedTokenKind;
        this.foundToken = foundToken;
        Object.setPrototypeOf(this, ExpectedClosingTokenKind.prototype);
    }
}
exports.ExpectedClosingTokenKind = ExpectedClosingTokenKind;
class ExpectedTokenKindError extends Error {
    constructor(expectedTokenKind, foundToken, locale) {
        super(localization_1.Localization.error_parse_expectTokenKind(localization_1.LocalizationUtils.getLocalizationTemplates(locale), expectedTokenKind, foundToken));
        this.expectedTokenKind = expectedTokenKind;
        this.foundToken = foundToken;
        Object.setPrototypeOf(this, ExpectedTokenKindError.prototype);
    }
}
exports.ExpectedTokenKindError = ExpectedTokenKindError;
class ExpectedGeneralizedIdentifierError extends Error {
    constructor(foundToken, locale) {
        super(localization_1.Localization.error_parse_expectGeneralizedIdentifier(localization_1.LocalizationUtils.getLocalizationTemplates(locale), foundToken));
        this.foundToken = foundToken;
        Object.setPrototypeOf(this, ExpectedGeneralizedIdentifierError.prototype);
    }
}
exports.ExpectedGeneralizedIdentifierError = ExpectedGeneralizedIdentifierError;
class InvalidCatchFunctionError extends Error {
    constructor(startToken, positionStart, locale) {
        super(localization_1.Localization.error_parse_invalidCatchFunction(localization_1.LocalizationUtils.getLocalizationTemplates(locale)));
        this.startToken = startToken;
        this.positionStart = positionStart;
        Object.setPrototypeOf(this, InvalidCatchFunctionError.prototype);
    }
}
exports.InvalidCatchFunctionError = InvalidCatchFunctionError;
class InvalidPrimitiveTypeError extends Error {
    constructor(token, positionStart, locale) {
        super(localization_1.Localization.error_parse_invalidPrimitiveType(localization_1.LocalizationUtils.getLocalizationTemplates(locale), token));
        this.token = token;
        this.positionStart = positionStart;
        Object.setPrototypeOf(this, InvalidPrimitiveTypeError.prototype);
    }
}
exports.InvalidPrimitiveTypeError = InvalidPrimitiveTypeError;
class RequiredParameterAfterOptionalParameterError extends Error {
    constructor(missingOptionalToken, positionStart, locale) {
        super(localization_1.Localization.error_parse_requiredParameterAfterOptional(localization_1.LocalizationUtils.getLocalizationTemplates(locale)));
        this.missingOptionalToken = missingOptionalToken;
        this.positionStart = positionStart;
        Object.setPrototypeOf(this, RequiredParameterAfterOptionalParameterError.prototype);
    }
}
exports.RequiredParameterAfterOptionalParameterError = RequiredParameterAfterOptionalParameterError;
class UnterminatedSequence extends Error {
    constructor(kind, startToken, positionStart, locale) {
        super(localization_1.Localization.error_parse_unterminated_sequence(localization_1.LocalizationUtils.getLocalizationTemplates(locale), kind));
        this.kind = kind;
        this.startToken = startToken;
        this.positionStart = positionStart;
        Object.setPrototypeOf(this, UnterminatedSequence.prototype);
    }
}
exports.UnterminatedSequence = UnterminatedSequence;
class UnusedTokensRemainError extends Error {
    constructor(firstUnusedToken, positionStart, locale) {
        super(localization_1.Localization.error_parse_unusedTokens(localization_1.LocalizationUtils.getLocalizationTemplates(locale)));
        this.firstUnusedToken = firstUnusedToken;
        this.positionStart = positionStart;
        Object.setPrototypeOf(this, UnusedTokensRemainError.prototype);
    }
}
exports.UnusedTokensRemainError = UnusedTokensRemainError;
function assertIsParseError(error) {
    common_1.Assert.isTrue(isParseError(error), "isParseError(error)");
    return true;
}
exports.assertIsParseError = assertIsParseError;
function isParseError(error) {
    return error instanceof ParseError;
}
exports.isParseError = isParseError;
function isTParseError(error) {
    return isParseError(error) || common_1.CommonError.isCommonError(error);
}
exports.isTParseError = isTParseError;
function isTInnerParseError(error) {
    return (error instanceof ExpectedAnyTokenKindError ||
        error instanceof ExpectedClosingTokenKind ||
        error instanceof ExpectedCsvContinuationError ||
        error instanceof ExpectedGeneralizedIdentifierError ||
        error instanceof ExpectedTokenKindError ||
        error instanceof InvalidCatchFunctionError ||
        error instanceof InvalidPrimitiveTypeError ||
        error instanceof RequiredParameterAfterOptionalParameterError ||
        error instanceof UnterminatedSequence ||
        error instanceof UnusedTokensRemainError);
}
exports.isTInnerParseError = isTInnerParseError;
function tokenFrom(error) {
    var _a;
    if (error instanceof ExpectedAnyTokenKindError ||
        error instanceof ExpectedClosingTokenKind ||
        error instanceof ExpectedCsvContinuationError ||
        error instanceof ExpectedGeneralizedIdentifierError ||
        error instanceof ExpectedTokenKindError) {
        return (_a = error.foundToken) === null || _a === void 0 ? void 0 : _a.token;
    }
    else if (error instanceof InvalidPrimitiveTypeError) {
        return error.token;
    }
    else if (error instanceof RequiredParameterAfterOptionalParameterError) {
        return error.missingOptionalToken;
    }
    else if (error instanceof UnterminatedSequence) {
        return error.startToken;
    }
    else if (error instanceof UnusedTokensRemainError) {
        return error.firstUnusedToken;
    }
    else if (error instanceof InvalidCatchFunctionError) {
        return undefined;
    }
    else {
        throw common_1.Assert.isNever(error);
    }
}
exports.tokenFrom = tokenFrom;
//# sourceMappingURL=error.js.map