"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.nthChecked = exports.nth = exports.indexOfNodeKind = exports.findNodeKind = exports.assertNthChecked = exports.assertNth = exports.assertAncestry = void 0;
const common_1 = require("../../common");
const _1 = require(".");
// Builds up an TXorNode path starting from nodeId and goes up to the root of the Ast.
function assertAncestry(nodeIdMapCollection, nodeId) {
    const ancestryIds = [nodeId];
    let parentId = nodeIdMapCollection.parentIdById.get(nodeId);
    while (parentId) {
        ancestryIds.push(parentId);
        parentId = nodeIdMapCollection.parentIdById.get(parentId);
    }
    return _1.NodeIdMapIterator.assertIterXor(nodeIdMapCollection, ancestryIds);
}
exports.assertAncestry = assertAncestry;
function assertNth(ancestry, ancestryIndex) {
    return common_1.Assert.asDefined(nth(ancestry, ancestryIndex), "ancestryIndex was out of bounds", {
        ancestryLength: ancestry.length,
        ancestryIndex,
    });
}
exports.assertNth = assertNth;
function assertNthChecked(ancestry, ancestryIndex, expectedNodeKinds) {
    const xorNode = assertNth(ancestry, ancestryIndex);
    _1.XorNodeUtils.assertIsNodeKind(xorNode, expectedNodeKinds);
    return xorNode;
}
exports.assertNthChecked = assertNthChecked;
function findNodeKind(ancestry, expectedNodeKinds) {
    for (const xorNode of ancestry) {
        if (_1.XorNodeUtils.isNodeKind(xorNode, expectedNodeKinds)) {
            return xorNode;
        }
    }
    return undefined;
}
exports.findNodeKind = findNodeKind;
function indexOfNodeKind(ancestry, expectedNodeKinds) {
    for (const [xorNode, index] of common_1.ArrayUtils.enumerate(ancestry)) {
        if (_1.XorNodeUtils.isNodeKind(xorNode, expectedNodeKinds)) {
            return index;
        }
    }
    return undefined;
}
exports.indexOfNodeKind = indexOfNodeKind;
function nth(ancestry, ancestryIndex) {
    return ancestry[ancestryIndex];
}
exports.nth = nth;
function nthChecked(ancestry, ancestryIndex, expectedNodeKinds) {
    const xorNode = nth(ancestry, ancestryIndex);
    return xorNode && _1.XorNodeUtils.isNodeKind(xorNode, expectedNodeKinds) ? xorNode : undefined;
}
exports.nthChecked = nthChecked;
//# sourceMappingURL=ancestryUtils.js.map