import { Ast, Constant } from "../../language";
import { NodeIdMap, TXorNode } from ".";
import { XorNode } from "./xorNode";
export declare type TKeyValuePair = FieldSpecificationKeyValuePair | LetKeyValuePair | RecordKeyValuePair | SectionKeyValuePair;
export interface IKeyValuePair<Key extends Ast.GeneralizedIdentifier | Ast.Identifier> {
    readonly source: TXorNode;
    readonly key: Key;
    readonly pairKind: PairKind;
    readonly keyLiteral: string;
    readonly normalizedKeyLiteral: string;
    readonly value: TXorNode | undefined;
}
export interface FieldSpecificationKeyValuePair extends IKeyValuePair<Ast.GeneralizedIdentifier> {
    readonly pairKind: PairKind.FieldSpecification;
    readonly optional: Ast.IConstant<Constant.LanguageConstant.Optional> | undefined;
}
export interface LetKeyValuePair extends IKeyValuePair<Ast.Identifier> {
    readonly pairKind: PairKind.LetExpression;
}
export interface RecordKeyValuePair extends IKeyValuePair<Ast.GeneralizedIdentifier> {
    readonly pairKind: PairKind.Record;
}
export interface SectionKeyValuePair extends IKeyValuePair<Ast.Identifier> {
    readonly pairKind: PairKind.SectionMember;
}
export declare enum PairKind {
    FieldSpecification = "RecordType",
    LetExpression = "LetExpression",
    Record = "Record",
    SectionMember = "Section"
}
export declare function assertIterChildrenAst(nodeIdMapCollection: NodeIdMap.Collection, parentId: number): ReadonlyArray<Ast.TNode>;
export declare function assertIterChildrenXor(nodeIdMapCollection: NodeIdMap.Collection, parentId: number): ReadonlyArray<TXorNode>;
export declare function assertIterXor(nodeIdMapCollection: NodeIdMap.Collection, nodeIds: ReadonlyArray<number>): ReadonlyArray<TXorNode>;
export declare function iterChildrenAst(nodeIdMapCollection: NodeIdMap.Collection, parentId: number): ReadonlyArray<Ast.TNode> | undefined;
export declare function nextSiblingXor(nodeIdMapCollection: NodeIdMap.Collection, nodeId: number): TXorNode | undefined;
export declare function nthSiblingXor(nodeIdMapCollection: NodeIdMap.Collection, nodeId: number, offset: number): TXorNode | undefined;
export declare function iterArrayWrapper(nodeIdMapCollection: NodeIdMap.Collection, arrayWrapper: TXorNode): ReadonlyArray<TXorNode>;
export declare function iterFieldProjection(nodeIdMapCollection: NodeIdMap.Collection, fieldProjection: TXorNode): ReadonlyArray<XorNode<Ast.FieldSelector>>;
export declare function iterFieldProjectionFieldLiterals(nodeIdMapCollection: NodeIdMap.Collection, fieldProjection: TXorNode): ReadonlyArray<string>;
export declare function iterFunctionExpressionParameters(nodeIdMapCollection: NodeIdMap.Collection, functionExpression: TXorNode): ReadonlyArray<XorNode<Ast.IParameter<Ast.AsNullablePrimitiveType | undefined>>>;
export declare function iterFunctionExpressionParameterNames(nodeIdMapCollection: NodeIdMap.Collection, functionExpression: TXorNode): ReadonlyArray<Ast.Identifier>;
export declare function iterFunctionExpressionParameterNameLiterals(nodeIdMapCollection: NodeIdMap.Collection, functionExpression: TXorNode): ReadonlyArray<string>;
export declare function iterFieldSpecificationList(nodeIdMapCollection: NodeIdMap.Collection, fieldSpecificationList: TXorNode): ReadonlyArray<FieldSpecificationKeyValuePair>;
export declare function iterInvokeExpression(nodeIdMapCollection: NodeIdMap.Collection, invokeExpression: TXorNode): ReadonlyArray<TXorNode>;
export declare function iterLetExpression(nodeIdMapCollection: NodeIdMap.Collection, letExpression: TXorNode): ReadonlyArray<LetKeyValuePair>;
export declare function iterListItems(nodeIdMapCollection: NodeIdMap.Collection, list: TXorNode): ReadonlyArray<TXorNode>;
export declare function iterRecord(nodeIdMapCollection: NodeIdMap.Collection, record: TXorNode): ReadonlyArray<RecordKeyValuePair>;
export declare function iterRecordType(nodeIdMapCollection: NodeIdMap.Collection, recordType: TXorNode): ReadonlyArray<FieldSpecificationKeyValuePair>;
export declare function iterSection(nodeIdMapCollection: NodeIdMap.Collection, section: TXorNode): ReadonlyArray<SectionKeyValuePair>;
