"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.iterSection = exports.iterRecordType = exports.iterRecord = exports.iterListItems = exports.iterLetExpression = exports.iterInvokeExpression = exports.iterFieldSpecificationList = exports.iterFunctionExpressionParameterNameLiterals = exports.iterFunctionExpressionParameterNames = exports.iterFunctionExpressionParameters = exports.iterFieldProjectionFieldLiterals = exports.iterFieldProjection = exports.iterArrayWrapper = exports.nthSiblingXor = exports.nextSiblingXor = exports.iterChildrenAst = exports.assertIterXor = exports.assertIterChildrenXor = exports.assertIterChildrenAst = exports.PairKind = void 0;
const language_1 = require("../../language");
const _1 = require(".");
const common_1 = require("../../common");
const nodeIdMapUtils_1 = require("./nodeIdMapUtils");
var PairKind;
(function (PairKind) {
    PairKind["FieldSpecification"] = "RecordType";
    PairKind["LetExpression"] = "LetExpression";
    PairKind["Record"] = "Record";
    PairKind["SectionMember"] = "Section";
})(PairKind = exports.PairKind || (exports.PairKind = {}));
// -------------------------------
// -------- Simple iters  --------
// -------------------------------
// Assert the existence of children for the node and that they are Ast nodes.
// Returns an array of children (which are TNodes) for the given node.
function assertIterChildrenAst(nodeIdMapCollection, parentId) {
    return _1.NodeIdMapUtils.assertChildIds(nodeIdMapCollection.childIdsById, parentId).map((childId) => _1.NodeIdMapUtils.assertAst(nodeIdMapCollection, childId));
}
exports.assertIterChildrenAst = assertIterChildrenAst;
// Assert the existence of children for the node.
// Returns an array of children (as XorNodes) for the given node.
function assertIterChildrenXor(nodeIdMapCollection, parentId) {
    const childIds = nodeIdMapCollection.childIdsById.get(parentId);
    if (childIds === undefined) {
        return [];
    }
    return assertIterXor(nodeIdMapCollection, childIds);
}
exports.assertIterChildrenXor = assertIterChildrenXor;
// Given a list of nodeIds, assert the existence of then return them as XorNodes.
function assertIterXor(nodeIdMapCollection, nodeIds) {
    return nodeIds.map((nodeId) => _1.NodeIdMapUtils.assertXor(nodeIdMapCollection, nodeId));
}
exports.assertIterXor = assertIterXor;
// If any exist, returns all Ast nodes under the given node.
function iterChildrenAst(nodeIdMapCollection, parentId) {
    const childIds = nodeIdMapCollection.childIdsById.get(parentId);
    if (childIds === undefined) {
        return undefined;
    }
    return childIds.map((childId) => _1.NodeIdMapUtils.assertAst(nodeIdMapCollection, childId));
}
exports.iterChildrenAst = iterChildrenAst;
function nextSiblingXor(nodeIdMapCollection, nodeId) {
    return nthSiblingXor(nodeIdMapCollection, nodeId, 1);
}
exports.nextSiblingXor = nextSiblingXor;
// Grabs the parent for the given nodeId, then returns the nth child of the parent where that child's attributeIndex is
// (givenNode.attributeIndex + offset) as an XorNode if such a child exists.
function nthSiblingXor(nodeIdMapCollection, nodeId, offset) {
    const childXorNode = _1.NodeIdMapUtils.assertXor(nodeIdMapCollection, nodeId);
    if (childXorNode.node.attributeIndex === undefined) {
        return undefined;
    }
    const attributeIndex = childXorNode.node.attributeIndex + offset;
    if (attributeIndex < 0) {
        return undefined;
    }
    const parentXorNode = _1.NodeIdMapUtils.assertParentXor(nodeIdMapCollection, nodeId);
    const childIds = _1.NodeIdMapUtils.assertChildIds(nodeIdMapCollection.childIdsById, parentXorNode.node.id);
    if (childIds.length >= attributeIndex) {
        return undefined;
    }
    return _1.NodeIdMapUtils.xor(nodeIdMapCollection, childIds[attributeIndex]);
}
exports.nthSiblingXor = nthSiblingXor;
// ------------------------------------------
// -------- NodeKind Specific Iters  --------
// ------------------------------------------
// Iterates over Ast.TCsv.node
function iterArrayWrapper(nodeIdMapCollection, arrayWrapper) {
    _1.XorNodeUtils.assertIsNodeKind(arrayWrapper, language_1.Ast.NodeKind.ArrayWrapper);
    if (_1.XorNodeUtils.isAst(arrayWrapper)) {
        return arrayWrapper.node.elements.map((wrapper) => _1.XorNodeUtils.boxAst(wrapper.node));
    }
    const partial = [];
    for (const csvXorNode of assertIterChildrenXor(nodeIdMapCollection, arrayWrapper.node.id)) {
        switch (csvXorNode.kind) {
            case _1.XorNodeKind.Ast:
                partial.push(_1.XorNodeUtils.boxAst(csvXorNode.node.node));
                break;
            case _1.XorNodeKind.Context: {
                const child = _1.NodeIdMapUtils.nthChildXor(nodeIdMapCollection, csvXorNode.node.id, 0);
                if (child !== undefined) {
                    partial.push(child);
                }
                break;
            }
            default:
                throw common_1.Assert.isNever(csvXorNode);
        }
    }
    return partial;
}
exports.iterArrayWrapper = iterArrayWrapper;
// Return all FieldSelector children under the given FieldProjection.
function iterFieldProjection(nodeIdMapCollection, fieldProjection) {
    _1.XorNodeUtils.assertIsNodeKind(fieldProjection, language_1.Ast.NodeKind.FieldProjection);
    const arrayWrapper = _1.NodeIdMapUtils.arrayWrapperContentXor(nodeIdMapCollection, fieldProjection.node.id);
    if (!arrayWrapper) {
        return [];
    }
    const result = [];
    for (const child of iterArrayWrapper(nodeIdMapCollection, arrayWrapper)) {
        _1.XorNodeUtils.assertIsNodeKind(child, language_1.Ast.NodeKind.FieldSelector);
        result.push(child);
    }
    return result;
}
exports.iterFieldProjection = iterFieldProjection;
// Return all FieldSelector names under the given FieldProjection.
function iterFieldProjectionFieldLiterals(nodeIdMapCollection, fieldProjection) {
    const result = [];
    for (const selector of iterFieldProjection(nodeIdMapCollection, fieldProjection)) {
        const identifier = _1.NodeIdMapUtils.wrappedContentXorChecked(nodeIdMapCollection, selector.node.id, language_1.Ast.NodeKind.GeneralizedIdentifier);
        if (identifier && _1.XorNodeUtils.isAst(identifier)) {
            result.push(identifier.node.literal);
        }
    }
    return result;
}
exports.iterFieldProjectionFieldLiterals = iterFieldProjectionFieldLiterals;
function iterFunctionExpressionParameters(nodeIdMapCollection, functionExpression) {
    _1.XorNodeUtils.assertIsNodeKind(functionExpression, language_1.Ast.NodeKind.FunctionExpression);
    if (_1.XorNodeUtils.isAstChecked(functionExpression, language_1.Ast.NodeKind.FunctionExpression)) {
        return functionExpression.node.parameters.content.elements.map((parameter) => _1.XorNodeUtils.boxAst(parameter.node));
    }
    const parameterList = _1.NodeIdMapUtils.nthChildXorChecked(nodeIdMapCollection, functionExpression.node.id, 0, language_1.Ast.NodeKind.ParameterList);
    if (parameterList === undefined) {
        return [];
    }
    const result = [];
    for (const xorNode of iterArrayWrapperInWrappedContent(nodeIdMapCollection, parameterList)) {
        _1.XorNodeUtils.assertIsNodeKind(xorNode, language_1.Ast.NodeKind.Parameter);
        result.push(xorNode);
    }
    return result;
}
exports.iterFunctionExpressionParameters = iterFunctionExpressionParameters;
function iterFunctionExpressionParameterNames(nodeIdMapCollection, functionExpression) {
    const result = [];
    for (const parameter of iterFunctionExpressionParameters(nodeIdMapCollection, functionExpression)) {
        const name = (0, nodeIdMapUtils_1.parameterIdentifier)(nodeIdMapCollection, parameter);
        if (name === undefined) {
            break;
        }
        result.push(name);
    }
    return result;
}
exports.iterFunctionExpressionParameterNames = iterFunctionExpressionParameterNames;
function iterFunctionExpressionParameterNameLiterals(nodeIdMapCollection, functionExpression) {
    return iterFunctionExpressionParameterNames(nodeIdMapCollection, functionExpression).map((identifier) => identifier.literal);
}
exports.iterFunctionExpressionParameterNameLiterals = iterFunctionExpressionParameterNameLiterals;
// Return all FieldSpecification children under the given FieldSpecificationList.
function iterFieldSpecificationList(nodeIdMapCollection, fieldSpecificationList) {
    _1.XorNodeUtils.assertIsNodeKind(fieldSpecificationList, language_1.Ast.NodeKind.FieldSpecificationList);
    const result = [];
    for (const fieldSpecification of iterArrayWrapperInWrappedContent(nodeIdMapCollection, fieldSpecificationList)) {
        const key = _1.NodeIdMapUtils.nthChildAstChecked(nodeIdMapCollection, fieldSpecification.node.id, 1, language_1.Ast.NodeKind.GeneralizedIdentifier);
        if (key === undefined) {
            break;
        }
        const optional = _1.NodeIdMapUtils.nthChildAstChecked(nodeIdMapCollection, fieldSpecification.node.id, 0, language_1.Ast.NodeKind.Constant);
        const value = _1.NodeIdMapUtils.nthChildXorChecked(nodeIdMapCollection, fieldSpecification.node.id, 2, language_1.Ast.NodeKind.FieldSpecification);
        const keyLiteral = key.literal;
        result.push({
            key,
            keyLiteral,
            optional,
            value,
            normalizedKeyLiteral: language_1.IdentifierUtils.normalizeIdentifier(keyLiteral),
            pairKind: PairKind.FieldSpecification,
            source: fieldSpecification,
        });
    }
    return result;
}
exports.iterFieldSpecificationList = iterFieldSpecificationList;
function iterInvokeExpression(nodeIdMapCollection, invokeExpression) {
    _1.XorNodeUtils.assertIsNodeKind(invokeExpression, language_1.Ast.NodeKind.InvokeExpression);
    return iterArrayWrapperInWrappedContent(nodeIdMapCollection, invokeExpression);
}
exports.iterInvokeExpression = iterInvokeExpression;
// Return all key-value-pair children under the given LetExpression.
function iterLetExpression(nodeIdMapCollection, letExpression) {
    _1.XorNodeUtils.assertIsNodeKind(letExpression, language_1.Ast.NodeKind.LetExpression);
    const arrayWrapper = _1.NodeIdMapUtils.arrayWrapperContentXor(nodeIdMapCollection, letExpression.node.id);
    if (arrayWrapper === undefined) {
        return [];
    }
    return iterKeyValuePairs(nodeIdMapCollection, arrayWrapper, PairKind.LetExpression);
}
exports.iterLetExpression = iterLetExpression;
// Return all ListItem children under the given ListExpression/ListLiteral.
function iterListItems(nodeIdMapCollection, list) {
    _1.XorNodeUtils.assertIsList(list);
    return iterArrayWrapperInWrappedContent(nodeIdMapCollection, list);
}
exports.iterListItems = iterListItems;
// Return all key-value-pair children under the given RecordExpression/RecordLiteral.
function iterRecord(nodeIdMapCollection, record) {
    _1.XorNodeUtils.assertIsRecord(record);
    const arrayWrapper = _1.NodeIdMapUtils.arrayWrapperContentXor(nodeIdMapCollection, record.node.id);
    if (arrayWrapper === undefined) {
        return [];
    }
    return iterKeyValuePairs(nodeIdMapCollection, arrayWrapper, PairKind.Record);
}
exports.iterRecord = iterRecord;
function iterRecordType(nodeIdMapCollection, recordType) {
    _1.XorNodeUtils.assertIsNodeKind(recordType, language_1.Ast.NodeKind.RecordType);
    const fields = _1.NodeIdMapUtils.nthChildXorChecked(nodeIdMapCollection, recordType.node.id, 0, language_1.Ast.NodeKind.FieldSpecificationList);
    if (fields === undefined) {
        return [];
    }
    return iterFieldSpecificationList(nodeIdMapCollection, fields);
}
exports.iterRecordType = iterRecordType;
// Return all key-value-pair children under the given Section.
function iterSection(nodeIdMapCollection, section) {
    _1.XorNodeUtils.assertIsNodeKind(section, language_1.Ast.NodeKind.Section);
    if (_1.XorNodeUtils.isAstChecked(section, language_1.Ast.NodeKind.Section)) {
        return section.node.sectionMembers.elements.map((sectionMember) => {
            const namePairedExpression = sectionMember.namePairedExpression;
            const keyLiteral = namePairedExpression.key.literal;
            return {
                source: _1.XorNodeUtils.boxAst(namePairedExpression),
                key: namePairedExpression.key,
                keyLiteral,
                normalizedKeyLiteral: language_1.IdentifierUtils.normalizeIdentifier(keyLiteral),
                value: _1.XorNodeUtils.boxAst(namePairedExpression.value),
                pairKind: PairKind.SectionMember,
            };
        });
    }
    const sectionMemberArrayWrapper = _1.NodeIdMapUtils.nthChildXorChecked(nodeIdMapCollection, section.node.id, 4, language_1.Ast.NodeKind.ArrayWrapper);
    if (sectionMemberArrayWrapper === undefined) {
        return [];
    }
    const partial = [];
    for (const sectionMember of assertIterChildrenXor(nodeIdMapCollection, sectionMemberArrayWrapper.node.id)) {
        const keyValuePair = _1.NodeIdMapUtils.nthChildXorChecked(nodeIdMapCollection, sectionMember.node.id, 2, language_1.Ast.NodeKind.IdentifierPairedExpression);
        if (keyValuePair === undefined) {
            continue;
        }
        const keyValuePairNodeId = keyValuePair.node.id;
        const keyKey = _1.NodeIdMapUtils.nthChildAstChecked(nodeIdMapCollection, keyValuePair.node.id, 0, language_1.Ast.NodeKind.Identifier);
        if (keyKey === undefined) {
            continue;
        }
        const key = keyKey;
        const keyLiteral = key.literal;
        partial.push({
            source: keyValuePair,
            key,
            keyLiteral,
            normalizedKeyLiteral: language_1.IdentifierUtils.normalizeIdentifier(keyLiteral),
            value: _1.NodeIdMapUtils.nthChildXor(nodeIdMapCollection, keyValuePairNodeId, 2),
            pairKind: PairKind.SectionMember,
        });
    }
    return partial;
}
exports.iterSection = iterSection;
function iterKeyValuePairs(nodeIdMapCollection, arrayWrapper, pairKind) {
    const partial = [];
    for (const keyValuePair of iterArrayWrapper(nodeIdMapCollection, arrayWrapper)) {
        const key = _1.NodeIdMapUtils.nthChildAstChecked(nodeIdMapCollection, keyValuePair.node.id, 0, [
            language_1.Ast.NodeKind.GeneralizedIdentifier,
            language_1.Ast.NodeKind.Identifier,
        ]);
        if (key === undefined) {
            break;
        }
        const keyLiteral = key.literal;
        partial.push({
            source: keyValuePair,
            key,
            keyLiteral,
            normalizedKeyLiteral: language_1.IdentifierUtils.normalizeIdentifier(keyLiteral),
            value: _1.NodeIdMapUtils.nthChildXor(nodeIdMapCollection, keyValuePair.node.id, 2),
            pairKind,
        });
    }
    return partial;
}
function iterArrayWrapperInWrappedContent(nodeIdMapCollection, xorNode) {
    const arrayWrapper = _1.NodeIdMapUtils.arrayWrapperContentXor(nodeIdMapCollection, xorNode.node.id);
    if (arrayWrapper === undefined) {
        return [];
    }
    return iterArrayWrapper(nodeIdMapCollection, arrayWrapper);
}
//# sourceMappingURL=nodeIdMapIterator.js.map