"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.wrappedContentAstChecked = exports.wrappedContentAst = exports.wrappedContentXorChecked = exports.wrappedContentXor = exports.arrayWrapperContentXor = exports.nthChildContextChecked = exports.nthChildContext = exports.nthChildAstChecked = exports.nthChildAst = exports.nthChildXorChecked = exports.nthChildXor = exports.assertArrayWrapperContentAst = exports.assertNthChildContextChecked = exports.assertNthChildContext = exports.assertNthChildAstChecked = exports.assertNthChildAst = exports.assertNthChildXorChecked = exports.assertNthChildXor = exports.assertChildIds = void 0;
const commonSelectors_1 = require("./commonSelectors");
const language_1 = require("../../../language");
const context_1 = require("../../context");
const common_1 = require("../../../common");
const __1 = require("..");
// You can think of a node as a collection which holds other nodes.
// The ArithmeticExpression `1 + 2` has three nodes (i.e. attributes):
//  * a literal node `1` as the first attribute.
//  * a literal operator constant `+` as the second attribute.
//  * a literal node `2` as the third attribute.
//
// The `INode` interface has the nullable field `attributeIndex`.
// A truthy value indicates it contains a parent and if so what attribute number it is under the parent.
function assertChildIds(childIdsById, parentId) {
    return common_1.Assert.asDefined(childIdsById.get(parentId), `parentId doesn't have any children`, { parentId });
}
exports.assertChildIds = assertChildIds;
function assertNthChildXor(nodeIdMapCollection, parentId, attributeIndex) {
    return common_1.Assert.asDefined(nthChildXor(nodeIdMapCollection, parentId, attributeIndex), `parentId doesn't have a child at the given index`, {
        parentId,
        attributeIndex,
    });
}
exports.assertNthChildXor = assertNthChildXor;
function assertNthChildXorChecked(nodeIdMapCollection, parentId, attributeIndex, expectedNodeKinds) {
    return __1.XorNodeUtils.assertAsNodeKind(assertNthChildXor(nodeIdMapCollection, parentId, attributeIndex), expectedNodeKinds);
}
exports.assertNthChildXorChecked = assertNthChildXorChecked;
function assertNthChildAst(nodeIdMapCollection, parentId, attributeIndex) {
    return common_1.Assert.asDefined(nthChildAst(nodeIdMapCollection, parentId, attributeIndex), `parentId doesn't have an Ast child at the given index`, {
        parentId,
        attributeIndex,
    });
}
exports.assertNthChildAst = assertNthChildAst;
function assertNthChildAstChecked(nodeIdMapCollection, parentId, attributeIndex, expectedNodeKinds) {
    return language_1.AstUtils.assertAsNodeKind(assertNthChildAst(nodeIdMapCollection, parentId, attributeIndex), expectedNodeKinds);
}
exports.assertNthChildAstChecked = assertNthChildAstChecked;
function assertNthChildContext(nodeIdMapCollection, parentId, attributeIndex) {
    return common_1.Assert.asDefined(nthChildContext(nodeIdMapCollection, parentId, attributeIndex), `parentId doesn't have a ParseContext child at the given index`, {
        parentId,
        attributeIndex,
    });
}
exports.assertNthChildContext = assertNthChildContext;
function assertNthChildContextChecked(nodeIdMapCollection, parentId, attributeIndex, expectedNodeKinds) {
    return context_1.ParseContextUtils.assertAsNodeKind(assertNthChildContext(nodeIdMapCollection, parentId, attributeIndex), expectedNodeKinds);
}
exports.assertNthChildContextChecked = assertNthChildContextChecked;
function assertArrayWrapperContentAst(nodeIdMapCollection, nodeId) {
    const xorNode = common_1.Assert.asDefined(arrayWrapperContentXor(nodeIdMapCollection, nodeId), "failure in assertArrayWrapperContentAst", {
        nodeId,
    });
    __1.XorNodeUtils.assertIsAst(xorNode);
    return xorNode.node;
}
exports.assertArrayWrapperContentAst = assertArrayWrapperContentAst;
function nthChildXor(nodeIdMapCollection, parentId, attributeIndex) {
    // Grab the node's childIds.
    const childIds = nodeIdMapCollection.childIdsById.get(parentId);
    if (childIds === undefined) {
        return undefined;
    }
    // Iterate over the children and try to find one which matches attributeIndex.
    for (const childId of childIds) {
        const xorNode = (0, commonSelectors_1.assertXor)(nodeIdMapCollection, childId);
        if (xorNode.node.attributeIndex === attributeIndex) {
            return xorNode;
        }
    }
    return undefined;
}
exports.nthChildXor = nthChildXor;
function nthChildXorChecked(nodeIdMapCollection, parentId, attributeIndex, expectedNodeKinds) {
    const xorNode = nthChildXor(nodeIdMapCollection, parentId, attributeIndex);
    return xorNode && __1.XorNodeUtils.isNodeKind(xorNode, expectedNodeKinds) ? xorNode : undefined;
}
exports.nthChildXorChecked = nthChildXorChecked;
function nthChildAst(nodeIdMapCollection, parentId, attributeIndex) {
    const xorNode = nthChildXor(nodeIdMapCollection, parentId, attributeIndex);
    return xorNode && __1.XorNodeUtils.isAst(xorNode) ? xorNode.node : undefined;
}
exports.nthChildAst = nthChildAst;
function nthChildAstChecked(nodeIdMapCollection, parentId, attributeIndex, expectedNodeKinds) {
    const astNode = nthChildAst(nodeIdMapCollection, parentId, attributeIndex);
    return astNode && language_1.AstUtils.isNodeKind(astNode, expectedNodeKinds) ? astNode : undefined;
}
exports.nthChildAstChecked = nthChildAstChecked;
function nthChildContext(nodeIdMapCollection, parentId, attributeIndex) {
    const xorNode = nthChildXor(nodeIdMapCollection, parentId, attributeIndex);
    return xorNode && __1.XorNodeUtils.isContext(xorNode) ? xorNode.node : undefined;
}
exports.nthChildContext = nthChildContext;
function nthChildContextChecked(nodeIdMapCollection, parentId, attributeIndex, expectedNodeKinds) {
    const contextNode = nthChildContext(nodeIdMapCollection, parentId, attributeIndex);
    return contextNode && context_1.ParseContextUtils.isNodeKind(contextNode, expectedNodeKinds) ? contextNode : undefined;
}
exports.nthChildContextChecked = nthChildContextChecked;
function arrayWrapperContentXor(nodeIdMapCollection, nodeId) {
    return nthChildXorChecked(nodeIdMapCollection, nodeId, 1, language_1.Ast.NodeKind.ArrayWrapper);
}
exports.arrayWrapperContentXor = arrayWrapperContentXor;
function wrappedContentXor(nodeIdMapCollection, nodeId) {
    const wrapperXorNode = (0, commonSelectors_1.xor)(nodeIdMapCollection, nodeId);
    if (wrapperXorNode === undefined || !__1.XorNodeUtils.isTWrapped(wrapperXorNode)) {
        return undefined;
    }
    return nthChildXor(nodeIdMapCollection, nodeId, 1);
}
exports.wrappedContentXor = wrappedContentXor;
function wrappedContentXorChecked(nodeIdMapCollection, nodeId, expectedNodeKinds) {
    const xorNode = wrappedContentXor(nodeIdMapCollection, nodeId);
    return xorNode && __1.XorNodeUtils.isNodeKind(xorNode, expectedNodeKinds) ? xorNode : undefined;
}
exports.wrappedContentXorChecked = wrappedContentXorChecked;
function wrappedContentAst(nodeIdMapCollection, nodeId) {
    const xorNode = wrappedContentXor(nodeIdMapCollection, nodeId);
    return xorNode && __1.XorNodeUtils.isAst(xorNode) ? xorNode.node : undefined;
}
exports.wrappedContentAst = wrappedContentAst;
function wrappedContentAstChecked(nodeIdMapCollection, nodeId, expectedNodeKinds) {
    const astNode = wrappedContentAst(nodeIdMapCollection, nodeId);
    return astNode && language_1.AstUtils.isNodeKind(astNode, expectedNodeKinds) ? astNode : undefined;
}
exports.wrappedContentAstChecked = wrappedContentAstChecked;
//# sourceMappingURL=childSelectors.js.map