"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.contextChecked = exports.context = exports.astChecked = exports.ast = exports.xorChecked = exports.xor = exports.assertContextChecked = exports.assertContext = exports.assertAstChecked = exports.assertAst = exports.assertXorChecked = exports.assertXor = void 0;
const language_1 = require("../../../language");
const context_1 = require("../../context");
const common_1 = require("../../../common");
const __1 = require("..");
function assertXor(nodeIdMapCollection, nodeId) {
    return common_1.Assert.asDefined(xor(nodeIdMapCollection, nodeId), "failed to find the expected node", { nodeId });
}
exports.assertXor = assertXor;
function assertXorChecked(nodeIdMapCollection, nodeId, expectedNodeKinds) {
    return __1.XorNodeUtils.assertAsNodeKind(assertXor(nodeIdMapCollection, nodeId), expectedNodeKinds);
}
exports.assertXorChecked = assertXorChecked;
function assertAst(nodeIdMapCollection, nodeId) {
    return common_1.Assert.asDefined(ast(nodeIdMapCollection, nodeId), "failed to find the expected Ast node", {
        nodeId,
    });
}
exports.assertAst = assertAst;
function assertAstChecked(nodeIdMapCollection, nodeId, expectedNodeKinds) {
    return language_1.AstUtils.assertAsNodeKind(assertAst(nodeIdMapCollection, nodeId), expectedNodeKinds);
}
exports.assertAstChecked = assertAstChecked;
function assertContext(nodeIdMapCollection, nodeId) {
    return common_1.Assert.asDefined(context(nodeIdMapCollection, nodeId), "failed to find the expected Context node", {
        nodeId,
    });
}
exports.assertContext = assertContext;
function assertContextChecked(nodeIdMapCollection, nodeId, expectedNodeKinds) {
    return context_1.ParseContextUtils.assertAsNodeKind(assertContext(nodeIdMapCollection, nodeId), expectedNodeKinds);
}
exports.assertContextChecked = assertContextChecked;
function xor(nodeIdMapCollection, nodeId) {
    const astNode = nodeIdMapCollection.astNodeById.get(nodeId);
    if (astNode !== undefined) {
        return __1.XorNodeUtils.boxAst(astNode);
    }
    const contextNode = nodeIdMapCollection.contextNodeById.get(nodeId);
    if (contextNode !== undefined) {
        return __1.XorNodeUtils.boxContext(contextNode);
    }
    return undefined;
}
exports.xor = xor;
function xorChecked(nodeIdMapCollection, nodeId, expectedNodeKinds) {
    const xorNode = xor(nodeIdMapCollection, nodeId);
    return xorNode && __1.XorNodeUtils.isNodeKind(xorNode, expectedNodeKinds) ? xorNode : undefined;
}
exports.xorChecked = xorChecked;
function ast(nodeIdMapCollection, nodeId) {
    const xorNode = xor(nodeIdMapCollection, nodeId);
    return xorNode && __1.XorNodeUtils.isAst(xorNode) ? xorNode.node : undefined;
}
exports.ast = ast;
function astChecked(nodeIdMapCollection, nodeId, expectedNodeKinds) {
    const astNode = ast(nodeIdMapCollection, nodeId);
    return astNode && language_1.AstUtils.isNodeKind(astNode, expectedNodeKinds) ? astNode : undefined;
}
exports.astChecked = astChecked;
function context(nodeIdMapCollection, nodeId) {
    const xorNode = xor(nodeIdMapCollection, nodeId);
    return xorNode && __1.XorNodeUtils.isContext(xorNode) ? xorNode.node : undefined;
}
exports.context = context;
function contextChecked(nodeIdMapCollection, nodeId, expectedNodeKinds) {
    const contextNode = context(nodeIdMapCollection, nodeId);
    return contextNode && context_1.ParseContextUtils.isNodeKind(contextNode, expectedNodeKinds) ? contextNode : undefined;
}
exports.contextChecked = contextChecked;
//# sourceMappingURL=commonSelectors.js.map