"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.rightMostLeafWhere = exports.rightMostLeaf = exports.leftMostLeaf = exports.leftMostXor = exports.assertLeftMostXor = exports.assertLeftMostLeaf = void 0;
const __1 = require("..");
const common_1 = require("../../../common");
const commonSelectors_1 = require("./commonSelectors");
// As it's a leaf it's expected to be a Ast.TNode.
function assertLeftMostLeaf(nodeIdMapCollection, nodeId) {
    return __1.XorNodeUtils.assertAst(common_1.Assert.asDefined(leftMostXor(nodeIdMapCollection, nodeId), `nodeId does not exist in nodeIdMapCollection`, {
        nodeId,
    }));
}
exports.assertLeftMostLeaf = assertLeftMostLeaf;
function assertLeftMostXor(nodeIdMapCollection, nodeId) {
    return common_1.Assert.asDefined(leftMostXor(nodeIdMapCollection, nodeId), `nodeId does not exist in nodeIdMapCollection`, {
        nodeId,
    });
}
exports.assertLeftMostXor = assertLeftMostXor;
// Travels down the left most node under the given nodeId by way of the children collection.
function leftMostXor(nodeIdMapCollection, nodeId) {
    const currentNode = (0, commonSelectors_1.xor)(nodeIdMapCollection, nodeId);
    if (currentNode === undefined) {
        return undefined;
    }
    let currentNodeId = currentNode.node.id;
    let childIds = nodeIdMapCollection.childIdsById.get(currentNodeId);
    while (childIds === null || childIds === void 0 ? void 0 : childIds.length) {
        currentNodeId = childIds[0];
        childIds = nodeIdMapCollection.childIdsById.get(currentNodeId);
    }
    return (0, commonSelectors_1.xor)(nodeIdMapCollection, currentNodeId);
}
exports.leftMostXor = leftMostXor;
// Same as leftMostXor but also checks if it's an Ast node.
function leftMostLeaf(nodeIdMapCollection, nodeId) {
    const xorNode = leftMostXor(nodeIdMapCollection, nodeId);
    return xorNode && __1.XorNodeUtils.isAst(xorNode) ? xorNode.node : undefined;
}
exports.leftMostLeaf = leftMostLeaf;
// There are a few assumed invariants about children:
//  * Children were read left to right.
//  * Children were placed in childIdsById in the order they were read.
//  * Therefore the right-most child is the most recently read which also appears last in the document.
function rightMostLeaf(nodeIdMapCollection, nodeId, predicate = undefined) {
    const astNodeById = nodeIdMapCollection.astNodeById;
    let nodeIdsToExplore = [nodeId];
    let rightMost;
    while (nodeIdsToExplore.length) {
        const nodeId = common_1.Assert.asDefined(nodeIdsToExplore.pop());
        const astNode = astNodeById.get(nodeId);
        let addChildren = false;
        // Check if Ast.TNode or ParserContext.Node
        if (astNode !== undefined) {
            if (predicate && !predicate(astNode)) {
                continue;
            }
            // Is leaf, check if it's more right than the previous record.
            // As it's a leaf there are no children to add.
            if (astNode.isLeaf) {
                // Is the first leaf encountered.
                if (rightMost === undefined) {
                    rightMost = astNode;
                }
                // Compare current leaf node to the existing record.
                else if (astNode.tokenRange.tokenIndexStart > rightMost.tokenRange.tokenIndexStart) {
                    rightMost = astNode;
                }
            }
            // Is not a leaf, no previous record exists.
            // Add all children to the queue.
            else if (rightMost === undefined) {
                addChildren = true;
            }
            // Is not a leaf, previous record exists.
            // Check if we can cull the branch, otherwise add all children to the queue.
            else if (astNode.tokenRange.tokenIndexEnd > rightMost.tokenRange.tokenIndexStart) {
                addChildren = true;
            }
        }
        // Must be a ParserContext.Node.
        // Add all children to the queue as ParserContext.Nodes can have Ast children which are leafs.
        else {
            addChildren = true;
        }
        if (addChildren) {
            const childIds = nodeIdMapCollection.childIdsById.get(nodeId);
            if (childIds !== undefined) {
                // Add the child ids in reversed order to prioritize visiting the right most nodes first.
                const reversedChildIds = [...childIds];
                reversedChildIds.reverse();
                nodeIdsToExplore = [...reversedChildIds, ...nodeIdsToExplore];
            }
        }
    }
    return Promise.resolve(rightMost);
}
exports.rightMostLeaf = rightMostLeaf;
function rightMostLeafWhere(nodeIdMapCollection, nodeId, predicateFn) {
    return rightMostLeaf(nodeIdMapCollection, nodeId, predicateFn);
}
exports.rightMostLeafWhere = rightMostLeafWhere;
//# sourceMappingURL=leafSelectors.js.map