"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.parentContextChecked = exports.parentContext = exports.parentAstChecked = exports.parentAst = exports.parentXorChecked = exports.parentXor = exports.assertParentContextChecked = exports.assertParentContext = exports.assertParentAstChecked = exports.assertParentAst = exports.assertParentXorChecked = exports.assertParentXor = void 0;
const language_1 = require("../../../language");
const xorNodeUtils_1 = require("../xorNodeUtils");
const context_1 = require("../../context");
const common_1 = require("../../../common");
const __1 = require("..");
function assertParentXor(nodeIdMapCollection, nodeId) {
    return common_1.Assert.asDefined(parentXor(nodeIdMapCollection, nodeId), `nodeId doesn't have a parent`, {
        nodeId,
    });
}
exports.assertParentXor = assertParentXor;
function assertParentXorChecked(nodeIdMapCollection, nodeId, expectedNodeKinds) {
    return __1.XorNodeUtils.assertAsNodeKind(assertParentXor(nodeIdMapCollection, nodeId), expectedNodeKinds);
}
exports.assertParentXorChecked = assertParentXorChecked;
function assertParentAst(nodeIdMapCollection, nodeId) {
    return common_1.Assert.asDefined(parentAst(nodeIdMapCollection, nodeId), "couldn't find the expected parent Ast for nodeId", {
        nodeId,
    });
}
exports.assertParentAst = assertParentAst;
function assertParentAstChecked(nodeIdMapCollection, nodeId, expectedNodeKinds) {
    return language_1.AstUtils.assertAsNodeKind(assertParentAst(nodeIdMapCollection, nodeId), expectedNodeKinds);
}
exports.assertParentAstChecked = assertParentAstChecked;
function assertParentContext(nodeIdMapCollection, nodeId) {
    return common_1.Assert.asDefined(parentContext(nodeIdMapCollection, nodeId), "couldn't find the expected parent ParseContext for nodeId", {
        nodeId,
    });
}
exports.assertParentContext = assertParentContext;
function assertParentContextChecked(nodeIdMapCollection, nodeId, expectedNodeKinds) {
    return context_1.ParseContextUtils.assertAsNodeKind(assertParentContext(nodeIdMapCollection, nodeId), expectedNodeKinds);
}
exports.assertParentContextChecked = assertParentContextChecked;
function parentXor(nodeIdMapCollection, childId) {
    const astNode = parentAst(nodeIdMapCollection, childId);
    if (astNode !== undefined) {
        return (0, xorNodeUtils_1.boxAst)(astNode);
    }
    const context = parentContext(nodeIdMapCollection, childId);
    if (context !== undefined) {
        return (0, xorNodeUtils_1.boxContext)(context);
    }
    return undefined;
}
exports.parentXor = parentXor;
function parentXorChecked(nodeIdMapCollection, childId, expectedNodeKinds) {
    const xorNode = parentXor(nodeIdMapCollection, childId);
    return xorNode && __1.XorNodeUtils.isNodeKind(xorNode, expectedNodeKinds) ? xorNode : undefined;
}
exports.parentXorChecked = parentXorChecked;
function parentAst(nodeIdMapCollection, childId) {
    const parentId = nodeIdMapCollection.parentIdById.get(childId);
    return parentId !== undefined ? nodeIdMapCollection.astNodeById.get(parentId) : undefined;
}
exports.parentAst = parentAst;
function parentAstChecked(nodeIdMapCollection, childId, expectedNodeKinds) {
    const astNode = parentAst(nodeIdMapCollection, childId);
    return astNode && language_1.AstUtils.isNodeKind(astNode, expectedNodeKinds) ? astNode : undefined;
}
exports.parentAstChecked = parentAstChecked;
function parentContext(nodeIdMapCollection, childId) {
    const parentId = nodeIdMapCollection.parentIdById.get(childId);
    return parentId !== undefined ? nodeIdMapCollection.contextNodeById.get(parentId) : undefined;
}
exports.parentContext = parentContext;
function parentContextChecked(nodeIdMapCollection, childId, expectedNodeKinds) {
    const contextNode = parentContext(nodeIdMapCollection, childId);
    return contextNode && context_1.ParseContextUtils.isNodeKind(contextNode, expectedNodeKinds) ? contextNode : undefined;
}
exports.parentContextChecked = parentContextChecked;
//# sourceMappingURL=parentSelectors.js.map