import { Ast } from "../../language";
import { AstXorNode, ContextXorNode, TAstXorNode, TContextXorNode, TXorNode, XorNode } from "./xorNode";
import { ParseContext } from "..";
export declare function boxAst<T extends Ast.TNode>(node: T): XorNode<T>;
export declare function boxContext<T extends Ast.TNode>(node: ParseContext.Node<T>): ContextXorNode<T>;
export declare function ast(xorNode: TXorNode): Ast.TNode | undefined;
export declare function astChecked<T extends Ast.TNode>(xorNode: TXorNode, expectedNodeKinds: ReadonlyArray<T["kind"]> | T["kind"]): T | undefined;
export declare function context(xorNode: TXorNode): ParseContext.TNode | undefined;
export declare function contextChecked<T extends Ast.TNode>(xorNode: TXorNode, expectedNodeKinds: ReadonlyArray<T["kind"]> | T["kind"]): ParseContext.Node<T> | undefined;
export declare function isAst(xorNode: TXorNode): xorNode is TAstXorNode;
export declare function isAstChecked<T extends Ast.TNode>(xorNode: TXorNode, expectedNodeKinds: ReadonlyArray<T["kind"]> | T["kind"]): xorNode is AstXorNode<T>;
export declare function isContext<T extends Ast.TNode>(xorNode: TXorNode): xorNode is ContextXorNode<T>;
export declare function isContextChecked<T extends Ast.TNode>(xorNode: TXorNode, expectedNodeKinds: ReadonlyArray<T["kind"]> | T["kind"]): xorNode is ContextXorNode<T>;
export declare function isNodeKind<T extends Ast.TNode>(xorNode: TXorNode, expectedNodeKinds: ReadonlyArray<T["kind"]> | T["kind"]): xorNode is XorNode<T>;
export declare function isTAnyLiteral(xorNode: TXorNode): xorNode is XorNode<Ast.TAnyLiteral>;
export declare function isTArithmeticExpression(xorNode: TXorNode): xorNode is XorNode<Ast.TArithmeticExpression>;
export declare function isTAsExpression(xorNode: TXorNode): xorNode is XorNode<Ast.TAsExpression>;
export declare function isTEqualityExpression(xorNode: TXorNode): xorNode is XorNode<Ast.TEqualityExpression>;
export declare function isTExpression(xorNode: TXorNode): xorNode is XorNode<Ast.TExpression>;
export declare function isTFieldAccessExpression(xorNode: TXorNode): xorNode is XorNode<Ast.TFieldAccessExpression>;
export declare function isTIsExpression(xorNode: TXorNode): xorNode is XorNode<Ast.TIsExpression>;
export declare function isLeaf(xorNode: TXorNode): xorNode is XorNode<Ast.TLeaf>;
export declare function isTListItem(xorNode: TXorNode): xorNode is XorNode<Ast.TListItem>;
export declare function isTLogicalExpression(xorNode: TXorNode): xorNode is XorNode<Ast.TLogicalExpression>;
export declare function isTMetadataExpression(xorNode: TXorNode): xorNode is XorNode<Ast.TMetadataExpression>;
export declare function isTNullCoalescingExpression(xorNode: TXorNode): xorNode is XorNode<Ast.TNullCoalescingExpression>;
export declare function isTNullablePrimitiveType(xorNode: TXorNode): xorNode is XorNode<Ast.TNullablePrimitiveType>;
export declare function isTPrimaryExpression(xorNode: TXorNode): xorNode is XorNode<Ast.TPrimaryExpression>;
export declare function isTPrimaryType(xorNode: TXorNode): xorNode is XorNode<Ast.TPrimaryType>;
export declare function isTRecursivePrimaryExpression(xorNode: TXorNode): xorNode is XorNode<Ast.TRecursivePrimaryExpression>;
export declare function isTRelationalExpression(xorNode: TXorNode): xorNode is XorNode<Ast.TEqualityExpression>;
export declare function isTType(xorNode: TXorNode): xorNode is XorNode<Ast.TType>;
export declare function isTUnaryExpression(xorNode: TXorNode): xorNode is XorNode<Ast.TUnaryExpression>;
export declare function assertAst(xorNode: TXorNode): Ast.TNode;
export declare function assertAstChecked<T extends Ast.TNode>(xorNode: TXorNode, expectedNodeKinds: ReadonlyArray<T["kind"]> | T["kind"]): T;
export declare function assertContext(xorNode: TXorNode): ParseContext.TNode;
export declare function assertContextChecked<T extends Ast.TNode>(xorNode: TXorNode, expectedNodeKinds: ReadonlyArray<T["kind"]> | T["kind"]): ParseContext.Node<T>;
export declare function assertAsNodeKind<T extends Ast.TNode>(xorNode: TXorNode, expectedNodeKinds: ReadonlyArray<T["kind"]> | T["kind"]): XorNode<T>;
export declare function assertIsAst(xorNode: TXorNode): asserts xorNode is TAstXorNode;
export declare function assertIsAstChecked<T extends Ast.TNode>(xorNode: TXorNode, expectedNodeKinds: ReadonlyArray<T["kind"]> | T["kind"]): asserts xorNode is AstXorNode<T>;
export declare function assertIsContext(xorNode: TXorNode): asserts xorNode is TContextXorNode;
export declare function assertIsContextChecked<T extends Ast.TNode>(xorNode: TXorNode, expectedNodeKinds: ReadonlyArray<T["kind"]> | T["kind"]): asserts xorNode is ContextXorNode<T>;
export declare function assertIsNodeKind<T extends Ast.TNode>(xorNode: TXorNode, expectedNodeKinds: ReadonlyArray<T["kind"]> | T["kind"]): asserts xorNode is XorNode<T>;
export declare function assertIsIdentifier(xorNode: TXorNode): asserts xorNode is XorNode<Ast.Identifier | Ast.IdentifierExpression>;
export declare function assertIsList(xorNode: TXorNode): asserts xorNode is XorNode<Ast.ListExpression | Ast.ListLiteral>;
export declare function assertIsRecord(xorNode: TXorNode): asserts xorNode is XorNode<Ast.RecordExpression | Ast.RecordLiteral>;
export declare function isTWrapped(xorNode: TXorNode): xorNode is XorNode<Ast.TWrapped>;
export declare function identifierExpressionLiteral(xorNode: TXorNode): string | undefined;
