"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.identifierExpressionLiteral = exports.isTWrapped = exports.assertIsRecord = exports.assertIsList = exports.assertIsIdentifier = exports.assertIsNodeKind = exports.assertIsContextChecked = exports.assertIsContext = exports.assertIsAstChecked = exports.assertIsAst = exports.assertAsNodeKind = exports.assertContextChecked = exports.assertContext = exports.assertAstChecked = exports.assertAst = exports.isTUnaryExpression = exports.isTType = exports.isTRelationalExpression = exports.isTRecursivePrimaryExpression = exports.isTPrimaryType = exports.isTPrimaryExpression = exports.isTNullablePrimitiveType = exports.isTNullCoalescingExpression = exports.isTMetadataExpression = exports.isTLogicalExpression = exports.isTListItem = exports.isLeaf = exports.isTIsExpression = exports.isTFieldAccessExpression = exports.isTExpression = exports.isTEqualityExpression = exports.isTAsExpression = exports.isTArithmeticExpression = exports.isTAnyLiteral = exports.isNodeKind = exports.isContextChecked = exports.isContext = exports.isAstChecked = exports.isAst = exports.contextChecked = exports.context = exports.astChecked = exports.ast = exports.boxContext = exports.boxAst = void 0;
const common_1 = require("../../common");
const language_1 = require("../../language");
const xorNode_1 = require("./xorNode");
const context_1 = require("../context");
function boxAst(node) {
    return {
        kind: xorNode_1.XorNodeKind.Ast,
        node,
    };
}
exports.boxAst = boxAst;
function boxContext(node) {
    return {
        kind: xorNode_1.XorNodeKind.Context,
        node,
    };
}
exports.boxContext = boxContext;
function ast(xorNode) {
    return isAst(xorNode) ? xorNode.node : undefined;
}
exports.ast = ast;
function astChecked(xorNode, expectedNodeKinds) {
    const astNode = ast(xorNode);
    return astNode && language_1.AstUtils.isNodeKind(astNode, expectedNodeKinds) ? astNode : undefined;
}
exports.astChecked = astChecked;
function context(xorNode) {
    return isContext(xorNode) ? xorNode.node : undefined;
}
exports.context = context;
function contextChecked(xorNode, expectedNodeKinds) {
    const contextNode = context(xorNode);
    return contextNode && context_1.ParseContextUtils.isNodeKind(contextNode, expectedNodeKinds) ? contextNode : undefined;
}
exports.contextChecked = contextChecked;
function isAst(xorNode) {
    return xorNode.kind === xorNode_1.XorNodeKind.Ast;
}
exports.isAst = isAst;
function isAstChecked(xorNode, expectedNodeKinds) {
    return isAst(xorNode) && language_1.AstUtils.isNodeKind(xorNode.node, expectedNodeKinds);
}
exports.isAstChecked = isAstChecked;
function isContext(xorNode) {
    return xorNode.kind === xorNode_1.XorNodeKind.Context;
}
exports.isContext = isContext;
function isContextChecked(xorNode, expectedNodeKinds) {
    return isContext(xorNode) && context_1.ParseContextUtils.isNodeKind(xorNode.node, expectedNodeKinds);
}
exports.isContextChecked = isContextChecked;
function isNodeKind(xorNode, expectedNodeKinds) {
    return (xorNode.node.kind === expectedNodeKinds ||
        (Array.isArray(expectedNodeKinds) && expectedNodeKinds.includes(xorNode.node.kind)));
}
exports.isNodeKind = isNodeKind;
function isTAnyLiteral(xorNode) {
    return language_1.Ast.NodeKindsForTAnyLiteral.has(xorNode.node.kind);
}
exports.isTAnyLiteral = isTAnyLiteral;
function isTArithmeticExpression(xorNode) {
    return language_1.Ast.NodeKindsForTArithmeticExpression.has(xorNode.node.kind);
}
exports.isTArithmeticExpression = isTArithmeticExpression;
function isTAsExpression(xorNode) {
    return language_1.Ast.NodeKindsForTAsExpression.has(xorNode.node.kind);
}
exports.isTAsExpression = isTAsExpression;
function isTEqualityExpression(xorNode) {
    return language_1.Ast.NodeKindsForTEqualityExpression.has(xorNode.node.kind);
}
exports.isTEqualityExpression = isTEqualityExpression;
function isTExpression(xorNode) {
    return language_1.Ast.NodeKindsForTExpression.has(xorNode.node.kind);
}
exports.isTExpression = isTExpression;
function isTFieldAccessExpression(xorNode) {
    return language_1.Ast.NodeKindsForTFieldAccessExpression.has(xorNode.node.kind);
}
exports.isTFieldAccessExpression = isTFieldAccessExpression;
function isTIsExpression(xorNode) {
    return language_1.Ast.NodeKindsForTIsExpression.has(xorNode.node.kind);
}
exports.isTIsExpression = isTIsExpression;
function isLeaf(xorNode) {
    return language_1.Ast.NodeKindsForTLeaf.has(xorNode.node.kind);
}
exports.isLeaf = isLeaf;
function isTListItem(xorNode) {
    return language_1.Ast.NodeKindsForTListItem.has(xorNode.node.kind);
}
exports.isTListItem = isTListItem;
function isTLogicalExpression(xorNode) {
    return language_1.Ast.NodeKindsForTLogicalExpression.has(xorNode.node.kind);
}
exports.isTLogicalExpression = isTLogicalExpression;
function isTMetadataExpression(xorNode) {
    return language_1.Ast.NodeKindsForTMetadataExpression.has(xorNode.node.kind);
}
exports.isTMetadataExpression = isTMetadataExpression;
function isTNullCoalescingExpression(xorNode) {
    return language_1.Ast.NodeKindsForTNullCoalescingExpression.has(xorNode.node.kind);
}
exports.isTNullCoalescingExpression = isTNullCoalescingExpression;
function isTNullablePrimitiveType(xorNode) {
    return language_1.Ast.NodeKindsForTNullablePrimitiveType.has(xorNode.node.kind);
}
exports.isTNullablePrimitiveType = isTNullablePrimitiveType;
function isTPrimaryExpression(xorNode) {
    return language_1.Ast.NodeKindsForTPrimaryExpression.has(xorNode.node.kind);
}
exports.isTPrimaryExpression = isTPrimaryExpression;
function isTPrimaryType(xorNode) {
    return language_1.Ast.NodeKindsForTPrimaryType.has(xorNode.node.kind);
}
exports.isTPrimaryType = isTPrimaryType;
function isTRecursivePrimaryExpression(xorNode) {
    return language_1.Ast.NodeKindsForTRecursivePrimaryExpression.has(xorNode.node.kind);
}
exports.isTRecursivePrimaryExpression = isTRecursivePrimaryExpression;
function isTRelationalExpression(xorNode) {
    return language_1.Ast.NodeKindsForTRelationalExpression.has(xorNode.node.kind);
}
exports.isTRelationalExpression = isTRelationalExpression;
function isTType(xorNode) {
    return language_1.Ast.NodeKindsForTType.has(xorNode.node.kind);
}
exports.isTType = isTType;
function isTUnaryExpression(xorNode) {
    return language_1.Ast.NodeKindsForTUnaryExpression.has(xorNode.node.kind);
}
exports.isTUnaryExpression = isTUnaryExpression;
function assertAst(xorNode) {
    assertIsAst(xorNode);
    return xorNode.node;
}
exports.assertAst = assertAst;
function assertAstChecked(xorNode, expectedNodeKinds) {
    const astNode = assertAst(xorNode);
    language_1.AstUtils.assertIsNodeKind(astNode, expectedNodeKinds);
    return astNode;
}
exports.assertAstChecked = assertAstChecked;
function assertContext(xorNode) {
    assertIsContext(xorNode);
    return xorNode.node;
}
exports.assertContext = assertContext;
function assertContextChecked(xorNode, expectedNodeKinds) {
    assertIsContextChecked(xorNode, expectedNodeKinds);
    return xorNode.node;
}
exports.assertContextChecked = assertContextChecked;
function assertAsNodeKind(xorNode, expectedNodeKinds) {
    assertIsNodeKind(xorNode, expectedNodeKinds);
    return xorNode;
}
exports.assertAsNodeKind = assertAsNodeKind;
function assertIsAst(xorNode) {
    common_1.Assert.isTrue(isAst(xorNode), "expected xorNode to hold an Ast node", {
        xorNodeKind: xorNode.kind,
        xorNodeId: xorNode.node.id,
    });
}
exports.assertIsAst = assertIsAst;
function assertIsAstChecked(xorNode, expectedNodeKinds) {
    assertIsAst(xorNode);
    assertIsNodeKind(xorNode, expectedNodeKinds);
}
exports.assertIsAstChecked = assertIsAstChecked;
function assertIsContext(xorNode) {
    common_1.Assert.isTrue(isContext(xorNode), "expected xorNode to hold an Context node", {
        xorNodeKind: xorNode.kind,
        xorNodeId: xorNode.node.id,
    });
}
exports.assertIsContext = assertIsContext;
function assertIsContextChecked(xorNode, expectedNodeKinds) {
    assertIsContext(xorNode);
    assertIsNodeKind(xorNode, expectedNodeKinds);
}
exports.assertIsContextChecked = assertIsContextChecked;
function assertIsNodeKind(xorNode, expectedNodeKinds) {
    if (Array.isArray(expectedNodeKinds)) {
        common_1.ArrayUtils.assertIncludes(expectedNodeKinds, xorNode.node.kind, `ArrayUtils.assertIncludes(expectedNodeKinds, xorNodeKind)`, {
            actualNodeKind: xorNode.node.kind,
            actualNodeId: xorNode.node.id,
            expectedNodeKinds,
        });
    }
    else {
        common_1.Assert.isTrue(xorNode.node.kind === expectedNodeKinds, "xorNode.node.kind === expectedNodeKinds", {
            nodeId: xorNode.node.id,
            nodeKind: xorNode.node.kind,
            expectedNodeKind: expectedNodeKinds,
        });
    }
}
exports.assertIsNodeKind = assertIsNodeKind;
function assertIsIdentifier(xorNode) {
    assertIsNodeKind(xorNode, [language_1.Ast.NodeKind.Identifier, language_1.Ast.NodeKind.IdentifierExpression]);
}
exports.assertIsIdentifier = assertIsIdentifier;
function assertIsList(xorNode) {
    assertIsNodeKind(xorNode, [language_1.Ast.NodeKind.ListExpression, language_1.Ast.NodeKind.ListLiteral]);
}
exports.assertIsList = assertIsList;
function assertIsRecord(xorNode) {
    assertIsNodeKind(xorNode, [language_1.Ast.NodeKind.RecordExpression, language_1.Ast.NodeKind.RecordLiteral]);
}
exports.assertIsRecord = assertIsRecord;
function isTWrapped(xorNode) {
    return isNodeKind(xorNode, [
        language_1.Ast.NodeKind.FieldProjection,
        language_1.Ast.NodeKind.FieldSelector,
        language_1.Ast.NodeKind.FieldSpecificationList,
        language_1.Ast.NodeKind.InvokeExpression,
        language_1.Ast.NodeKind.ItemAccessExpression,
        language_1.Ast.NodeKind.ListExpression,
        language_1.Ast.NodeKind.ListLiteral,
        language_1.Ast.NodeKind.ListType,
        language_1.Ast.NodeKind.ParameterList,
        language_1.Ast.NodeKind.ParenthesizedExpression,
        language_1.Ast.NodeKind.RecordExpression,
        language_1.Ast.NodeKind.RecordLiteral,
    ]);
}
exports.isTWrapped = isTWrapped;
function identifierExpressionLiteral(xorNode) {
    const identifierExpression = astChecked(xorNode, language_1.Ast.NodeKind.IdentifierExpression);
    if (identifierExpression === undefined) {
        return undefined;
    }
    return identifierExpression.inclusiveConstant === undefined
        ? identifierExpression.identifier.literal
        : identifierExpression.inclusiveConstant.constantKind + identifierExpression.identifier.literal;
}
exports.identifierExpressionLiteral = identifierExpressionLiteral;
//# sourceMappingURL=xorNodeUtils.js.map