"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tokenWithColumnNumber = exports.currentTokenWithColumnNumber = exports.unterminatedParenthesesError = exports.unterminatedBracketError = exports.assertIsDoneParsing = exports.assertNoOpenContext = exports.assertNoMoreTokens = exports.testIsOnAnyTokenKind = exports.testIsOnTokenKind = exports.testClosingTokenKind = exports.testCsvContinuationDanglingComma = exports.testCsvContinuationLetExpression = exports.assertGetTokenAt = exports.assertGetContextNodeMetadata = exports.isRecursivePrimaryExpressionNext = exports.isOnGeneralizedIdentifierStart = exports.isOnConstantKind = exports.isOnTokenKind = exports.isNextTokenKind = exports.isTokenKind = exports.incrementAttributeCounter = exports.deleteContext = exports.endContext = exports.startContextAsParent = exports.startContext = exports.copyState = exports.applyState = exports.newState = void 0;
const common_1 = require("../../common");
const language_1 = require("../../language");
const __1 = require("..");
const trace_1 = require("../../common/trace");
const localization_1 = require("../../localization");
const disambiguation_1 = require("../disambiguation");
const error_1 = require("../error");
function newState(lexerSnapshot, overrides) {
    var _a, _b, _c, _d, _e, _f;
    const tokenIndex = (_a = overrides === null || overrides === void 0 ? void 0 : overrides.tokenIndex) !== null && _a !== void 0 ? _a : 0;
    const currentToken = lexerSnapshot.tokens[tokenIndex];
    const currentTokenKind = currentToken === null || currentToken === void 0 ? void 0 : currentToken.kind;
    const contextState = (_b = overrides === null || overrides === void 0 ? void 0 : overrides.contextState) !== null && _b !== void 0 ? _b : __1.ParseContextUtils.newState();
    const nodeIdMapCollection = contextState.nodeIdMapCollection;
    const currentContextNodeId = nodeIdMapCollection.contextNodeById.size > 0
        ? Math.max(...nodeIdMapCollection.contextNodeById.keys())
        : undefined;
    const currentContextNode = currentContextNodeId !== undefined
        ? common_1.MapUtils.assertGet(nodeIdMapCollection.contextNodeById, currentContextNodeId)
        : undefined;
    return Object.assign(Object.assign({}, overrides), { disambiguationBehavior: (_c = overrides === null || overrides === void 0 ? void 0 : overrides.disambiguationBehavior) !== null && _c !== void 0 ? _c : disambiguation_1.Disambiguation.DismabiguationBehavior.Thorough, lexerSnapshot, locale: (_d = overrides === null || overrides === void 0 ? void 0 : overrides.locale) !== null && _d !== void 0 ? _d : localization_1.DefaultLocale, cancellationToken: overrides === null || overrides === void 0 ? void 0 : overrides.cancellationToken, traceManager: (_e = overrides === null || overrides === void 0 ? void 0 : overrides.traceManager) !== null && _e !== void 0 ? _e : trace_1.NoOpTraceManagerInstance, contextState: (_f = overrides === null || overrides === void 0 ? void 0 : overrides.contextState) !== null && _f !== void 0 ? _f : __1.ParseContextUtils.newState(), currentToken,
        currentContextNode,
        currentTokenKind,
        tokenIndex });
}
exports.newState = newState;
// If you have a custom parser + parser state, then you'll have to create your own copyState/applyState functions.
// eslint-disable-next-line require-await
function applyState(state, update) {
    return __awaiter(this, void 0, void 0, function* () {
        state.tokenIndex = update.tokenIndex;
        state.currentToken = update.currentToken;
        state.currentTokenKind = update.currentTokenKind;
        state.contextState = update.contextState;
        state.currentContextNode = update.currentContextNode;
    });
}
exports.applyState = applyState;
// If you have a custom parser + parser state, then you'll have to create your own copyState/applyState functions.
// eslint-disable-next-line require-await
function copyState(state) {
    return __awaiter(this, void 0, void 0, function* () {
        return Object.assign(Object.assign({}, state), { contextState: __1.ParseContextUtils.copyState(state.contextState) });
    });
}
exports.copyState = copyState;
function startContext(state, nodeKind) {
    const newContextNode = __1.ParseContextUtils.startContext(state.contextState, nodeKind, state.tokenIndex, state.currentToken, state.currentContextNode);
    state.currentContextNode = newContextNode;
    return newContextNode;
}
exports.startContext = startContext;
// Inserts a new context as the parent of an existing node.
// Requires a re-calculation of all nodeIds associated with or under the inserted context.
function startContextAsParent(state, nodeKind, existingNodeId, correlationId) {
    const trace = state.traceManager.entry("ParseStateUtils" /* ParseStateUtils */, startContextAsParent.name, correlationId, { existingNodeId });
    // We need to find the starting token for the existing node as it'll be the starting token for the new context.
    const nodeIdMapCollection = state.contextState.nodeIdMapCollection;
    const existingNode = __1.NodeIdMapUtils.assertXor(nodeIdMapCollection, existingNodeId);
    const tokenStart = __1.XorNodeUtils.isAst(existingNode)
        ? common_1.ArrayUtils.assertGet(state.lexerSnapshot.tokens, existingNode.node.tokenRange.tokenIndexStart)
        : existingNode.node.tokenStart;
    const insertedContext = __1.ParseContextUtils.startContextAsParent(state.contextState, nodeKind, existingNodeId, tokenStart);
    trace.exit();
    return insertedContext;
}
exports.startContextAsParent = startContextAsParent;
function endContext(state, astNode) {
    const contextNode = common_1.Assert.asDefined(state.currentContextNode, `can't end a context if one doesn't exist`);
    const parentOfContextNode = __1.ParseContextUtils.endContext(state.contextState, contextNode, astNode);
    state.currentContextNode = parentOfContextNode;
}
exports.endContext = endContext;
function deleteContext(state, nodeId) {
    if (nodeId === undefined) {
        nodeId = common_1.Assert.asDefined(state.currentContextNode, `can't delete a context if one doesn't exist`).id;
    }
    state.currentContextNode = __1.ParseContextUtils.deleteContext(state.contextState, nodeId);
}
exports.deleteContext = deleteContext;
function incrementAttributeCounter(state) {
    common_1.Assert.asDefined(state.currentContextNode, `state.currentContextNode`).attributeCounter += 1;
}
exports.incrementAttributeCounter = incrementAttributeCounter;
// -------------------------
// ---------- IsX ----------
// -------------------------
function isTokenKind(state, tokenKind, tokenIndex) {
    var _a, _b;
    return (_b = ((_a = state.lexerSnapshot.tokens[tokenIndex]) === null || _a === void 0 ? void 0 : _a.kind) === tokenKind) !== null && _b !== void 0 ? _b : false;
}
exports.isTokenKind = isTokenKind;
function isNextTokenKind(state, tokenKind) {
    return isTokenKind(state, tokenKind, state.tokenIndex + 1);
}
exports.isNextTokenKind = isNextTokenKind;
function isOnTokenKind(state, tokenKind, tokenIndex = state.tokenIndex) {
    return isTokenKind(state, tokenKind, tokenIndex);
}
exports.isOnTokenKind = isOnTokenKind;
function isOnConstantKind(state, constantKind) {
    if (isOnTokenKind(state, language_1.Token.TokenKind.Identifier)) {
        const currentToken = state.lexerSnapshot.tokens[state.tokenIndex];
        if ((currentToken === null || currentToken === void 0 ? void 0 : currentToken.data) === undefined) {
            const details = { currentToken };
            throw new common_1.CommonError.InvariantError(`expected data on Token`, details);
        }
        const data = currentToken.data;
        return data === constantKind;
    }
    else {
        return false;
    }
}
exports.isOnConstantKind = isOnConstantKind;
function isOnGeneralizedIdentifierStart(state, tokenIndex = state.tokenIndex) {
    var _a;
    const tokenKind = (_a = state.lexerSnapshot.tokens[tokenIndex]) === null || _a === void 0 ? void 0 : _a.kind;
    if (tokenKind === undefined) {
        return false;
    }
    switch (tokenKind) {
        case language_1.Token.TokenKind.Identifier:
        case language_1.Token.TokenKind.KeywordAnd:
        case language_1.Token.TokenKind.KeywordAs:
        case language_1.Token.TokenKind.KeywordEach:
        case language_1.Token.TokenKind.KeywordElse:
        case language_1.Token.TokenKind.KeywordError:
        case language_1.Token.TokenKind.KeywordFalse:
        case language_1.Token.TokenKind.KeywordHashBinary:
        case language_1.Token.TokenKind.KeywordHashDate:
        case language_1.Token.TokenKind.KeywordHashDateTime:
        case language_1.Token.TokenKind.KeywordHashDateTimeZone:
        case language_1.Token.TokenKind.KeywordHashDuration:
        case language_1.Token.TokenKind.KeywordHashInfinity:
        case language_1.Token.TokenKind.KeywordHashNan:
        case language_1.Token.TokenKind.KeywordHashSections:
        case language_1.Token.TokenKind.KeywordHashShared:
        case language_1.Token.TokenKind.KeywordHashTable:
        case language_1.Token.TokenKind.KeywordHashTime:
        case language_1.Token.TokenKind.KeywordIf:
        case language_1.Token.TokenKind.KeywordIn:
        case language_1.Token.TokenKind.KeywordIs:
        case language_1.Token.TokenKind.KeywordLet:
        case language_1.Token.TokenKind.KeywordMeta:
        case language_1.Token.TokenKind.KeywordNot:
        case language_1.Token.TokenKind.KeywordOr:
        case language_1.Token.TokenKind.KeywordOtherwise:
        case language_1.Token.TokenKind.KeywordSection:
        case language_1.Token.TokenKind.KeywordShared:
        case language_1.Token.TokenKind.KeywordThen:
        case language_1.Token.TokenKind.KeywordTrue:
        case language_1.Token.TokenKind.KeywordTry:
        case language_1.Token.TokenKind.KeywordType:
            return true;
        default:
            return false;
    }
}
exports.isOnGeneralizedIdentifierStart = isOnGeneralizedIdentifierStart;
// Assumes a call to readPrimaryExpression has already happened.
function isRecursivePrimaryExpressionNext(state, tokenIndexStart = state.tokenIndex) {
    return (
    // section-access-expression
    // this.isOnTokenKind(TokenKind.Bang)
    // field-access-expression
    isTokenKind(state, language_1.Token.TokenKind.LeftBrace, tokenIndexStart) ||
        // item-access-expression
        isTokenKind(state, language_1.Token.TokenKind.LeftBracket, tokenIndexStart) ||
        // invoke-expression
        isTokenKind(state, language_1.Token.TokenKind.LeftParenthesis, tokenIndexStart));
}
exports.isRecursivePrimaryExpressionNext = isRecursivePrimaryExpressionNext;
// -----------------------------
// ---------- Asserts ----------
// -----------------------------
function assertGetContextNodeMetadata(state) {
    const currentContextNode = common_1.Assert.asDefined(state.currentContextNode);
    const tokenStart = common_1.Assert.asDefined(currentContextNode.tokenStart);
    // inclusive token index
    const tokenIndexEnd = state.tokenIndex - 1;
    const tokenEnd = common_1.Assert.asDefined(state.lexerSnapshot.tokens[tokenIndexEnd]);
    const tokenRange = {
        tokenIndexStart: currentContextNode.tokenIndexStart,
        tokenIndexEnd,
        positionStart: tokenStart.positionStart,
        positionEnd: tokenEnd.positionEnd,
    };
    return {
        id: currentContextNode.id,
        attributeIndex: currentContextNode.attributeIndex,
        tokenRange,
    };
}
exports.assertGetContextNodeMetadata = assertGetContextNodeMetadata;
function assertGetTokenAt(state, tokenIndex) {
    const lexerSnapshot = state.lexerSnapshot;
    const token = lexerSnapshot.tokens[tokenIndex];
    return common_1.Assert.asDefined(token, undefined, { tokenIndex });
}
exports.assertGetTokenAt = assertGetTokenAt;
// -------------------------------
// ---------- Csv Tests ----------
// -------------------------------
// All of these tests assume you're in a given context and have just read a `,`.
// Eg. testCsvEndLetExpression assumes you're in a LetExpression context and have just read a `,`.
function testCsvContinuationLetExpression(state) {
    if (state.currentTokenKind === language_1.Token.TokenKind.KeywordIn) {
        return new __1.ParseError.ExpectedCsvContinuationError(__1.ParseError.CsvContinuationKind.LetExpression, currentTokenWithColumnNumber(state), state.locale);
    }
    return undefined;
}
exports.testCsvContinuationLetExpression = testCsvContinuationLetExpression;
function testCsvContinuationDanglingComma(state, tokenKind) {
    if (state.currentTokenKind === tokenKind) {
        return new __1.ParseError.ExpectedCsvContinuationError(__1.ParseError.CsvContinuationKind.DanglingComma, currentTokenWithColumnNumber(state), state.locale);
    }
    else {
        return undefined;
    }
}
exports.testCsvContinuationDanglingComma = testCsvContinuationDanglingComma;
// -------------------------------------
// ---------- Asserts / Tests ----------
// -------------------------------------
// This test is run after seeing no additional commas in a CsvArray,
// when a closing terminator is expected.
function testClosingTokenKind(state, expectedTokenKind) {
    if (isOnTokenKind(state, expectedTokenKind)) {
        return undefined;
    }
    return new __1.ParseError.ExpectedClosingTokenKind(expectedTokenKind, currentTokenWithColumnNumber(state), state.locale);
}
exports.testClosingTokenKind = testClosingTokenKind;
function testIsOnTokenKind(state, expectedTokenKind) {
    if (expectedTokenKind !== state.currentTokenKind) {
        const token = currentTokenWithColumnNumber(state);
        return new __1.ParseError.ExpectedTokenKindError(expectedTokenKind, token, state.locale);
    }
    else {
        return undefined;
    }
}
exports.testIsOnTokenKind = testIsOnTokenKind;
function testIsOnAnyTokenKind(state, expectedAnyTokenKinds) {
    const isError = state.currentTokenKind === undefined || !expectedAnyTokenKinds.includes(state.currentTokenKind);
    if (isError) {
        const token = currentTokenWithColumnNumber(state);
        return new __1.ParseError.ExpectedAnyTokenKindError(expectedAnyTokenKinds, token, state.locale);
    }
    else {
        return undefined;
    }
}
exports.testIsOnAnyTokenKind = testIsOnAnyTokenKind;
function assertNoMoreTokens(state) {
    if (state.tokenIndex === state.lexerSnapshot.tokens.length) {
        return;
    }
    const token = assertGetTokenAt(state, state.tokenIndex);
    throw new __1.ParseError.UnusedTokensRemainError(token, state.lexerSnapshot.graphemePositionStartFrom(token), state.locale);
}
exports.assertNoMoreTokens = assertNoMoreTokens;
function assertNoOpenContext(state) {
    var _a;
    common_1.Assert.isUndefined(state.currentContextNode, undefined, {
        contextNodeId: (_a = state.currentContextNode) === null || _a === void 0 ? void 0 : _a.id,
    });
}
exports.assertNoOpenContext = assertNoOpenContext;
function assertIsDoneParsing(state) {
    assertNoMoreTokens(state);
    assertNoOpenContext(state);
}
exports.assertIsDoneParsing = assertIsDoneParsing;
// -------------------------------------
// ---------- Error factories ----------
// -------------------------------------
function unterminatedBracketError(state) {
    return unterminatedSequence(state, error_1.SequenceKind.Bracket);
}
exports.unterminatedBracketError = unterminatedBracketError;
function unterminatedParenthesesError(state) {
    return unterminatedSequence(state, error_1.SequenceKind.Parenthesis);
}
exports.unterminatedParenthesesError = unterminatedParenthesesError;
function unterminatedSequence(state, sequenceKind) {
    const token = assertGetTokenAt(state, state.tokenIndex);
    return new __1.ParseError.UnterminatedSequence(sequenceKind, token, state.lexerSnapshot.graphemePositionStartFrom(token), state.locale);
}
// ---------------------------------------------
// ---------- Column number factories ----------
// ---------------------------------------------
function currentTokenWithColumnNumber(state) {
    return tokenWithColumnNumber(state, state.tokenIndex);
}
exports.currentTokenWithColumnNumber = currentTokenWithColumnNumber;
function tokenWithColumnNumber(state, tokenIndex) {
    const token = state.lexerSnapshot.tokens[tokenIndex];
    if (!token) {
        return undefined;
    }
    const currentToken = token;
    return {
        token: currentToken,
        columnNumber: state.lexerSnapshot.columnNumberStartFrom(currentToken),
    };
}
exports.tokenWithColumnNumber = tokenWithColumnNumber;
//# sourceMappingURL=parseStateUtils.js.map