import { ParseError, ParseState } from "..";
import { Ast } from "../../language";
import { Result } from "../../common";
export declare type TriedParse = Result<ParseOk, ParseError.TParseError>;
export interface ParseStateCheckpoint {
    readonly tokenIndex: number;
    readonly contextStateIdCounter: number;
    readonly contextNodeId: number | undefined;
}
export interface ParseOk {
    readonly root: Ast.TNode;
    readonly state: ParseState;
}
export interface Parser {
    readonly applyState: (state: ParseState, update: ParseState) => Promise<void>;
    readonly copyState: (state: ParseState) => Promise<ParseState>;
    readonly checkpoint: (state: ParseState) => Promise<ParseStateCheckpoint>;
    readonly restoreCheckpoint: (state: ParseState, checkpoint: ParseStateCheckpoint) => Promise<void>;
    readonly readIdentifier: (state: ParseState, parser: Parser, identifierContextKind: Ast.IdentifierContextKind, correlationId: number | undefined) => Ast.Identifier;
    readonly readGeneralizedIdentifier: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.GeneralizedIdentifier>;
    readonly readKeyword: (state: ParseState, parser: Parser, correlationId: number | undefined) => Ast.IdentifierExpression;
    readonly readDocument: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TDocument>;
    readonly readSectionDocument: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.Section>;
    readonly readSectionMembers: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.IArrayWrapper<Ast.SectionMember>>;
    readonly readSectionMember: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.SectionMember>;
    readonly readNullCoalescingExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TExpression>;
    readonly readExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TExpression>;
    readonly readLogicalExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TLogicalExpression>;
    readonly readIsExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TIsExpression>;
    readonly readNullablePrimitiveType: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TNullablePrimitiveType>;
    readonly readAsExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TAsExpression>;
    readonly readEqualityExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TEqualityExpression>;
    readonly readRelationalExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TRelationalExpression>;
    readonly readArithmeticExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TArithmeticExpression>;
    readonly readMetadataExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TMetadataExpression>;
    readonly readUnaryExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TUnaryExpression>;
    readonly readPrimaryExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TPrimaryExpression>;
    readonly readRecursivePrimaryExpression: (state: ParseState, parser: Parser, head: Ast.TPrimaryExpression, correlationId: number | undefined) => Promise<Ast.RecursivePrimaryExpression>;
    readonly readLiteralExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Ast.LiteralExpression;
    readonly readIdentifierExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Ast.IdentifierExpression;
    readonly readParenthesizedExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.ParenthesizedExpression>;
    readonly readNotImplementedExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Ast.NotImplementedExpression;
    readonly readInvokeExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.InvokeExpression>;
    readonly readListExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.ListExpression>;
    readonly readListItem: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TListItem>;
    readonly readRecordExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.RecordExpression>;
    readonly readItemAccessExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.ItemAccessExpression>;
    readonly readFieldSelection: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.FieldSelector>;
    readonly readFieldProjection: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.FieldProjection>;
    readonly readFieldSelector: (state: ParseState, parser: Parser, allowOptional: boolean, correlationId: number | undefined) => Promise<Ast.FieldSelector>;
    readonly readFunctionExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.FunctionExpression>;
    readonly readParameterList: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.IParameterList<Ast.AsNullablePrimitiveType | undefined>>;
    readonly readAsType: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.AsType>;
    readonly readEachExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.EachExpression>;
    readonly readLetExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.LetExpression>;
    readonly readIfExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.IfExpression>;
    readonly readTypeExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TTypeExpression>;
    readonly readType: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TType>;
    readonly readPrimaryType: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TPrimaryType>;
    readonly readRecordType: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.RecordType>;
    readonly readTableType: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TableType>;
    readonly readFieldSpecificationList: (state: ParseState, parser: Parser, allowOpenMarker: boolean, testPostCommaError: (state: ParseState) => ParseError.TInnerParseError | undefined, correlationId: number | undefined) => Promise<Ast.FieldSpecificationList>;
    readonly readListType: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.ListType>;
    readonly readFunctionType: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.FunctionType>;
    readonly readParameterSpecificationList: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.IParameterList<Ast.AsType>>;
    readonly readNullableType: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.NullableType>;
    readonly readErrorRaisingExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.ErrorRaisingExpression>;
    readonly readErrorHandlingExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TErrorHandlingExpression>;
    readonly readRecordLiteral: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.RecordLiteral>;
    readonly readFieldNamePairedAnyLiterals: (state: ParseState, parser: Parser, onePairRequired: boolean, testPostCommaError: (state: ParseState) => ParseError.TInnerParseError | undefined, correlationId: number | undefined) => Promise<Ast.ICsvArray<Ast.GeneralizedIdentifierPairedAnyLiteral>>;
    readonly readListLiteral: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.ListLiteral>;
    readonly readAnyLiteral: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.TAnyLiteral>;
    readonly readPrimitiveType: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.PrimitiveType>;
    readonly readIdentifierPairedExpressions: (state: ParseState, parser: Parser, onePairRequired: boolean, testPostCommaError: (state: ParseState) => ParseError.TInnerParseError | undefined, correlationId: number | undefined) => Promise<Ast.ICsvArray<Ast.IdentifierPairedExpression>>;
    readonly readIdentifierPairedExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.IdentifierPairedExpression>;
    readonly readGeneralizedIdentifierPairedExpressions: (state: ParseState, parser: Parser, onePairRequired: boolean, testPostCommaError: (state: ParseState) => ParseError.TInnerParseError | undefined, correlationId: number | undefined) => Promise<Ast.ICsvArray<Ast.GeneralizedIdentifierPairedExpression>>;
    readonly readGeneralizedIdentifierPairedExpression: (state: ParseState, parser: Parser, correlationId: number | undefined) => Promise<Ast.GeneralizedIdentifierPairedExpression>;
}
