"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CombinatorialParserV2 = void 0;
const language_1 = require("../../../language");
const commonTypes_1 = require("./commonTypes");
const disambiguation_1 = require("../../disambiguation");
const parser_1 = require("../../parser");
const parseState_1 = require("../../parseState");
const common_1 = require("../../../common");
const combineOperatorsAndOperands_1 = require("./combineOperatorsAndOperands");
const __1 = require("..");
const readOperatorsAndOperands_1 = require("./readOperatorsAndOperands");
// An optimized parser, with two key changes:
//  1. readUnaryExpression looks at the current token and dispatches to the appropriate read function.
//  2. Binary expressions are read using a 2 phase process:
//      a. read all operators and operands before creating any binary expressions
//          - this should leave us with N operators and N+1 operands (where N >= 0)
//      b. create binary operation expressions by combining operators and operands by their precedence
//          - this reduces the list of operators and operands by 1 each time
//          - we should end up with 1 operand and 0 operators
exports.CombinatorialParserV2 = {
    applyState: parseState_1.ParseStateUtils.applyState,
    copyState: parseState_1.ParseStateUtils.copyState,
    checkpoint: parser_1.ParserUtils.checkpoint,
    restoreCheckpoint: parser_1.ParserUtils.restoreCheckpoint,
    readIdentifier: __1.NaiveParseSteps.readIdentifier,
    readGeneralizedIdentifier: __1.NaiveParseSteps.readGeneralizedIdentifier,
    readKeyword: __1.NaiveParseSteps.readKeyword,
    readDocument: __1.NaiveParseSteps.readDocument,
    readSectionDocument: __1.NaiveParseSteps.readSectionDocument,
    readSectionMembers: __1.NaiveParseSteps.readSectionMembers,
    readSectionMember: __1.NaiveParseSteps.readSectionMember,
    readNullCoalescingExpression: __1.NaiveParseSteps.readNullCoalescingExpression,
    readExpression: __1.NaiveParseSteps.readExpression,
    readLogicalExpression: (state, parser, correlationId) => readBinOpExpression(state, parser, language_1.Ast.NodeKind.LogicalExpression, correlationId),
    readIsExpression: __1.NaiveParseSteps.readIsExpression,
    readNullablePrimitiveType: __1.NaiveParseSteps.readNullablePrimitiveType,
    readAsExpression: __1.NaiveParseSteps.readAsExpression,
    readEqualityExpression: (state, parser, correlationId) => readBinOpExpression(state, parser, language_1.Ast.NodeKind.EqualityExpression, correlationId),
    readRelationalExpression: (state, parser, correlationId) => readBinOpExpression(state, parser, language_1.Ast.NodeKind.RelationalExpression, correlationId),
    readArithmeticExpression: (state, parser, correlationId) => readBinOpExpression(state, parser, language_1.Ast.NodeKind.ArithmeticExpression, correlationId),
    readMetadataExpression: (state, parser, correlationId) => readBinOpExpression(state, parser, language_1.Ast.NodeKind.MetadataExpression, correlationId),
    readUnaryExpression,
    readPrimaryExpression: __1.NaiveParseSteps.readPrimaryExpression,
    readRecursivePrimaryExpression: __1.NaiveParseSteps.readRecursivePrimaryExpression,
    readLiteralExpression: __1.NaiveParseSteps.readLiteralExpression,
    readIdentifierExpression: __1.NaiveParseSteps.readIdentifierExpression,
    readParenthesizedExpression: __1.NaiveParseSteps.readParenthesizedExpression,
    readNotImplementedExpression: __1.NaiveParseSteps.readNotImplementedExpression,
    readInvokeExpression: __1.NaiveParseSteps.readInvokeExpression,
    readListExpression: __1.NaiveParseSteps.readListExpression,
    readListItem: __1.NaiveParseSteps.readListItem,
    readRecordExpression: __1.NaiveParseSteps.readRecordExpression,
    readItemAccessExpression: __1.NaiveParseSteps.readItemAccessExpression,
    readFieldSelection: __1.NaiveParseSteps.readFieldSelection,
    readFieldProjection: __1.NaiveParseSteps.readFieldProjection,
    readFieldSelector: __1.NaiveParseSteps.readFieldSelector,
    readFunctionExpression: __1.NaiveParseSteps.readFunctionExpression,
    readParameterList: __1.NaiveParseSteps.readParameterList,
    readAsType: __1.NaiveParseSteps.readAsType,
    readEachExpression: __1.NaiveParseSteps.readEachExpression,
    readLetExpression: __1.NaiveParseSteps.readLetExpression,
    readIfExpression: __1.NaiveParseSteps.readIfExpression,
    readTypeExpression: __1.NaiveParseSteps.readTypeExpression,
    readType: __1.NaiveParseSteps.readType,
    readPrimaryType: __1.NaiveParseSteps.readPrimaryType,
    readRecordType: __1.NaiveParseSteps.readRecordType,
    readTableType: __1.NaiveParseSteps.readTableType,
    readFieldSpecificationList: __1.NaiveParseSteps.readFieldSpecificationList,
    readListType: __1.NaiveParseSteps.readListType,
    readFunctionType: __1.NaiveParseSteps.readFunctionType,
    readParameterSpecificationList: __1.NaiveParseSteps.readParameterSpecificationList,
    readNullableType: __1.NaiveParseSteps.readNullableType,
    readErrorRaisingExpression: __1.NaiveParseSteps.readErrorRaisingExpression,
    readErrorHandlingExpression: __1.NaiveParseSteps.readErrorHandlingExpression,
    readRecordLiteral: __1.NaiveParseSteps.readRecordLiteral,
    readFieldNamePairedAnyLiterals: __1.NaiveParseSteps.readFieldNamePairedAnyLiterals,
    readListLiteral: __1.NaiveParseSteps.readListLiteral,
    readAnyLiteral: __1.NaiveParseSteps.readAnyLiteral,
    readPrimitiveType: __1.NaiveParseSteps.readPrimitiveType,
    readIdentifierPairedExpressions: __1.NaiveParseSteps.readIdentifierPairedExpressions,
    readIdentifierPairedExpression: __1.NaiveParseSteps.readIdentifierPairedExpression,
    readGeneralizedIdentifierPairedExpressions: __1.NaiveParseSteps.readGeneralizedIdentifierPairedExpressions,
    readGeneralizedIdentifierPairedExpression: __1.NaiveParseSteps.readGeneralizedIdentifierPairedExpression,
};
// First, read all of the operators and operands.
// Second, continually combine them until we have a single Ast node.
function readBinOpExpression(state, parser, nodeKind, correlationId) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(commonTypes_1.CombinatorialParserV2TraceConstant.CombinatorialParseV2, readBinOpExpression.name, correlationId);
        const placeholderContextNode = parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const placeholderContextNodeId = placeholderContextNode.id;
        const operatorsAndOperands = yield (0, readOperatorsAndOperands_1.readOperatorsAndOperands)(state, parser, trace.id);
        let result;
        if (operatorsAndOperands.operands.length === 1) {
            result = common_1.ArrayUtils.assertGet(operatorsAndOperands.operands, 0);
        }
        else {
            result = (0, combineOperatorsAndOperands_1.combineOperatorsAndOperands)(state, parser, placeholderContextNodeId, operatorsAndOperands, trace.id);
        }
        parseState_1.ParseStateUtils.deleteContext(state, placeholderContextNode.id);
        trace.exit();
        return result;
    });
}
// If we look at the current token we might deterministically know what we'll end up trying to parse downstream.
// Eg. if we encounter a left brace, `{`, the only valid construct from this point is a list expression.
// In comparison to the naive parser we would travel down the following steps to reach the same outcome:
//  - readUnaryExpression
//  - readTypeExpression
//  - readPrimaryExpression
//  - readListExpression
function readUnaryExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry(commonTypes_1.CombinatorialParserV2TraceConstant.CombinatorialParseV2, readUnaryExpression.name, correlationId);
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        let primaryExpression;
        switch (state.currentTokenKind) {
            // PrimaryExpression
            case language_1.Token.TokenKind.AtSign:
            case language_1.Token.TokenKind.Identifier:
                primaryExpression = __1.NaiveParseSteps.readIdentifierExpression(state, parser, trace.id);
                break;
            case language_1.Token.TokenKind.LeftParenthesis:
                primaryExpression = yield __1.NaiveParseSteps.readParenthesizedExpression(state, parser, trace.id);
                break;
            case language_1.Token.TokenKind.LeftBracket:
                primaryExpression = yield disambiguation_1.DisambiguationUtils.readAmbiguousBracket(state, parser, [
                    disambiguation_1.Disambiguation.BracketDisambiguation.RecordExpression,
                    disambiguation_1.Disambiguation.BracketDisambiguation.FieldSelection,
                    disambiguation_1.Disambiguation.BracketDisambiguation.FieldProjection,
                ], trace.id);
                break;
            case language_1.Token.TokenKind.LeftBrace:
                primaryExpression = yield __1.NaiveParseSteps.readListExpression(state, parser, trace.id);
                break;
            case language_1.Token.TokenKind.Ellipsis:
                primaryExpression = __1.NaiveParseSteps.readNotImplementedExpression(state, parser, trace.id);
                break;
            // LiteralExpression
            case language_1.Token.TokenKind.HexLiteral:
            case language_1.Token.TokenKind.KeywordFalse:
            case language_1.Token.TokenKind.KeywordTrue:
            case language_1.Token.TokenKind.NumericLiteral:
            case language_1.Token.TokenKind.NullLiteral:
            case language_1.Token.TokenKind.TextLiteral:
                trace.exit();
                return __1.NaiveParseSteps.readLiteralExpression(state, parser, trace.id);
            // TypeExpression
            case language_1.Token.TokenKind.KeywordType:
                trace.exit();
                return __1.NaiveParseSteps.readTypeExpression(state, parser, trace.id);
            case language_1.Token.TokenKind.KeywordHashSections:
            case language_1.Token.TokenKind.KeywordHashShared:
            case language_1.Token.TokenKind.KeywordHashBinary:
            case language_1.Token.TokenKind.KeywordHashDate:
            case language_1.Token.TokenKind.KeywordHashDateTime:
            case language_1.Token.TokenKind.KeywordHashDateTimeZone:
            case language_1.Token.TokenKind.KeywordHashDuration:
            case language_1.Token.TokenKind.KeywordHashTable:
            case language_1.Token.TokenKind.KeywordHashTime:
                primaryExpression = parser.readKeyword(state, parser, trace.id);
                break;
            // Let NaiveParseSteps throw an error.
            default:
                trace.exit();
                return __1.NaiveParseSteps.readUnaryExpression(state, parser, trace.id);
        }
        let result;
        if (parseState_1.ParseStateUtils.isRecursivePrimaryExpressionNext(state, state.tokenIndex)) {
            result = yield parser.readRecursivePrimaryExpression(state, parser, primaryExpression, trace.id);
        }
        else {
            result = primaryExpression;
        }
        trace.exit();
        return result;
    });
}
//# sourceMappingURL=combinatorialParserV2.js.map