import { Ast, Constant } from "../../language";
import { ParseError } from "..";
import { Parser } from "../parser";
import { ParseState } from "../parseState";
import { TokenKind } from "../../language/token";
export declare function readIdentifier(state: ParseState, _parser: Parser, identifierContextKind: Ast.IdentifierContextKind, correlationId: number | undefined): Ast.Identifier;
export declare function readGeneralizedIdentifier(state: ParseState, _parser: Parser, correlationId: number | undefined): Promise<Ast.GeneralizedIdentifier>;
export declare function readKeyword(state: ParseState, _parser: Parser, correlationId: number | undefined): Ast.IdentifierExpression;
export declare function readDocument(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TDocument>;
export declare function readSectionDocument(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.Section>;
export declare function readSectionMembers(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.IArrayWrapper<Ast.SectionMember>>;
export declare function readSectionMember(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.SectionMember>;
export declare function readNullCoalescingExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TExpression>;
export declare function readExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TExpression>;
export declare function readLogicalExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TLogicalExpression>;
export declare function readIsExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TIsExpression>;
export declare function readNullablePrimitiveType(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TNullablePrimitiveType>;
export declare function readAsExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TAsExpression>;
export declare function readEqualityExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TEqualityExpression>;
export declare function readRelationalExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TRelationalExpression>;
export declare function readArithmeticExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TArithmeticExpression>;
export declare function readMetadataExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TMetadataExpression>;
export declare function readUnaryExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TUnaryExpression>;
export declare function readPrimaryExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TPrimaryExpression>;
export declare function readRecursivePrimaryExpression(state: ParseState, parser: Parser, head: Ast.TPrimaryExpression, correlationId: number | undefined): Promise<Ast.RecursivePrimaryExpression>;
export declare function readLiteralExpression(state: ParseState, _parser: Parser, correlationId: number | undefined): Ast.LiteralExpression;
export declare function readIdentifierExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Ast.IdentifierExpression;
export declare function readParenthesizedExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.ParenthesizedExpression>;
export declare function readNotImplementedExpression(state: ParseState, _parser: Parser, correlationId: number | undefined): Ast.NotImplementedExpression;
export declare function readInvokeExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.InvokeExpression>;
export declare function readListExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.ListExpression>;
export declare function readListItem(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TListItem>;
export declare function readRecordExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.RecordExpression>;
export declare function readItemAccessExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.ItemAccessExpression>;
export declare function readFieldSelection(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.FieldSelector>;
export declare function readFieldProjection(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.FieldProjection>;
export declare function readFieldSelector(state: ParseState, parser: Parser, allowOptional: boolean, correlationId: number | undefined): Promise<Ast.FieldSelector>;
export declare function readFunctionExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.FunctionExpression>;
export declare function readParameterList(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.IParameterList<Ast.AsNullablePrimitiveType | undefined>>;
export declare function readAsType(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.AsType>;
export declare function readEachExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.EachExpression>;
export declare function readLetExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.LetExpression>;
export declare function readIfExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.IfExpression>;
export declare function readTypeExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TTypeExpression>;
export declare function readType(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TType>;
export declare function readPrimaryType(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TPrimaryType>;
export declare function readRecordType(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.RecordType>;
export declare function readTableType(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TableType>;
export declare function readFieldSpecificationList(state: ParseState, parser: Parser, allowOpenMarker: boolean, testPostCommaError: (state: ParseState) => ParseError.TInnerParseError | undefined, correlationId: number | undefined): Promise<Ast.FieldSpecificationList>;
export declare function readListType(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.ListType>;
export declare function readFunctionType(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.FunctionType>;
export declare function readParameterSpecificationList(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.IParameterList<Ast.AsType>>;
export declare function readNullableType(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.NullableType>;
export declare function readErrorRaisingExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.ErrorRaisingExpression>;
export declare function readErrorHandlingExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TErrorHandlingExpression>;
export declare function readRecordLiteral(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.RecordLiteral>;
export declare function readFieldNamePairedAnyLiterals(state: ParseState, parser: Parser, continueReadingValues: boolean, testPostCommaError: (state: ParseState) => ParseError.TInnerParseError | undefined, correlationId: number | undefined): Promise<Ast.ICsvArray<Ast.GeneralizedIdentifierPairedAnyLiteral>>;
export declare function readListLiteral(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.ListLiteral>;
export declare function readAnyLiteral(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.TAnyLiteral>;
export declare function readPrimitiveType(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.PrimitiveType>;
export declare function readIdentifierPairedExpressions(state: ParseState, parser: Parser, continueReadingValues: boolean, testPostCommaError: (state: ParseState) => ParseError.TInnerParseError | undefined, correlationId: number | undefined): Promise<Ast.ICsvArray<Ast.IdentifierPairedExpression>>;
export declare function readGeneralizedIdentifierPairedExpressions(state: ParseState, parser: Parser, continueReadingValues: boolean, testPostCommaError: (state: ParseState) => ParseError.TInnerParseError | undefined, correlationId: number | undefined): Promise<Ast.ICsvArray<Ast.GeneralizedIdentifierPairedExpression>>;
export declare function readGeneralizedIdentifierPairedExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.GeneralizedIdentifierPairedExpression>;
export declare function readIdentifierPairedExpression(state: ParseState, parser: Parser, correlationId: number | undefined): Promise<Ast.IdentifierPairedExpression>;
interface OperatorTokenKindByLeftBaseBinOpExpressionKind {
    [Ast.NodeKind.AsExpression]: TokenKind.KeywordAs;
    [Ast.NodeKind.IsExpression]: TokenKind.KeywordIs;
    [Ast.NodeKind.NullCoalescingExpression]: TokenKind.NullCoalescingOperator;
}
export declare function readLeftBasedBinOpExpression<Node extends Ast.AsExpression | Ast.IsExpression | Ast.NullCoalescingExpression>(state: ParseState, nodeKind: Node["kind"], initialLeft: Node["left"], operatorTokenKind: OperatorTokenKindByLeftBaseBinOpExpressionKind[Node["kind"]], operatorConstantKind: Node["operatorConstant"]["constantKind"], readRight: (correlationId: number) => Promise<Node["right"]>, correlationId: number | undefined): Promise<Node | Node["left"]>;
export declare function readToken(state: ParseState): string;
export declare function readClosingTokenKindAsConstant<C extends Constant.TConstant>(state: ParseState, tokenKind: TokenKind, constantKind: C, correlationId: number | undefined): Ast.TConstant & Ast.IConstant<C>;
export declare function readTokenKindAsConstant<C extends Constant.TConstant>(state: ParseState, tokenKind: TokenKind, constantKind: C, correlationId: number | undefined): Ast.TConstant & Ast.IConstant<C>;
export declare function readTokenKindAsConstantOrUndefined<ConstantKind extends Constant.TConstant>(state: ParseState, tokenKind: TokenKind, constantKind: ConstantKind): (Ast.TConstant & Ast.IConstant<ConstantKind>) | undefined;
export {};
