"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.readErrorHandlingExpression = exports.readErrorRaisingExpression = exports.readNullableType = exports.readParameterSpecificationList = exports.readFunctionType = exports.readListType = exports.readFieldSpecificationList = exports.readTableType = exports.readRecordType = exports.readPrimaryType = exports.readType = exports.readTypeExpression = exports.readIfExpression = exports.readLetExpression = exports.readEachExpression = exports.readAsType = exports.readParameterList = exports.readFunctionExpression = exports.readFieldSelector = exports.readFieldProjection = exports.readFieldSelection = exports.readItemAccessExpression = exports.readRecordExpression = exports.readListItem = exports.readListExpression = exports.readInvokeExpression = exports.readNotImplementedExpression = exports.readParenthesizedExpression = exports.readIdentifierExpression = exports.readLiteralExpression = exports.readRecursivePrimaryExpression = exports.readPrimaryExpression = exports.readUnaryExpression = exports.readMetadataExpression = exports.readArithmeticExpression = exports.readRelationalExpression = exports.readEqualityExpression = exports.readAsExpression = exports.readNullablePrimitiveType = exports.readIsExpression = exports.readLogicalExpression = exports.readExpression = exports.readNullCoalescingExpression = exports.readSectionMember = exports.readSectionMembers = exports.readSectionDocument = exports.readDocument = exports.readKeyword = exports.readGeneralizedIdentifier = exports.readIdentifier = void 0;
exports.readTokenKindAsConstantOrUndefined = exports.readTokenKindAsConstant = exports.readClosingTokenKindAsConstant = exports.readToken = exports.readLeftBasedBinOpExpression = exports.readIdentifierPairedExpression = exports.readGeneralizedIdentifierPairedExpression = exports.readGeneralizedIdentifierPairedExpressions = exports.readIdentifierPairedExpressions = exports.readPrimitiveType = exports.readAnyLiteral = exports.readListLiteral = exports.readFieldNamePairedAnyLiterals = exports.readRecordLiteral = void 0;
const common_1 = require("../../common");
const language_1 = require("../../language");
const disambiguation_1 = require("../disambiguation");
const __1 = require("..");
const parseState_1 = require("../parseState");
const trace_1 = require("../../common/trace");
const token_1 = require("../../language/token");
const GeneralizedIdentifierTerminatorTokenKinds = [
    token_1.TokenKind.Comma,
    token_1.TokenKind.Equal,
    token_1.TokenKind.RightBracket,
];
// ----------------------------------------
// ---------- 12.1.6 Identifiers ----------
// ----------------------------------------
function readIdentifier(state, _parser, identifierContextKind, correlationId) {
    const nodeKind = language_1.Ast.NodeKind.Identifier;
    const trace = state.traceManager.entry("Parse" /* Parse */, readIdentifier.name, correlationId, {
        ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
    });
    parseState_1.ParseStateUtils.startContext(state, nodeKind);
    const literal = readTokenKind(state, token_1.TokenKind.Identifier);
    const identifier = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: true, identifierContextKind,
        literal });
    parseState_1.ParseStateUtils.endContext(state, identifier);
    trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
    return identifier;
}
exports.readIdentifier = readIdentifier;
// This behavior matches the C# parser and not the language specification.
// eslint-disable-next-line require-await
function readGeneralizedIdentifier(state, _parser, correlationId) {
    return __awaiter(this, void 0, void 0, function* () {
        const nodeKind = language_1.Ast.NodeKind.GeneralizedIdentifier;
        const trace = state.traceManager.entry("Parse" /* Parse */, readGeneralizedIdentifier.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const tokenRangeStartIndex = state.tokenIndex;
        let tokenRangeEndIndex = tokenRangeStartIndex;
        while (state.currentTokenKind && !GeneralizedIdentifierTerminatorTokenKinds.includes(state.currentTokenKind)) {
            readToken(state);
            tokenRangeEndIndex = state.tokenIndex;
        }
        if (tokenRangeStartIndex === tokenRangeEndIndex) {
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                [trace_1.TraceConstant.IsThrowing]: true,
            });
            throw new __1.ParseError.ExpectedGeneralizedIdentifierError(parseState_1.ParseStateUtils.tokenWithColumnNumber(state, state.tokenIndex + 1), state.locale);
        }
        const lexerSnapshot = state.lexerSnapshot;
        const tokens = lexerSnapshot.tokens;
        const contiguousIdentifierStartIndex = tokens[tokenRangeStartIndex].positionStart.codeUnit;
        const contiguousIdentifierEndIndex = tokens[tokenRangeEndIndex - 1].positionEnd.codeUnit;
        const literal = lexerSnapshot.text.slice(contiguousIdentifierStartIndex, contiguousIdentifierEndIndex);
        const literalKind = language_1.IdentifierUtils.getIdentifierKind(literal, true);
        if (literalKind === language_1.IdentifierUtils.IdentifierKind.Invalid) {
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                [trace_1.TraceConstant.IsThrowing]: true,
            });
            throw new __1.ParseError.ExpectedGeneralizedIdentifierError(parseState_1.ParseStateUtils.tokenWithColumnNumber(state, state.tokenIndex + 1), state.locale);
        }
        const generalizedIdentifier = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: true, literal });
        parseState_1.ParseStateUtils.endContext(state, generalizedIdentifier);
        trace.exit({
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
            [trace_1.TraceConstant.IsThrowing]: false,
        });
        return generalizedIdentifier;
    });
}
exports.readGeneralizedIdentifier = readGeneralizedIdentifier;
function readKeyword(state, _parser, correlationId) {
    const nodeKind = language_1.Ast.NodeKind.IdentifierExpression;
    const trace = state.traceManager.entry("Parse" /* Parse */, readKeyword.name, correlationId, {
        ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
    });
    parseState_1.ParseStateUtils.startContext(state, nodeKind);
    // Keywords can't have a "@" prefix constant
    parseState_1.ParseStateUtils.incrementAttributeCounter(state);
    const identifierNodeKind = language_1.Ast.NodeKind.Identifier;
    parseState_1.ParseStateUtils.startContext(state, identifierNodeKind);
    const literal = readToken(state);
    const identifier = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: identifierNodeKind, isLeaf: true, identifierContextKind: language_1.Ast.IdentifierContextKind.Keyword, literal });
    parseState_1.ParseStateUtils.endContext(state, identifier);
    const identifierExpression = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, inclusiveConstant: undefined, identifier });
    parseState_1.ParseStateUtils.endContext(state, identifierExpression);
    trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
    return identifierExpression;
}
exports.readKeyword = readKeyword;
// --------------------------------------
// ---------- 12.2.1 Documents ----------
// --------------------------------------
function readDocument(state, parser, correlationId) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readDocument.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        let document;
        // Try parsing as an Expression document first.
        // If Expression document fails (including UnusedTokensRemainError) then try parsing a SectionDocument.
        // If both fail then return the error which parsed more tokens.
        try {
            document = yield parser.readExpression(state, parser, trace.id);
            parseState_1.ParseStateUtils.assertIsDoneParsing(state);
        }
        catch (expressionError) {
            common_1.Assert.isInstanceofError(expressionError);
            common_1.CommonError.throwIfCancellationError(expressionError);
            // Fast backup deletes context state, but we want to preserve it for the case
            // where both parsing an expression and section document error out.
            const expressionCheckpoint = yield parser.checkpoint(state);
            const expressionErrorContextState = state.contextState;
            // Reset the parser's state.
            state.tokenIndex = 0;
            state.contextState = __1.ParseContextUtils.newState();
            state.currentContextNode = undefined;
            if (state.lexerSnapshot.tokens.length) {
                state.currentToken = state.lexerSnapshot.tokens[0];
                state.currentTokenKind = (_b = state.currentToken) === null || _b === void 0 ? void 0 : _b.kind;
            }
            try {
                document = yield readSectionDocument(state, parser, trace.id);
                parseState_1.ParseStateUtils.assertIsDoneParsing(state);
            }
            catch (sectionError) {
                common_1.Assert.isInstanceofError(sectionError);
                common_1.CommonError.throwIfCancellationError(sectionError);
                let triedError;
                if (expressionCheckpoint.tokenIndex > /* sectionErrorState */ state.tokenIndex) {
                    triedError = expressionError;
                    yield parser.restoreCheckpoint(state, expressionCheckpoint);
                    // eslint-disable-next-line require-atomic-updates
                    state.contextState = expressionErrorContextState;
                }
                else {
                    triedError = sectionError;
                }
                trace.exit({
                    ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                    [trace_1.TraceConstant.IsThrowing]: true,
                });
                throw triedError;
            }
        }
        trace.exit({
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
            [trace_1.TraceConstant.IsThrowing]: false,
        });
        return document;
    });
}
exports.readDocument = readDocument;
// ----------------------------------------------
// ---------- 12.2.2 Section Documents ----------
// ----------------------------------------------
function readSectionDocument(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const nodeKind = language_1.Ast.NodeKind.Section;
        const trace = state.traceManager.entry("Parse" /* Parse */, readSectionDocument.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const literalAttributes = yield readLiteralAttributes(state, parser, trace.id);
        const sectionConstant = readTokenKindAsConstant(state, token_1.TokenKind.KeywordSection, language_1.Constant.KeywordConstant.Section, trace.id);
        let name;
        if (parseState_1.ParseStateUtils.isOnTokenKind(state, token_1.TokenKind.Identifier)) {
            name = parser.readIdentifier(state, parser, language_1.Ast.IdentifierContextKind.Key, trace.id);
        }
        else {
            parseState_1.ParseStateUtils.incrementAttributeCounter(state);
        }
        const semicolonConstant = readTokenKindAsConstant(state, token_1.TokenKind.Semicolon, language_1.Constant.MiscConstant.Semicolon, trace.id);
        const sectionMembers = yield parser.readSectionMembers(state, parser, trace.id);
        const section = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, literalAttributes,
            sectionConstant,
            name,
            semicolonConstant,
            sectionMembers });
        parseState_1.ParseStateUtils.endContext(state, section);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return section;
    });
}
exports.readSectionDocument = readSectionDocument;
function readSectionMembers(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const nodeKind = language_1.Ast.NodeKind.ArrayWrapper;
        const trace = state.traceManager.entry("Parse" /* Parse */, readSectionMembers.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const totalTokens = state.lexerSnapshot.tokens.length;
        const sectionMembers = [];
        while (state.tokenIndex < totalTokens) {
            // eslint-disable-next-line no-await-in-loop
            sectionMembers.push(yield parser.readSectionMember(state, parser, trace.id));
        }
        const sectionMemberArray = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, elements: sectionMembers });
        parseState_1.ParseStateUtils.endContext(state, sectionMemberArray);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return sectionMemberArray;
    });
}
exports.readSectionMembers = readSectionMembers;
function readSectionMember(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const nodeKind = language_1.Ast.NodeKind.SectionMember;
        const trace = state.traceManager.entry("Parse" /* Parse */, readSectionMember.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const literalAttributes = yield readLiteralAttributes(state, parser, trace.id);
        const sharedConstant = readTokenKindAsConstantOrUndefined(state, token_1.TokenKind.KeywordShared, language_1.Constant.KeywordConstant.Shared);
        const namePairedExpression = yield parser.readIdentifierPairedExpression(state, parser, trace.id);
        const semicolonConstant = readTokenKindAsConstant(state, token_1.TokenKind.Semicolon, language_1.Constant.MiscConstant.Semicolon, trace.id);
        const sectionMember = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, literalAttributes,
            sharedConstant,
            namePairedExpression,
            semicolonConstant });
        parseState_1.ParseStateUtils.endContext(state, sectionMember);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return sectionMember;
    });
}
exports.readSectionMember = readSectionMember;
// ------------------------------------------
// ---------- 12.2.3.1 Expressions ----------
// ------------------------------------------
// ------------------------------------
// ---------- NullCoalescing ----------
// ------------------------------------
function readNullCoalescingExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readNullCoalescingExpression.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const nullCoalescingExpression = yield readLeftBasedBinOpExpression(state, language_1.Ast.NodeKind.NullCoalescingExpression, yield parser.readLogicalExpression(state, parser, correlationId), token_1.TokenKind.NullCoalescingOperator, language_1.Constant.MiscConstant.NullCoalescingOperator, (correlationId) => parser.readLogicalExpression(state, parser, correlationId), trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return nullCoalescingExpression;
    });
}
exports.readNullCoalescingExpression = readNullCoalescingExpression;
function readExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readExpression.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        let expression;
        switch (state.currentTokenKind) {
            case token_1.TokenKind.KeywordEach:
                expression = yield parser.readEachExpression(state, parser, trace.id);
                break;
            case token_1.TokenKind.KeywordLet:
                expression = yield parser.readLetExpression(state, parser, trace.id);
                break;
            case token_1.TokenKind.KeywordIf:
                expression = yield parser.readIfExpression(state, parser, trace.id);
                break;
            case token_1.TokenKind.KeywordError:
                expression = yield parser.readErrorRaisingExpression(state, parser, trace.id);
                break;
            case token_1.TokenKind.KeywordTry:
                expression = yield parser.readErrorHandlingExpression(state, parser, trace.id);
                break;
            case token_1.TokenKind.LeftParenthesis:
                expression = yield disambiguation_1.DisambiguationUtils.readAmbiguousParenthesis(state, parser, trace.id);
                break;
            default:
                expression = yield parser.readNullCoalescingExpression(state, parser, trace.id);
                break;
        }
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return expression;
    });
}
exports.readExpression = readExpression;
// --------------------------------------------------
// ---------- 12.2.3.2 Logical expressions ----------
// --------------------------------------------------
function readLogicalExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readLogicalExpression.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const logicalExpression = yield recursiveReadBinOpExpression(state, language_1.Ast.NodeKind.LogicalExpression, () => parser.readIsExpression(state, parser, trace.id), (currentTokenKind) => language_1.ConstantUtils.logicalOperatorKindFrom(currentTokenKind), () => parser.readIsExpression(state, parser, trace.id), trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return logicalExpression;
    });
}
exports.readLogicalExpression = readLogicalExpression;
// --------------------------------------------
// ---------- 12.2.3.3 Is expression ----------
// --------------------------------------------
function readIsExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readIsExpression.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const isExpression = yield readLeftBasedBinOpExpression(state, language_1.Ast.NodeKind.IsExpression, yield parser.readAsExpression(state, parser, correlationId), token_1.TokenKind.KeywordIs, language_1.Constant.KeywordConstant.Is, (correlationId) => parser.readNullablePrimitiveType(state, parser, correlationId), trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return isExpression;
    });
}
exports.readIsExpression = readIsExpression;
// sub-item of 12.2.3.3 Is expression
function readNullablePrimitiveType(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readNullablePrimitiveType.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        let nullablePrimitiveType;
        if (parseState_1.ParseStateUtils.isOnConstantKind(state, language_1.Constant.LanguageConstant.Nullable)) {
            nullablePrimitiveType = yield readPairedConstant(state, language_1.Ast.NodeKind.NullablePrimitiveType, () => readConstantKind(state, language_1.Constant.LanguageConstant.Nullable), () => parser.readPrimitiveType(state, parser, trace.id), trace.id);
        }
        else {
            nullablePrimitiveType = yield parser.readPrimitiveType(state, parser, trace.id);
        }
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return nullablePrimitiveType;
    });
}
exports.readNullablePrimitiveType = readNullablePrimitiveType;
// --------------------------------------------
// ---------- 12.2.3.4 As expression ----------
// --------------------------------------------
function readAsExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readAsExpression.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const asExpression = yield readLeftBasedBinOpExpression(state, language_1.Ast.NodeKind.AsExpression, yield parser.readEqualityExpression(state, parser, correlationId), token_1.TokenKind.KeywordAs, language_1.Constant.KeywordConstant.As, (correlationId) => parser.readNullablePrimitiveType(state, parser, correlationId), trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return asExpression;
    });
}
exports.readAsExpression = readAsExpression;
// --------------------------------------------------
// ---------- 12.2.3.5 Equality expression ----------
// --------------------------------------------------
function readEqualityExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readEqualityExpression.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const equalityExpression = yield recursiveReadBinOpExpression(state, language_1.Ast.NodeKind.EqualityExpression, () => parser.readRelationalExpression(state, parser, trace.id), (currentTokenKind) => language_1.ConstantUtils.equalityOperatorKindFrom(currentTokenKind), () => parser.readRelationalExpression(state, parser, trace.id), trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return equalityExpression;
    });
}
exports.readEqualityExpression = readEqualityExpression;
// ----------------------------------------------------
// ---------- 12.2.3.6 Relational expression ----------
// ----------------------------------------------------
function readRelationalExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readRelationalExpression.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const relationalExpression = yield recursiveReadBinOpExpression(state, language_1.Ast.NodeKind.RelationalExpression, () => parser.readArithmeticExpression(state, parser, trace.id), (currentTokenKind) => language_1.ConstantUtils.relationalOperatorKindFrom(currentTokenKind), () => parser.readArithmeticExpression(state, parser, trace.id), trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return relationalExpression;
    });
}
exports.readRelationalExpression = readRelationalExpression;
// -----------------------------------------------------
// ---------- 12.2.3.7 Arithmetic expressions ----------
// -----------------------------------------------------
function readArithmeticExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readArithmeticExpression.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const arithmeticExpression = yield recursiveReadBinOpExpression(state, language_1.Ast.NodeKind.ArithmeticExpression, () => parser.readMetadataExpression(state, parser, trace.id), (currentTokenKind) => language_1.ConstantUtils.arithmeticOperatorKindFrom(currentTokenKind), () => parser.readMetadataExpression(state, parser, trace.id), trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return arithmeticExpression;
    });
}
exports.readArithmeticExpression = readArithmeticExpression;
// --------------------------------------------------
// ---------- 12.2.3.8 Metadata expression ----------
// --------------------------------------------------
function readMetadataExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const nodeKind = language_1.Ast.NodeKind.MetadataExpression;
        const trace = state.traceManager.entry("Parse" /* Parse */, readMetadataExpression.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const left = yield parser.readUnaryExpression(state, parser, trace.id);
        const metaConstant = readTokenKindAsConstantOrUndefined(state, token_1.TokenKind.KeywordMeta, language_1.Constant.KeywordConstant.Meta);
        if (metaConstant !== undefined) {
            const operatorConstant = metaConstant;
            const right = yield parser.readUnaryExpression(state, parser, trace.id);
            const metadataExpression = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, left,
                operatorConstant,
                right });
            parseState_1.ParseStateUtils.endContext(state, metadataExpression);
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                ["IsOperatorPresent" /* IsOperatorPresent */]: true,
            });
            return metadataExpression;
        }
        else {
            parseState_1.ParseStateUtils.deleteContext(state);
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                ["IsOperatorPresent" /* IsOperatorPresent */]: false,
            });
            return left;
        }
    });
}
exports.readMetadataExpression = readMetadataExpression;
// -----------------------------------------------
// ---------- 12.2.3.9 Unary expression ----------
// -----------------------------------------------
function readUnaryExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readUnaryExpression.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        let operator = language_1.ConstantUtils.unaryOperatorKindFrom(state.currentTokenKind);
        if (operator === undefined) {
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                ["IsOperatorPresent" /* IsOperatorPresent */]: false,
            });
            return parser.readTypeExpression(state, parser, trace.id);
        }
        const unaryNodeKind = language_1.Ast.NodeKind.UnaryExpression;
        parseState_1.ParseStateUtils.startContext(state, unaryNodeKind);
        const arrayNodeKind = language_1.Ast.NodeKind.ArrayWrapper;
        parseState_1.ParseStateUtils.startContext(state, arrayNodeKind);
        const operatorConstants = [];
        while (operator) {
            operatorConstants.push(readTokenKindAsConstant(state, state.currentTokenKind, operator, trace.id));
            operator = language_1.ConstantUtils.unaryOperatorKindFrom(state.currentTokenKind);
        }
        const operators = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: arrayNodeKind, isLeaf: false, elements: operatorConstants });
        parseState_1.ParseStateUtils.endContext(state, operators);
        const typeExpression = yield parser.readTypeExpression(state, parser, trace.id);
        const unaryExpression = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: unaryNodeKind, isLeaf: false, operators,
            typeExpression });
        parseState_1.ParseStateUtils.endContext(state, unaryExpression);
        trace.exit({
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
            ["IsOperatorPresent" /* IsOperatorPresent */]: true,
        });
        return unaryExpression;
    });
}
exports.readUnaryExpression = readUnaryExpression;
// --------------------------------------------------
// ---------- 12.2.3.10 Primary expression ----------
// --------------------------------------------------
function readPrimaryExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readPrimaryExpression.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        let primaryExpression;
        const currentTokenKind = state.currentTokenKind;
        const isIdentifierExpressionNext = currentTokenKind === token_1.TokenKind.AtSign || currentTokenKind === token_1.TokenKind.Identifier;
        if (isIdentifierExpressionNext) {
            primaryExpression = parser.readIdentifierExpression(state, parser, trace.id);
        }
        else {
            switch (currentTokenKind) {
                case token_1.TokenKind.LeftParenthesis:
                    primaryExpression = yield parser.readParenthesizedExpression(state, parser, trace.id);
                    break;
                case token_1.TokenKind.LeftBracket:
                    primaryExpression = yield disambiguation_1.DisambiguationUtils.readAmbiguousBracket(state, parser, [
                        disambiguation_1.Disambiguation.BracketDisambiguation.FieldProjection,
                        disambiguation_1.Disambiguation.BracketDisambiguation.FieldSelection,
                        disambiguation_1.Disambiguation.BracketDisambiguation.RecordExpression,
                    ], trace.id);
                    break;
                case token_1.TokenKind.LeftBrace:
                    primaryExpression = yield parser.readListExpression(state, parser, trace.id);
                    break;
                case token_1.TokenKind.Ellipsis:
                    primaryExpression = parser.readNotImplementedExpression(state, parser, trace.id);
                    break;
                case token_1.TokenKind.KeywordHashSections:
                case token_1.TokenKind.KeywordHashShared:
                case token_1.TokenKind.KeywordHashBinary:
                case token_1.TokenKind.KeywordHashDate:
                case token_1.TokenKind.KeywordHashDateTime:
                case token_1.TokenKind.KeywordHashDateTimeZone:
                case token_1.TokenKind.KeywordHashDuration:
                case token_1.TokenKind.KeywordHashTable:
                case token_1.TokenKind.KeywordHashTime:
                    primaryExpression = parser.readKeyword(state, parser, trace.id);
                    break;
                default:
                    primaryExpression = parser.readLiteralExpression(state, parser, trace.id);
            }
        }
        if (parseState_1.ParseStateUtils.isRecursivePrimaryExpressionNext(state)) {
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                ["IsRecursive" /* IsRecursive */]: true,
            });
            return parser.readRecursivePrimaryExpression(state, parser, primaryExpression, trace.id);
        }
        else {
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                ["IsRecursive" /* IsRecursive */]: false,
            });
            return primaryExpression;
        }
    });
}
exports.readPrimaryExpression = readPrimaryExpression;
function readRecursivePrimaryExpression(state, parser, head, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const nodeKind = language_1.Ast.NodeKind.RecursivePrimaryExpression;
        const trace = state.traceManager.entry("Parse" /* Parse */, readRecursivePrimaryExpression.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        state.currentContextNode = parseState_1.ParseStateUtils.startContextAsParent(state, nodeKind, head.id, trace.id);
        const recursiveArrayNodeKind = language_1.Ast.NodeKind.ArrayWrapper;
        parseState_1.ParseStateUtils.startContext(state, recursiveArrayNodeKind);
        const recursiveExpressions = [];
        let continueReadingValues = true;
        while (continueReadingValues) {
            const currentTokenKind = state.currentTokenKind;
            if (currentTokenKind === token_1.TokenKind.LeftParenthesis) {
                // eslint-disable-next-line no-await-in-loop
                recursiveExpressions.push(yield parser.readInvokeExpression(state, parser, trace.id));
            }
            else if (currentTokenKind === token_1.TokenKind.LeftBrace) {
                // eslint-disable-next-line no-await-in-loop
                recursiveExpressions.push(yield parser.readItemAccessExpression(state, parser, trace.id));
            }
            else if (currentTokenKind === token_1.TokenKind.LeftBracket) {
                // eslint-disable-next-line no-await-in-loop
                const bracketExpression = (yield disambiguation_1.DisambiguationUtils.readAmbiguousBracket(state, parser, [
                    disambiguation_1.Disambiguation.BracketDisambiguation.FieldSelection,
                    disambiguation_1.Disambiguation.BracketDisambiguation.FieldProjection,
                ], trace.id));
                recursiveExpressions.push(bracketExpression);
            }
            else {
                continueReadingValues = false;
            }
        }
        const recursiveArray = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: recursiveArrayNodeKind, isLeaf: false, elements: recursiveExpressions });
        parseState_1.ParseStateUtils.endContext(state, recursiveArray);
        const recursivePrimaryExpression = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, head, recursiveExpressions: recursiveArray });
        parseState_1.ParseStateUtils.endContext(state, recursivePrimaryExpression);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return recursivePrimaryExpression;
    });
}
exports.readRecursivePrimaryExpression = readRecursivePrimaryExpression;
// --------------------------------------------------
// ---------- 12.2.3.11 Literal expression ----------
// --------------------------------------------------
function readLiteralExpression(state, _parser, correlationId) {
    var _a;
    const nodeKind = language_1.Ast.NodeKind.LiteralExpression;
    const trace = state.traceManager.entry("Parse" /* Parse */, readLiteralExpression.name, correlationId, {
        ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
    });
    (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
    parseState_1.ParseStateUtils.startContext(state, nodeKind);
    const expectedTokenKinds = [
        token_1.TokenKind.HexLiteral,
        token_1.TokenKind.KeywordFalse,
        token_1.TokenKind.KeywordHashInfinity,
        token_1.TokenKind.KeywordHashNan,
        token_1.TokenKind.KeywordTrue,
        token_1.TokenKind.NumericLiteral,
        token_1.TokenKind.NullLiteral,
        token_1.TokenKind.TextLiteral,
    ];
    const error = parseState_1.ParseStateUtils.testIsOnAnyTokenKind(state, expectedTokenKinds);
    if (error) {
        trace.exit({
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
            [trace_1.TraceConstant.IsThrowing]: true,
        });
        throw error;
    }
    const literalKind = common_1.Assert.asDefined(language_1.AstUtils.literalKindFrom(state.currentTokenKind), `couldn't convert TokenKind into LiteralKind`, { currentTokenKind: state.currentTokenKind });
    const literal = readToken(state);
    const literalExpression = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: true, literal,
        literalKind });
    parseState_1.ParseStateUtils.endContext(state, literalExpression);
    trace.exit({
        ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        [trace_1.TraceConstant.IsThrowing]: false,
    });
    return literalExpression;
}
exports.readLiteralExpression = readLiteralExpression;
// -----------------------------------------------------
// ---------- 12.2.3.12 Identifier expression ----------
// -----------------------------------------------------
function readIdentifierExpression(state, parser, correlationId) {
    var _a;
    const nodeKind = language_1.Ast.NodeKind.IdentifierExpression;
    const trace = state.traceManager.entry("Parse" /* Parse */, readIdentifierExpression.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
    (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
    parseState_1.ParseStateUtils.startContext(state, nodeKind);
    const inclusiveConstant = readTokenKindAsConstantOrUndefined(state, token_1.TokenKind.AtSign, language_1.Constant.MiscConstant.AtSign);
    const identifier = parser.readIdentifier(state, parser, language_1.Ast.IdentifierContextKind.Value, trace.id);
    const identifierExpression = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, inclusiveConstant,
        identifier });
    parseState_1.ParseStateUtils.endContext(state, identifierExpression);
    trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
    return identifierExpression;
}
exports.readIdentifierExpression = readIdentifierExpression;
// --------------------------------------------------------
// ---------- 12.2.3.14 Parenthesized expression ----------
// --------------------------------------------------------
function readParenthesizedExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readParenthesizedExpression.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const parenthesizedExpression = yield readWrapped(state, language_1.Ast.NodeKind.ParenthesizedExpression, () => readTokenKindAsConstant(state, token_1.TokenKind.LeftParenthesis, language_1.Constant.WrapperConstant.LeftParenthesis, trace.id), () => parser.readExpression(state, parser, trace.id), () => readTokenKindAsConstant(state, token_1.TokenKind.RightParenthesis, language_1.Constant.WrapperConstant.RightParenthesis, trace.id), false, trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return parenthesizedExpression;
    });
}
exports.readParenthesizedExpression = readParenthesizedExpression;
// ----------------------------------------------------------
// ---------- 12.2.3.15 Not-implemented expression ----------
// ----------------------------------------------------------
function readNotImplementedExpression(state, _parser, correlationId) {
    var _a;
    const nodeKind = language_1.Ast.NodeKind.NotImplementedExpression;
    const trace = state.traceManager.entry("Parse" /* Parse */, readNotImplementedExpression.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
    (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
    parseState_1.ParseStateUtils.startContext(state, nodeKind);
    const ellipsisConstant = readTokenKindAsConstant(state, token_1.TokenKind.Ellipsis, language_1.Constant.MiscConstant.Ellipsis, trace.id);
    const notImplementedExpression = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, ellipsisConstant });
    parseState_1.ParseStateUtils.endContext(state, notImplementedExpression);
    trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
    return notImplementedExpression;
}
exports.readNotImplementedExpression = readNotImplementedExpression;
// ---------------------------------------
// ---------- Invoke expression ----------
// ---------------------------------------
function readInvokeExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readInvokeExpression.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const continueReadingValues = !parseState_1.ParseStateUtils.isNextTokenKind(state, token_1.TokenKind.RightParenthesis);
        const invokeExpression = yield readWrapped(state, language_1.Ast.NodeKind.InvokeExpression, () => readTokenKindAsConstant(state, token_1.TokenKind.LeftParenthesis, language_1.Constant.WrapperConstant.LeftParenthesis, trace.id), () => 
        // The type inference in VSCode considers the lambda below a type error, but it compiles just fine.
        // I'm adding an explicit type to stop it from (incorrectly) saying it's an error.
        readCsvArray(state, () => parser.readExpression(state, parser, trace.id), continueReadingValues, testCsvContinuationDanglingCommaForParenthesis, trace.id), () => readTokenKindAsConstant(state, token_1.TokenKind.RightParenthesis, language_1.Constant.WrapperConstant.RightParenthesis, trace.id), false, trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return invokeExpression;
    });
}
exports.readInvokeExpression = readInvokeExpression;
// -----------------------------------------------
// ---------- 12.2.3.17 List expression ----------
// -----------------------------------------------
function readListExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readListExpression.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const continueReadingValues = !parseState_1.ParseStateUtils.isNextTokenKind(state, token_1.TokenKind.RightBrace);
        const listExpression = yield readWrapped(state, language_1.Ast.NodeKind.ListExpression, () => readTokenKindAsConstant(state, token_1.TokenKind.LeftBrace, language_1.Constant.WrapperConstant.LeftBrace, trace.id), () => __awaiter(this, void 0, void 0, function* () {
            return yield readCsvArray(state, () => __awaiter(this, void 0, void 0, function* () { return yield parser.readListItem(state, parser, trace.id); }), continueReadingValues, testCsvContinuationDanglingCommaForBrace, trace.id);
        }), () => readClosingTokenKindAsConstant(state, token_1.TokenKind.RightBrace, language_1.Constant.WrapperConstant.RightBrace, trace.id), false, trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return listExpression;
    });
}
exports.readListExpression = readListExpression;
function readListItem(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const nodeKind = language_1.Ast.NodeKind.RangeExpression;
        const trace = state.traceManager.entry("Parse" /* Parse */, readListItem.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const left = yield parser.readExpression(state, parser, trace.id);
        if (parseState_1.ParseStateUtils.isOnTokenKind(state, token_1.TokenKind.DotDot)) {
            const rangeConstant = readTokenKindAsConstant(state, token_1.TokenKind.DotDot, language_1.Constant.MiscConstant.DotDot, trace.id);
            const right = yield parser.readExpression(state, parser, trace.id);
            const rangeExpression = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, left,
                rangeConstant,
                right });
            parseState_1.ParseStateUtils.endContext(state, rangeExpression);
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                ["IsOperatorPresent" /* IsOperatorPresent */]: true,
            });
            return rangeExpression;
        }
        else {
            parseState_1.ParseStateUtils.deleteContext(state);
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                ["IsOperatorPresent" /* IsOperatorPresent */]: false,
            });
            return left;
        }
    });
}
exports.readListItem = readListItem;
// -------------------------------------------------
// ---------- 12.2.3.18 Record expression ----------
// -------------------------------------------------
function readRecordExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readRecordExpression.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const continueReadingValues = !parseState_1.ParseStateUtils.isNextTokenKind(state, token_1.TokenKind.RightBracket);
        const recordExpression = yield readWrapped(state, language_1.Ast.NodeKind.RecordExpression, () => readTokenKindAsConstant(state, token_1.TokenKind.LeftBracket, language_1.Constant.WrapperConstant.LeftBracket, trace.id), () => parser.readGeneralizedIdentifierPairedExpressions(state, parser, continueReadingValues, testCsvContinuationDanglingCommaForBracket, trace.id), () => readClosingTokenKindAsConstant(state, token_1.TokenKind.RightBracket, language_1.Constant.WrapperConstant.RightBracket, trace.id), false, trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return recordExpression;
    });
}
exports.readRecordExpression = readRecordExpression;
// ------------------------------------------------------
// ---------- 12.2.3.19 Item access expression ----------
// ------------------------------------------------------
function readItemAccessExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readItemAccessExpression.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const itemAccessExpression = yield readWrapped(state, language_1.Ast.NodeKind.ItemAccessExpression, () => readTokenKindAsConstant(state, token_1.TokenKind.LeftBrace, language_1.Constant.WrapperConstant.LeftBrace, trace.id), () => __awaiter(this, void 0, void 0, function* () { return yield parser.readExpression(state, parser, trace.id); }), () => readTokenKindAsConstant(state, token_1.TokenKind.RightBrace, language_1.Constant.WrapperConstant.RightBrace, trace.id), true, trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return itemAccessExpression;
    });
}
exports.readItemAccessExpression = readItemAccessExpression;
// -------------------------------------------------------
// ---------- 12.2.3.20 Field access expression ----------
// -------------------------------------------------------
function readFieldSelection(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readFieldSelection.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const fieldSelector = yield readFieldSelector(state, parser, true, trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return fieldSelector;
    });
}
exports.readFieldSelection = readFieldSelection;
function readFieldProjection(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readFieldProjection.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const fieldProjection = yield readWrapped(state, language_1.Ast.NodeKind.FieldProjection, () => readTokenKindAsConstant(state, token_1.TokenKind.LeftBracket, language_1.Constant.WrapperConstant.LeftBracket, trace.id), () => readCsvArray(state, () => parser.readFieldSelector(state, parser, false, trace.id), true, testCsvContinuationDanglingCommaForBracket, trace.id), () => readTokenKindAsConstant(state, token_1.TokenKind.RightBracket, language_1.Constant.WrapperConstant.RightBracket, trace.id), true, trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return fieldProjection;
    });
}
exports.readFieldProjection = readFieldProjection;
function readFieldSelector(state, parser, allowOptional, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readFieldSelector.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const fieldSelector = yield readWrapped(state, language_1.Ast.NodeKind.FieldSelector, () => readTokenKindAsConstant(state, token_1.TokenKind.LeftBracket, language_1.Constant.WrapperConstant.LeftBracket, trace.id), () => parser.readGeneralizedIdentifier(state, parser, trace.id), () => readTokenKindAsConstant(state, token_1.TokenKind.RightBracket, language_1.Constant.WrapperConstant.RightBracket, trace.id), allowOptional, trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return fieldSelector;
    });
}
exports.readFieldSelector = readFieldSelector;
// ---------------------------------------------------
// ---------- 12.2.3.21 Function expression ----------
// ---------------------------------------------------
function readFunctionExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const nodeKind = language_1.Ast.NodeKind.FunctionExpression;
        const trace = state.traceManager.entry("Parse" /* Parse */, readFunctionExpression.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const parameters = yield parser.readParameterList(state, parser, trace.id);
        const functionReturnType = yield readAsNullablePrimitiveType(state, parser, trace.id);
        const fatArrowConstant = readTokenKindAsConstant(state, token_1.TokenKind.FatArrow, language_1.Constant.MiscConstant.FatArrow, trace.id);
        const expression = yield parser.readExpression(state, parser, trace.id);
        const functionExpression = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, parameters,
            functionReturnType,
            fatArrowConstant,
            expression });
        parseState_1.ParseStateUtils.endContext(state, functionExpression);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return functionExpression;
    });
}
exports.readFunctionExpression = readFunctionExpression;
function readParameterList(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readParameterList.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const parameterList = yield genericReadParameterList(state, parser, () => readAsNullablePrimitiveType(state, parser, trace.id), trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return parameterList;
    });
}
exports.readParameterList = readParameterList;
function readAsNullablePrimitiveType(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readAsNullablePrimitiveType.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const asNullablePrimitiveType = yield readPairedConstantOrUndefined(state, language_1.Ast.NodeKind.AsNullablePrimitiveType, () => parseState_1.ParseStateUtils.isOnTokenKind(state, token_1.TokenKind.KeywordAs), () => readTokenKindAsConstant(state, token_1.TokenKind.KeywordAs, language_1.Constant.KeywordConstant.As, trace.id), () => parser.readNullablePrimitiveType(state, parser, trace.id), trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return asNullablePrimitiveType;
    });
}
function readAsType(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readAsType.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const asType = yield readPairedConstant(state, language_1.Ast.NodeKind.AsType, () => readTokenKindAsConstant(state, token_1.TokenKind.KeywordAs, language_1.Constant.KeywordConstant.As, trace.id), () => parser.readType(state, parser, trace.id), trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return asType;
    });
}
exports.readAsType = readAsType;
// -----------------------------------------------
// ---------- 12.2.3.22 Each expression ----------
// -----------------------------------------------
function readEachExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readEachExpression.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const eachExpression = yield readPairedConstant(state, language_1.Ast.NodeKind.EachExpression, () => readTokenKindAsConstant(state, token_1.TokenKind.KeywordEach, language_1.Constant.KeywordConstant.Each, trace.id), () => parser.readExpression(state, parser, trace.id), trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return eachExpression;
    });
}
exports.readEachExpression = readEachExpression;
// ----------------------------------------------
// ---------- 12.2.3.23 Let expression ----------
// ----------------------------------------------
function readLetExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const nodeKind = language_1.Ast.NodeKind.LetExpression;
        const trace = state.traceManager.entry("Parse" /* Parse */, readLetExpression.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const letConstant = readTokenKindAsConstant(state, token_1.TokenKind.KeywordLet, language_1.Constant.KeywordConstant.Let, trace.id);
        const identifierPairedExpression = yield parser.readIdentifierPairedExpressions(state, parser, !parseState_1.ParseStateUtils.isOnTokenKind(state, token_1.TokenKind.KeywordIn), parseState_1.ParseStateUtils.testCsvContinuationLetExpression, trace.id);
        const inConstant = readClosingTokenKindAsConstant(state, token_1.TokenKind.KeywordIn, language_1.Constant.KeywordConstant.In, trace.id);
        const expression = yield parser.readExpression(state, parser, trace.id);
        const letExpression = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: language_1.Ast.NodeKind.LetExpression, isLeaf: false, letConstant, variableList: identifierPairedExpression, inConstant,
            expression });
        parseState_1.ParseStateUtils.endContext(state, letExpression);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return letExpression;
    });
}
exports.readLetExpression = readLetExpression;
// ---------------------------------------------
// ---------- 12.2.3.24 If expression ----------
// ---------------------------------------------
function readIfExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const nodeKind = language_1.Ast.NodeKind.IfExpression;
        const trace = state.traceManager.entry("Parse" /* Parse */, readIfExpression.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const ifConstant = readTokenKindAsConstant(state, token_1.TokenKind.KeywordIf, language_1.Constant.KeywordConstant.If, trace.id);
        const condition = yield parser.readExpression(state, parser, trace.id);
        const thenConstant = readTokenKindAsConstant(state, token_1.TokenKind.KeywordThen, language_1.Constant.KeywordConstant.Then, trace.id);
        const trueExpression = yield parser.readExpression(state, parser, trace.id);
        const elseConstant = readTokenKindAsConstant(state, token_1.TokenKind.KeywordElse, language_1.Constant.KeywordConstant.Else, trace.id);
        const falseExpression = yield parser.readExpression(state, parser, trace.id);
        const ifExpression = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, ifConstant,
            condition,
            thenConstant,
            trueExpression,
            elseConstant,
            falseExpression });
        parseState_1.ParseStateUtils.endContext(state, ifExpression);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return ifExpression;
    });
}
exports.readIfExpression = readIfExpression;
// -----------------------------------------------
// ---------- 12.2.3.25 Type expression ----------
// -----------------------------------------------
function readTypeExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readTypeExpression.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        let typeExpression;
        if (parseState_1.ParseStateUtils.isOnTokenKind(state, token_1.TokenKind.KeywordType)) {
            typeExpression = yield readPairedConstant(state, language_1.Ast.NodeKind.TypePrimaryType, () => readTokenKindAsConstant(state, token_1.TokenKind.KeywordType, language_1.Constant.KeywordConstant.Type, trace.id), () => parser.readPrimaryType(state, parser, trace.id), trace.id);
        }
        else {
            typeExpression = yield parser.readPrimaryExpression(state, parser, trace.id);
        }
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return typeExpression;
    });
}
exports.readTypeExpression = readTypeExpression;
function readType(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readType.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const triedReadPrimaryType = yield tryReadPrimaryType(state, parser, trace.id);
        if (common_1.ResultUtils.isOk(triedReadPrimaryType)) {
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                [trace_1.TraceConstant.IsError]: false,
            });
            return triedReadPrimaryType.value;
        }
        else {
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                [trace_1.TraceConstant.IsError]: true,
            });
            return parser.readPrimaryExpression(state, parser, trace.id);
        }
    });
}
exports.readType = readType;
function readPrimaryType(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readPrimaryType.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const triedReadPrimaryType = yield tryReadPrimaryType(state, parser, trace.id);
        if (common_1.ResultUtils.isOk(triedReadPrimaryType)) {
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                [trace_1.TraceConstant.IsError]: false,
            });
            return triedReadPrimaryType.value;
        }
        else {
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                [trace_1.TraceConstant.IsError]: true,
            });
            throw triedReadPrimaryType.error;
        }
    });
}
exports.readPrimaryType = readPrimaryType;
function readRecordType(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const nodeKind = language_1.Ast.NodeKind.RecordType;
        const trace = state.traceManager.entry("Parse" /* Parse */, readRecordType.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const fields = yield parser.readFieldSpecificationList(state, parser, true, testCsvContinuationDanglingCommaForBracket, trace.id);
        const recordType = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, fields });
        parseState_1.ParseStateUtils.endContext(state, recordType);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return recordType;
    });
}
exports.readRecordType = readRecordType;
function readTableType(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const nodeKind = language_1.Ast.NodeKind.TableType;
        const trace = state.traceManager.entry("Parse" /* Parse */, readTableType.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const tableConstant = readConstantKind(state, language_1.Constant.PrimitiveTypeConstant.Table);
        const currentTokenKind = state.currentTokenKind;
        const isPrimaryExpressionExpected = currentTokenKind === token_1.TokenKind.AtSign ||
            currentTokenKind === token_1.TokenKind.Identifier ||
            currentTokenKind === token_1.TokenKind.LeftParenthesis;
        let rowType;
        if (isPrimaryExpressionExpected) {
            rowType = yield parser.readPrimaryExpression(state, parser, trace.id);
        }
        else {
            rowType = yield parser.readFieldSpecificationList(state, parser, false, testCsvContinuationDanglingCommaForBracket, trace.id);
        }
        const tableType = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, tableConstant,
            rowType });
        parseState_1.ParseStateUtils.endContext(state, tableType);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return tableType;
    });
}
exports.readTableType = readTableType;
function readFieldSpecificationList(state, parser, allowOpenMarker, testPostCommaError, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const nodeKind = language_1.Ast.NodeKind.FieldSpecificationList;
        const trace = state.traceManager.entry("Parse" /* Parse */, readFieldSpecificationList.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const leftBracketConstant = readTokenKindAsConstant(state, token_1.TokenKind.LeftBracket, language_1.Constant.WrapperConstant.LeftBracket, trace.id);
        const fields = [];
        let continueReadingValues = !parseState_1.ParseStateUtils.isOnTokenKind(state, token_1.TokenKind.RightBracket);
        let isOnOpenRecordMarker = false;
        const fieldArrayNodeKind = language_1.Ast.NodeKind.ArrayWrapper;
        parseState_1.ParseStateUtils.startContext(state, fieldArrayNodeKind);
        while (continueReadingValues) {
            const error = testPostCommaError(state);
            if (error) {
                trace.exit({
                    ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                    [trace_1.TraceConstant.IsThrowing]: true,
                });
                throw error;
            }
            if (parseState_1.ParseStateUtils.isOnTokenKind(state, token_1.TokenKind.Ellipsis)) {
                if (allowOpenMarker) {
                    if (isOnOpenRecordMarker) {
                        trace.exit({
                            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                            [trace_1.TraceConstant.IsThrowing]: true,
                        });
                        throw fieldSpecificationListReadError(state, false);
                    }
                    else {
                        isOnOpenRecordMarker = true;
                        continueReadingValues = false;
                    }
                }
                else {
                    trace.exit({
                        ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                        [trace_1.TraceConstant.IsThrowing]: true,
                    });
                    throw fieldSpecificationListReadError(state, allowOpenMarker);
                }
            }
            else if (parseState_1.ParseStateUtils.isOnGeneralizedIdentifierStart(state)) {
                const csvNodeKind = language_1.Ast.NodeKind.Csv;
                parseState_1.ParseStateUtils.startContext(state, csvNodeKind);
                const fieldSpecificationNodeKind = language_1.Ast.NodeKind.FieldSpecification;
                parseState_1.ParseStateUtils.startContext(state, fieldSpecificationNodeKind);
                const optionalConstant = readConstantKindOrUndefined(state, language_1.Constant.LanguageConstant.Optional);
                // eslint-disable-next-line no-await-in-loop
                const name = yield parser.readGeneralizedIdentifier(state, parser, trace.id);
                const fieldTypeSpecification = 
                // eslint-disable-next-line no-await-in-loop
                yield readFieldTypeSpecification(state, parser, trace.id);
                const field = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: fieldSpecificationNodeKind, isLeaf: false, optionalConstant,
                    name,
                    fieldTypeSpecification });
                parseState_1.ParseStateUtils.endContext(state, field);
                const commaConstant = readTokenKindAsConstantOrUndefined(state, token_1.TokenKind.Comma, language_1.Constant.MiscConstant.Comma);
                continueReadingValues = commaConstant !== undefined;
                const csv = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: csvNodeKind, isLeaf: false, node: field, commaConstant });
                parseState_1.ParseStateUtils.endContext(state, csv);
                fields.push(csv);
            }
            else {
                trace.exit({
                    ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                    [trace_1.TraceConstant.IsThrowing]: true,
                });
                throw fieldSpecificationListReadError(state, allowOpenMarker);
            }
        }
        const fieldArray = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: fieldArrayNodeKind, elements: fields, isLeaf: false });
        parseState_1.ParseStateUtils.endContext(state, fieldArray);
        const openRecordMarkerConstant = isOnOpenRecordMarker
            ? readTokenKindAsConstant(state, token_1.TokenKind.Ellipsis, language_1.Constant.MiscConstant.Ellipsis, trace.id)
            : undefined;
        const rightBracketConstant = readClosingTokenKindAsConstant(state, token_1.TokenKind.RightBracket, language_1.Constant.WrapperConstant.RightBracket, trace.id);
        const fieldSpecificationList = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, openWrapperConstant: leftBracketConstant, content: fieldArray, openRecordMarkerConstant, closeWrapperConstant: rightBracketConstant });
        parseState_1.ParseStateUtils.endContext(state, fieldSpecificationList);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return fieldSpecificationList;
    });
}
exports.readFieldSpecificationList = readFieldSpecificationList;
function readFieldTypeSpecification(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const nodeKind = language_1.Ast.NodeKind.FieldTypeSpecification;
        const trace = state.traceManager.entry("Parse" /* Parse */, readFieldTypeSpecification.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const equalConstant = readTokenKindAsConstantOrUndefined(state, token_1.TokenKind.Equal, language_1.Constant.MiscConstant.Equal);
        if (equalConstant) {
            const fieldType = yield parser.readType(state, parser, trace.id);
            const fieldTypeSpecification = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: language_1.Ast.NodeKind.FieldTypeSpecification, isLeaf: false, equalConstant,
                fieldType });
            parseState_1.ParseStateUtils.endContext(state, fieldTypeSpecification);
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                ["isFieldTypeSpecification" /* IsFieldTypeSpecification */]: true,
            });
            return fieldTypeSpecification;
        }
        else {
            parseState_1.ParseStateUtils.incrementAttributeCounter(state);
            parseState_1.ParseStateUtils.deleteContext(state);
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                ["isFieldTypeSpecification" /* IsFieldTypeSpecification */]: false,
            });
            return undefined;
        }
    });
}
function fieldSpecificationListReadError(state, allowOpenMarker) {
    if (allowOpenMarker) {
        const expectedTokenKinds = [token_1.TokenKind.Identifier, token_1.TokenKind.Ellipsis];
        return parseState_1.ParseStateUtils.testIsOnAnyTokenKind(state, expectedTokenKinds);
    }
    else {
        return parseState_1.ParseStateUtils.testIsOnTokenKind(state, token_1.TokenKind.Identifier);
    }
}
function readListType(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readListType.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const listType = yield readWrapped(state, language_1.Ast.NodeKind.ListType, () => readTokenKindAsConstant(state, token_1.TokenKind.LeftBrace, language_1.Constant.WrapperConstant.LeftBrace, trace.id), () => parser.readType(state, parser, trace.id), () => readTokenKindAsConstant(state, token_1.TokenKind.RightBrace, language_1.Constant.WrapperConstant.RightBrace, trace.id), false, trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return listType;
    });
}
exports.readListType = readListType;
function readFunctionType(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const nodeKind = language_1.Ast.NodeKind.FunctionType;
        const trace = state.traceManager.entry("Parse" /* Parse */, readFunctionType.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const functionConstant = readConstantKind(state, language_1.Constant.PrimitiveTypeConstant.Function);
        const parameters = yield parser.readParameterSpecificationList(state, parser, trace.id);
        const functionReturnType = yield parser.readAsType(state, parser, trace.id);
        const functionType = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, functionConstant,
            parameters,
            functionReturnType });
        parseState_1.ParseStateUtils.endContext(state, functionType);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return functionType;
    });
}
exports.readFunctionType = readFunctionType;
function tryReadPrimaryType(state, parser, correlationId) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, tryReadPrimaryType.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        const isTableTypeNext = parseState_1.ParseStateUtils.isOnConstantKind(state, language_1.Constant.PrimitiveTypeConstant.Table) &&
            (parseState_1.ParseStateUtils.isNextTokenKind(state, token_1.TokenKind.LeftBracket) ||
                parseState_1.ParseStateUtils.isNextTokenKind(state, token_1.TokenKind.LeftParenthesis) ||
                parseState_1.ParseStateUtils.isNextTokenKind(state, token_1.TokenKind.AtSign) ||
                parseState_1.ParseStateUtils.isNextTokenKind(state, token_1.TokenKind.Identifier));
        const isFunctionTypeNext = parseState_1.ParseStateUtils.isOnConstantKind(state, language_1.Constant.PrimitiveTypeConstant.Function) &&
            parseState_1.ParseStateUtils.isNextTokenKind(state, token_1.TokenKind.LeftParenthesis);
        let attempt;
        if (parseState_1.ParseStateUtils.isOnTokenKind(state, token_1.TokenKind.LeftBracket)) {
            attempt = common_1.ResultUtils.ok(yield parser.readRecordType(state, parser, trace.id));
        }
        else if (parseState_1.ParseStateUtils.isOnTokenKind(state, token_1.TokenKind.LeftBrace)) {
            attempt = common_1.ResultUtils.ok(yield parser.readListType(state, parser, trace.id));
        }
        else if (isTableTypeNext) {
            attempt = common_1.ResultUtils.ok(yield parser.readTableType(state, parser, trace.id));
        }
        else if (isFunctionTypeNext) {
            attempt = common_1.ResultUtils.ok(yield parser.readFunctionType(state, parser, trace.id));
        }
        else if (parseState_1.ParseStateUtils.isOnConstantKind(state, language_1.Constant.LanguageConstant.Nullable)) {
            attempt = common_1.ResultUtils.ok(yield parser.readNullableType(state, parser, trace.id));
        }
        else {
            const checkpoint = yield parser.checkpoint(state);
            const triedReadPrimitiveType = yield tryReadPrimitiveType(state, parser, trace.id);
            if (common_1.ResultUtils.isError(triedReadPrimitiveType)) {
                yield parser.restoreCheckpoint(state, checkpoint);
            }
            attempt = triedReadPrimitiveType;
        }
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return attempt;
    });
}
function readParameterSpecificationList(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readParameterSpecificationList.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const parameterList = yield genericReadParameterList(state, parser, () => __awaiter(this, void 0, void 0, function* () { return yield parser.readAsType(state, parser, trace.id); }), trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return parameterList;
    });
}
exports.readParameterSpecificationList = readParameterSpecificationList;
function readNullableType(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readNullableType.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const nullableType = yield readPairedConstant(state, language_1.Ast.NodeKind.NullableType, () => readConstantKind(state, language_1.Constant.LanguageConstant.Nullable), () => parser.readType(state, parser, trace.id), trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return nullableType;
    });
}
exports.readNullableType = readNullableType;
// --------------------------------------------------------
// ---------- 12.2.3.26 Error raising expression ----------
// --------------------------------------------------------
function readErrorRaisingExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readErrorRaisingExpression.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const errorRaisingExpression = yield readPairedConstant(state, language_1.Ast.NodeKind.ErrorRaisingExpression, () => readTokenKindAsConstant(state, token_1.TokenKind.KeywordError, language_1.Constant.KeywordConstant.Error, trace.id), () => parser.readExpression(state, parser, trace.id), trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return errorRaisingExpression;
    });
}
exports.readErrorRaisingExpression = readErrorRaisingExpression;
// ---------------------------------------------------------
// ---------- 12.2.3.27 Error handling expression ----------
// ---------------------------------------------------------
function readErrorHandlingExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const nodeKind = language_1.Ast.NodeKind.ErrorHandlingExpression;
        const trace = state.traceManager.entry("Parse" /* Parse */, readErrorHandlingExpression.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const tryConstant = readTokenKindAsConstant(state, token_1.TokenKind.KeywordTry, language_1.Constant.KeywordConstant.Try, trace.id);
        const protectedExpression = yield parser.readExpression(state, parser, trace.id);
        let result;
        // If the literal "catch" is present then read it as a CatchExpression
        if (parseState_1.ParseStateUtils.isOnConstantKind(state, language_1.Constant.LanguageConstant.Catch)) {
            const catchExpression = yield readPairedConstant(state, language_1.Ast.NodeKind.CatchExpression, () => readTokenKindAsConstant(state, token_1.TokenKind.Identifier, language_1.Constant.LanguageConstant.Catch, trace.id), () => parser.readFunctionExpression(state, parser, trace.id), trace.id);
            const error = testCatchFunction(state, catchExpression.paired);
            if (error) {
                trace.exit({
                    ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                    [trace_1.TraceConstant.IsError]: true,
                });
                throw error;
            }
            const errorHandlingCatchExpression = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, handlerKind: language_1.Ast.ErrorHandlerKind.Catch, isLeaf: false, tryConstant,
                protectedExpression, handler: catchExpression });
            result = errorHandlingCatchExpression;
        }
        // optionally read OtherwiseExpression
        else {
            const otherwiseExpression = yield readPairedConstantOrUndefined(state, language_1.Ast.NodeKind.OtherwiseExpression, () => parseState_1.ParseStateUtils.isOnTokenKind(state, token_1.TokenKind.KeywordOtherwise), () => readTokenKindAsConstant(state, token_1.TokenKind.KeywordOtherwise, language_1.Constant.KeywordConstant.Otherwise, trace.id), () => parser.readExpression(state, parser, trace.id), trace.id);
            const errorHandlingOtherwiseExpression = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, handlerKind: language_1.Ast.ErrorHandlerKind.Otherwise, isLeaf: false, tryConstant,
                protectedExpression, handler: otherwiseExpression });
            result = errorHandlingOtherwiseExpression;
        }
        parseState_1.ParseStateUtils.endContext(state, result);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return result;
    });
}
exports.readErrorHandlingExpression = readErrorHandlingExpression;
// -----------------------------------------------
// ---------- 12.2.4 Literal Attributes ----------
// -----------------------------------------------
function readRecordLiteral(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readRecordLiteral.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const continueReadingValues = !parseState_1.ParseStateUtils.isNextTokenKind(state, token_1.TokenKind.RightBracket);
        const wrappedRead = yield readWrapped(state, language_1.Ast.NodeKind.RecordLiteral, () => readTokenKindAsConstant(state, token_1.TokenKind.LeftBracket, language_1.Constant.WrapperConstant.LeftBracket, trace.id), () => parser.readFieldNamePairedAnyLiterals(state, parser, continueReadingValues, testCsvContinuationDanglingCommaForBracket, trace.id), () => readClosingTokenKindAsConstant(state, token_1.TokenKind.RightBracket, language_1.Constant.WrapperConstant.RightBracket, trace.id), false, trace.id);
        const recordLiteral = Object.assign({ literalKind: language_1.Ast.LiteralKind.Record }, wrappedRead);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return recordLiteral;
    });
}
exports.readRecordLiteral = readRecordLiteral;
function readFieldNamePairedAnyLiterals(state, parser, continueReadingValues, testPostCommaError, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readFieldNamePairedAnyLiterals.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const csvArray = yield readCsvArray(state, () => readKeyValuePair(state, language_1.Ast.NodeKind.GeneralizedIdentifierPairedAnyLiteral, () => parser.readGeneralizedIdentifier(state, parser, trace.id), () => parser.readAnyLiteral(state, parser, trace.id), trace.id), continueReadingValues, testPostCommaError, trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return csvArray;
    });
}
exports.readFieldNamePairedAnyLiterals = readFieldNamePairedAnyLiterals;
function readListLiteral(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readListLiteral.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const continueReadingValues = !parseState_1.ParseStateUtils.isNextTokenKind(state, token_1.TokenKind.RightBrace);
        const wrappedRead = yield readWrapped(state, language_1.Ast.NodeKind.ListLiteral, () => readTokenKindAsConstant(state, token_1.TokenKind.LeftBrace, language_1.Constant.WrapperConstant.LeftBrace, trace.id), () => readCsvArray(state, () => parser.readAnyLiteral(state, parser, trace.id), continueReadingValues, testCsvContinuationDanglingCommaForBrace, trace.id), () => readTokenKindAsConstant(state, token_1.TokenKind.RightBrace, language_1.Constant.WrapperConstant.RightBrace, trace.id), false, trace.id);
        const listLiteral = Object.assign({ literalKind: language_1.Ast.LiteralKind.List }, wrappedRead);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return listLiteral;
    });
}
exports.readListLiteral = readListLiteral;
function readAnyLiteral(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readAnyLiteral.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        let anyLiteral;
        if (parseState_1.ParseStateUtils.isOnTokenKind(state, token_1.TokenKind.LeftBracket)) {
            anyLiteral = yield parser.readRecordLiteral(state, parser, trace.id);
        }
        else if (parseState_1.ParseStateUtils.isOnTokenKind(state, token_1.TokenKind.LeftBrace)) {
            anyLiteral = yield parser.readListLiteral(state, parser, trace.id);
        }
        else {
            anyLiteral = parser.readLiteralExpression(state, parser, trace.id);
        }
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return anyLiteral;
    });
}
exports.readAnyLiteral = readAnyLiteral;
function readPrimitiveType(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readPrimitiveType.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const triedReadPrimitiveType = yield tryReadPrimitiveType(state, parser, trace.id);
        if (common_1.ResultUtils.isOk(triedReadPrimitiveType)) {
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                [trace_1.TraceConstant.IsError]: false,
            });
            return triedReadPrimitiveType.value;
        }
        else {
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                [trace_1.TraceConstant.IsError]: true,
            });
            throw triedReadPrimitiveType.error;
        }
    });
}
exports.readPrimitiveType = readPrimitiveType;
function tryReadPrimitiveType(state, parser, correlationId) {
    return __awaiter(this, void 0, void 0, function* () {
        const nodeKind = language_1.Ast.NodeKind.PrimitiveType;
        const trace = state.traceManager.entry("Parse" /* Parse */, tryReadPrimitiveType.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const checkpoint = yield parser.checkpoint(state);
        const expectedTokenKinds = [
            token_1.TokenKind.Identifier,
            token_1.TokenKind.KeywordType,
            token_1.TokenKind.NullLiteral,
        ];
        const error = parseState_1.ParseStateUtils.testIsOnAnyTokenKind(state, expectedTokenKinds);
        if (error) {
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                [trace_1.TraceConstant.IsError]: true,
            });
            return common_1.ResultUtils.error(error);
        }
        let primitiveTypeKind;
        if (parseState_1.ParseStateUtils.isOnTokenKind(state, token_1.TokenKind.Identifier)) {
            const currentTokenData = state.lexerSnapshot.tokens[state.tokenIndex].data;
            switch (currentTokenData) {
                case language_1.Constant.PrimitiveTypeConstant.Action:
                case language_1.Constant.PrimitiveTypeConstant.Any:
                case language_1.Constant.PrimitiveTypeConstant.AnyNonNull:
                case language_1.Constant.PrimitiveTypeConstant.Binary:
                case language_1.Constant.PrimitiveTypeConstant.Date:
                case language_1.Constant.PrimitiveTypeConstant.DateTime:
                case language_1.Constant.PrimitiveTypeConstant.DateTimeZone:
                case language_1.Constant.PrimitiveTypeConstant.Duration:
                case language_1.Constant.PrimitiveTypeConstant.Function:
                case language_1.Constant.PrimitiveTypeConstant.List:
                case language_1.Constant.PrimitiveTypeConstant.Logical:
                case language_1.Constant.PrimitiveTypeConstant.None:
                case language_1.Constant.PrimitiveTypeConstant.Number:
                case language_1.Constant.PrimitiveTypeConstant.Record:
                case language_1.Constant.PrimitiveTypeConstant.Table:
                case language_1.Constant.PrimitiveTypeConstant.Text:
                case language_1.Constant.PrimitiveTypeConstant.Time:
                    primitiveTypeKind = currentTokenData;
                    readToken(state);
                    break;
                default: {
                    const token = parseState_1.ParseStateUtils.assertGetTokenAt(state, state.tokenIndex);
                    yield parser.restoreCheckpoint(state, checkpoint);
                    return common_1.ResultUtils.error(new __1.ParseError.InvalidPrimitiveTypeError(token, state.lexerSnapshot.graphemePositionStartFrom(token), state.locale));
                }
            }
        }
        else if (parseState_1.ParseStateUtils.isOnTokenKind(state, token_1.TokenKind.KeywordType)) {
            primitiveTypeKind = language_1.Constant.PrimitiveTypeConstant.Type;
            readToken(state);
        }
        else if (parseState_1.ParseStateUtils.isOnTokenKind(state, token_1.TokenKind.NullLiteral)) {
            primitiveTypeKind = language_1.Constant.PrimitiveTypeConstant.Null;
            readToken(state);
        }
        else {
            const details = { tokenKind: state.currentTokenKind };
            yield parser.restoreCheckpoint(state, checkpoint);
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                [trace_1.TraceConstant.IsError]: true,
            });
            return common_1.ResultUtils.error(new common_1.CommonError.InvariantError(`unknown currentTokenKind, not found in [${expectedTokenKinds}]`, details));
        }
        const primitiveType = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: true, primitiveTypeKind });
        parseState_1.ParseStateUtils.endContext(state, primitiveType);
        trace.exit({
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
            [trace_1.TraceConstant.IsError]: false,
        });
        return common_1.ResultUtils.ok(primitiveType);
    });
}
// -------------------------------------
// ---------- key-value pairs ----------
// -------------------------------------
function readIdentifierPairedExpressions(state, parser, continueReadingValues, testPostCommaError, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readIdentifierPairedExpression.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const csvArray = yield readCsvArray(state, () => parser.readIdentifierPairedExpression(state, parser, trace.id), continueReadingValues, testPostCommaError, trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return csvArray;
    });
}
exports.readIdentifierPairedExpressions = readIdentifierPairedExpressions;
function readGeneralizedIdentifierPairedExpressions(state, parser, continueReadingValues, testPostCommaError, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readGeneralizedIdentifierPairedExpressions.name, correlationId);
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const csvArray = yield readCsvArray(state, () => parser.readGeneralizedIdentifierPairedExpression(state, parser, trace.id), continueReadingValues, testPostCommaError, trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return csvArray;
    });
}
exports.readGeneralizedIdentifierPairedExpressions = readGeneralizedIdentifierPairedExpressions;
function readGeneralizedIdentifierPairedExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readGeneralizedIdentifierPairedExpression.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const generalizedIdentifierPairedExpression = yield readKeyValuePair(state, language_1.Ast.NodeKind.GeneralizedIdentifierPairedExpression, () => parser.readGeneralizedIdentifier(state, parser, trace.id), () => parser.readExpression(state, parser, trace.id), trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return generalizedIdentifierPairedExpression;
    });
}
exports.readGeneralizedIdentifierPairedExpression = readGeneralizedIdentifierPairedExpression;
function readIdentifierPairedExpression(state, parser, correlationId) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readIdentifierPairedExpression.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
        const identifierPairedExpression = yield readKeyValuePair(state, language_1.Ast.NodeKind.IdentifierPairedExpression, 
        // eslint-disable-next-line require-await
        () => __awaiter(this, void 0, void 0, function* () { return parser.readIdentifier(state, parser, language_1.Ast.IdentifierContextKind.Key, trace.id); }), () => __awaiter(this, void 0, void 0, function* () { return yield parser.readExpression(state, parser, trace.id); }), trace.id);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return identifierPairedExpression;
    });
}
exports.readIdentifierPairedExpression = readIdentifierPairedExpression;
// Some binary expressions can be parsed in any order,
// eg. `1 + 2 + 3` could be as either `[[1 + 2] + 3]` or `[1 + [2 + 3]]`.
//
// However, some expressions need to be parsed iteratively, left-to-right,
// eg. `true is logical is logical` must be parsed `[[true is logical] is logical]`.
//
// This function takes the initial left value and will continually build it up the binary expression,
// so long as the curent token matches the operator token kind.
function readLeftBasedBinOpExpression(state, nodeKind, initialLeft, operatorTokenKind, operatorConstantKind, readRight, correlationId) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readLeftBasedBinOpExpression.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        let left = initialLeft;
        while (state.currentTokenKind === operatorTokenKind) {
            state.currentContextNode = parseState_1.ParseStateUtils.startContextAsParent(state, nodeKind, left.id, trace.id);
            const operatorConstant = __1.NaiveParseSteps.readTokenKindAsConstant(state, operatorTokenKind, operatorConstantKind, trace.id);
            // eslint-disable-next-line no-await-in-loop
            const right = yield readRight(trace.id);
            left = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, left,
                operatorConstant,
                right });
            parseState_1.ParseStateUtils.endContext(state, left);
        }
        trace.exit();
        return left;
    });
}
exports.readLeftBasedBinOpExpression = readLeftBasedBinOpExpression;
// Given the string `1 + 2 + 3` the function will parse the `1 +`,
// then pass the remainder of the string `2 + 3` into recursiveReadBinOpExpressionHelper.
// The helper function is nearly a copy except it replaces Left and leftReader with Right and rightReader.
//
// The reason the code is duplicated across two functions is because I can't think of a cleaner way to do it.
function recursiveReadBinOpExpression(state, nodeKind, leftReader, operatorFrom, rightReader, correlationId) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, recursiveReadBinOpExpression.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const left = yield leftReader();
        // If no operator, return Left
        const operator = operatorFrom(state.currentTokenKind);
        if (operator === undefined) {
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                ["IsOperatorPresent" /* IsOperatorPresent */]: false,
            });
            parseState_1.ParseStateUtils.deleteContext(state);
            return left;
        }
        const operatorConstant = readTokenKindAsConstant(state, common_1.Assert.asDefined(state.currentTokenKind), operator, trace.id);
        const right = yield recursiveReadBinOpExpressionHelper(state, nodeKind, operatorFrom, rightReader, trace.id);
        const binOpExpression = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, left,
            operatorConstant,
            right });
        parseState_1.ParseStateUtils.endContext(state, binOpExpression);
        trace.exit({
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
            ["IsOperatorPresent" /* IsOperatorPresent */]: true,
        });
        return binOpExpression;
    });
}
// Given the string `1 + 2 + 3` the function will recursively parse 2 Ast nodes,
// where their TokenRange's are represented by brackets:
// 1 + [2 + [3]]
function recursiveReadBinOpExpressionHelper(state, nodeKind, operatorFrom, rightReader, correlationId) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, recursiveReadBinOpExpressionHelper.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const rightAsLeft = yield rightReader();
        const operator = operatorFrom(state.currentTokenKind);
        if (operator === undefined) {
            parseState_1.ParseStateUtils.deleteContext(state);
            trace.exit({
                ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                ["IsOperatorPresent" /* IsOperatorPresent */]: false,
            });
            return rightAsLeft;
        }
        const operatorConstant = readTokenKindAsConstant(state, common_1.Assert.asDefined(state.currentTokenKind), operator, trace.id);
        const right = yield recursiveReadBinOpExpressionHelper(state, nodeKind, operatorFrom, rightReader, trace.id);
        const binOpExpression = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, left: rightAsLeft, operatorConstant,
            right });
        parseState_1.ParseStateUtils.endContext(state, binOpExpression);
        trace.exit({
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
            ["IsOperatorPresent" /* IsOperatorPresent */]: true,
        });
        return binOpExpression;
    });
}
function readCsvArray(state, valueReader, continueReadingValues, testPostCommaError, correlationId) {
    return __awaiter(this, void 0, void 0, function* () {
        const nodeKind = language_1.Ast.NodeKind.ArrayWrapper;
        const trace = state.traceManager.entry("Parse" /* Parse */, readCsvArray.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const elements = [];
        while (continueReadingValues) {
            const csvNodeKind = language_1.Ast.NodeKind.Csv;
            parseState_1.ParseStateUtils.startContext(state, csvNodeKind);
            const error = testPostCommaError(state);
            if (error) {
                trace.exit({
                    ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                    [trace_1.TraceConstant.IsThrowing]: true,
                });
                throw error;
            }
            // eslint-disable-next-line no-await-in-loop
            const node = yield valueReader();
            const commaConstant = readTokenKindAsConstantOrUndefined(state, token_1.TokenKind.Comma, language_1.Constant.MiscConstant.Comma);
            const element = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: csvNodeKind, isLeaf: false, node,
                commaConstant });
            parseState_1.ParseStateUtils.endContext(state, element);
            elements.push(element);
            continueReadingValues = commaConstant !== undefined;
        }
        const csvArray = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, elements });
        parseState_1.ParseStateUtils.endContext(state, csvArray);
        trace.exit({
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
            [trace_1.TraceConstant.IsThrowing]: false,
        });
        return csvArray;
    });
}
function readKeyValuePair(state, nodeKind, keyReader, valueReader, correlationId) {
    return __awaiter(this, void 0, void 0, function* () {
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const trace = state.traceManager.entry("Parse" /* Parse */, readKeyValuePair.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        const key = yield keyReader();
        const equalConstant = readTokenKindAsConstant(state, token_1.TokenKind.Equal, language_1.Constant.MiscConstant.Equal, trace.id);
        const value = yield valueReader();
        const keyValuePair = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, key,
            equalConstant,
            value });
        parseState_1.ParseStateUtils.endContext(state, keyValuePair);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return keyValuePair;
    });
}
function readPairedConstant(state, nodeKind, constantReader, pairedReader, correlationId) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readPairedConstant.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const constant = constantReader();
        const paired = yield pairedReader();
        const pairedConstant = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, constant,
            paired });
        parseState_1.ParseStateUtils.endContext(state, pairedConstant);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return pairedConstant;
    });
}
function readPairedConstantOrUndefined(state, nodeKind, condition, constantReader, pairedReader, correlationId) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readPairedConstantOrUndefined.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        let pairedConstant;
        if (condition()) {
            pairedConstant = yield readPairedConstant(state, nodeKind, constantReader, pairedReader, trace.id);
        }
        else {
            parseState_1.ParseStateUtils.incrementAttributeCounter(state);
            pairedConstant = undefined;
        }
        trace.exit({
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
            ["IsOperatorPresent" /* IsOperatorPresent */]: pairedConstant !== undefined,
        });
        return pairedConstant;
    });
}
function genericReadParameterList(state, parser, typeReader, correlationId) {
    return __awaiter(this, void 0, void 0, function* () {
        const nodeKind = language_1.Ast.NodeKind.ParameterList;
        const trace = state.traceManager.entry("Parse" /* Parse */, genericReadParameterList.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const leftParenthesisConstant = readTokenKindAsConstant(state, token_1.TokenKind.LeftParenthesis, language_1.Constant.WrapperConstant.LeftParenthesis, trace.id);
        let continueReadingValues = !parseState_1.ParseStateUtils.isOnTokenKind(state, token_1.TokenKind.RightParenthesis);
        let reachedOptionalParameter = false;
        const paramterArrayNodeKind = language_1.Ast.NodeKind.ArrayWrapper;
        parseState_1.ParseStateUtils.startContext(state, paramterArrayNodeKind);
        const parameters = [];
        while (continueReadingValues) {
            parseState_1.ParseStateUtils.startContext(state, language_1.Ast.NodeKind.Csv);
            parseState_1.ParseStateUtils.startContext(state, language_1.Ast.NodeKind.Parameter);
            const error = testCsvContinuationDanglingCommaForParenthesis(state);
            if (error) {
                trace.exit({
                    ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                    [trace_1.TraceConstant.IsThrowing]: true,
                });
                throw error;
            }
            const optionalConstant = readConstantKindOrUndefined(state, language_1.Constant.LanguageConstant.Optional);
            if (reachedOptionalParameter && !optionalConstant) {
                const token = parseState_1.ParseStateUtils.assertGetTokenAt(state, state.tokenIndex);
                trace.exit({
                    ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
                    [trace_1.TraceConstant.IsThrowing]: true,
                });
                throw new __1.ParseError.RequiredParameterAfterOptionalParameterError(token, state.lexerSnapshot.graphemePositionStartFrom(token), state.locale);
            }
            else if (optionalConstant) {
                reachedOptionalParameter = true;
            }
            // eslint-disable-next-line no-await-in-loop
            const name = parser.readIdentifier(state, parser, language_1.Ast.IdentifierContextKind.Parameter, trace.id);
            // eslint-disable-next-line no-await-in-loop
            const parameterType = yield typeReader();
            const parameter = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: language_1.Ast.NodeKind.Parameter, isLeaf: false, optionalConstant,
                name,
                parameterType });
            parseState_1.ParseStateUtils.endContext(state, parameter);
            const commaConstant = readTokenKindAsConstantOrUndefined(state, token_1.TokenKind.Comma, language_1.Constant.MiscConstant.Comma);
            continueReadingValues = commaConstant !== undefined;
            const csv = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: language_1.Ast.NodeKind.Csv, isLeaf: false, node: parameter, commaConstant });
            parseState_1.ParseStateUtils.endContext(state, csv);
            parameters.push(csv);
        }
        const parameterArray = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: paramterArrayNodeKind, elements: parameters, isLeaf: false });
        parseState_1.ParseStateUtils.endContext(state, parameterArray);
        const rightParenthesisConstant = readTokenKindAsConstant(state, token_1.TokenKind.RightParenthesis, language_1.Constant.WrapperConstant.RightParenthesis, trace.id);
        const parameterList = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, openWrapperConstant: leftParenthesisConstant, content: parameterArray, closeWrapperConstant: rightParenthesisConstant });
        parseState_1.ParseStateUtils.endContext(state, parameterList);
        trace.exit({
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
            [trace_1.TraceConstant.IsThrowing]: false,
        });
        return parameterList;
    });
}
function readWrapped(state, nodeKind, openConstantReader, contentReader, closeConstantReader, allowOptionalConstant, correlationId) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = state.traceManager.entry("Parse" /* Parse */, readWrapped.name, correlationId, {
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        });
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const openWrapperConstant = openConstantReader();
        const content = yield contentReader();
        const closeWrapperConstant = closeConstantReader();
        let optionalConstant;
        if (allowOptionalConstant) {
            optionalConstant = readTokenKindAsConstantOrUndefined(state, token_1.TokenKind.QuestionMark, language_1.Constant.MiscConstant.QuestionMark);
        }
        const wrapped = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: false, openWrapperConstant,
            content,
            closeWrapperConstant,
            optionalConstant });
        parseState_1.ParseStateUtils.endContext(state, wrapped);
        trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
        return wrapped;
    });
}
// ---------------------------------------------
// ---------- Helper functions (read) ----------
// ---------------------------------------------
function readToken(state) {
    var _a;
    (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
    const tokens = state.lexerSnapshot.tokens;
    common_1.Assert.isFalse(state.tokenIndex >= tokens.length, `index is beyond tokens.length`, {
        ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        tokensLength: tokens.length,
    });
    const data = tokens[state.tokenIndex].data;
    state.tokenIndex += 1;
    if (state.tokenIndex === tokens.length) {
        // Each node should have a token range of either [start, finish).
        // That idea breaks if a required parse takes place at the end of the token stream.
        // Eg. `let x = 1 |` will attempt a parse for `in`.
        // That means a correct implementation would have some sort of TokenRange | Eof union type,
        // but there's no clean way to introduce that.
        //
        // So, for now when a IParseState is Eof when currentTokenKind === undefined.
        state.currentTokenKind = undefined;
    }
    else {
        state.currentToken = tokens[state.tokenIndex];
        state.currentTokenKind = state.currentToken.kind;
    }
    return data;
}
exports.readToken = readToken;
function readClosingTokenKindAsConstant(state, tokenKind, constantKind, correlationId) {
    var _a;
    (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
    parseState_1.ParseStateUtils.startContext(state, language_1.Ast.NodeKind.Constant);
    const trace = state.traceManager.entry("Parse" /* Parse */, readTokenKindAsConstant.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
    const error = parseState_1.ParseStateUtils.testClosingTokenKind(state, tokenKind);
    if (error !== undefined) {
        trace.exit({
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
            [trace_1.TraceConstant.IsThrowing]: true,
        });
        throw error;
    }
    const result = readTokenKindAsConstantInternal(state, tokenKind, constantKind, trace.id);
    trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
    return result;
}
exports.readClosingTokenKindAsConstant = readClosingTokenKindAsConstant;
function readTokenKindAsConstant(state, tokenKind, constantKind, correlationId) {
    var _a;
    const trace = state.traceManager.entry("Parse" /* Parse */, readTokenKindAsConstant.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
    (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
    parseState_1.ParseStateUtils.startContext(state, language_1.Ast.NodeKind.Constant);
    const result = readTokenKindAsConstantInternal(state, tokenKind, constantKind, trace.id);
    trace.exit({ ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
    return result;
}
exports.readTokenKindAsConstant = readTokenKindAsConstant;
// Shares logic common to readTokenKindAsConstant and readClosingTokenKindAsConstant.
// Assumes the caller started a context for `Ast.NodeKInd.Constant`.
function readTokenKindAsConstantInternal(state, tokenKind, constantKind, correlationId) {
    const trace = state.traceManager.entry("Parse" /* Parse */, readTokenKindAsConstant.name, correlationId, { ["TokenIndex" /* TokenIndex */]: state.tokenIndex });
    const error = parseState_1.ParseStateUtils.testIsOnTokenKind(state, tokenKind);
    if (error !== undefined) {
        trace.exit({
            ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
            [trace_1.TraceConstant.IsError]: true,
        });
        throw error;
    }
    const tokenData = readToken(state);
    common_1.Assert.isTrue(tokenData === constantKind, `expected tokenData to equal constantKind`, { tokenData, constantKind });
    const constant = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: language_1.Ast.NodeKind.Constant, isLeaf: true, constantKind });
    parseState_1.ParseStateUtils.endContext(state, constant);
    trace.exit({
        ["TokenIndex" /* TokenIndex */]: state.tokenIndex,
        [trace_1.TraceConstant.IsError]: false,
    });
    return constant;
}
function readTokenKindAsConstantOrUndefined(state, tokenKind, constantKind) {
    var _a;
    (_a = state.cancellationToken) === null || _a === void 0 ? void 0 : _a.throwIfCancelled();
    if (parseState_1.ParseStateUtils.isOnTokenKind(state, tokenKind)) {
        const nodeKind = language_1.Ast.NodeKind.Constant;
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        const tokenData = readToken(state);
        common_1.Assert.isTrue(tokenData === constantKind, `expected tokenData to equal constantKind`, {
            tokenData,
            constantKind,
        });
        const constant = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: true, constantKind });
        parseState_1.ParseStateUtils.endContext(state, constant);
        return constant;
    }
    else {
        parseState_1.ParseStateUtils.incrementAttributeCounter(state);
        return undefined;
    }
}
exports.readTokenKindAsConstantOrUndefined = readTokenKindAsConstantOrUndefined;
function readTokenKind(state, tokenKind) {
    const error = parseState_1.ParseStateUtils.testIsOnTokenKind(state, tokenKind);
    if (error) {
        throw error;
    }
    return readToken(state);
}
function readConstantKind(state, constantKind) {
    return common_1.Assert.asDefined(readConstantKindOrUndefined(state, constantKind), `couldn't conver constantKind`, {
        constantKind,
    });
}
function readConstantKindOrUndefined(state, constantKind) {
    if (parseState_1.ParseStateUtils.isOnConstantKind(state, constantKind)) {
        const nodeKind = language_1.Ast.NodeKind.Constant;
        parseState_1.ParseStateUtils.startContext(state, nodeKind);
        readToken(state);
        const constant = Object.assign(Object.assign({}, parseState_1.ParseStateUtils.assertGetContextNodeMetadata(state)), { kind: nodeKind, isLeaf: true, constantKind });
        parseState_1.ParseStateUtils.endContext(state, constant);
        return constant;
    }
    else {
        parseState_1.ParseStateUtils.incrementAttributeCounter(state);
        return undefined;
    }
}
function readLiteralAttributes(state, parser, correlationId) {
    if (parseState_1.ParseStateUtils.isOnTokenKind(state, token_1.TokenKind.LeftBracket)) {
        return parser.readRecordLiteral(state, parser, correlationId);
    }
    else {
        parseState_1.ParseStateUtils.incrementAttributeCounter(state);
        return Promise.resolve(undefined);
    }
}
// -------------------------------------------------------
// ---------- Helper functions (test functions) ----------
// -------------------------------------------------------
function testCsvContinuationDanglingCommaForBrace(state) {
    return parseState_1.ParseStateUtils.testCsvContinuationDanglingComma(state, token_1.TokenKind.RightBrace);
}
function testCsvContinuationDanglingCommaForBracket(state) {
    return parseState_1.ParseStateUtils.testCsvContinuationDanglingComma(state, token_1.TokenKind.RightBracket);
}
function testCsvContinuationDanglingCommaForParenthesis(state) {
    return parseState_1.ParseStateUtils.testCsvContinuationDanglingComma(state, token_1.TokenKind.RightParenthesis);
}
// It must:
//  - have [0, 1] arguments
//  - no typing for either the argument or return
function testCatchFunction(state, catchFunction) {
    const parameters = catchFunction.parameters.content.elements;
    if (parameters.length > 1 ||
        (parameters.length === 1 && parameters[0].node.parameterType) ||
        catchFunction.functionReturnType) {
        const tokenStart = common_1.Assert.asDefined(state.lexerSnapshot.tokens[catchFunction.tokenRange.tokenIndexStart]);
        return new __1.ParseError.InvalidCatchFunctionError(tokenStart, state.lexerSnapshot.graphemePositionStartFrom(tokenStart), state.locale);
    }
    return undefined;
}
//# sourceMappingURL=naiveParseSteps.js.map