"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tryLexParse = exports.tryParse = exports.tryLex = exports.isParseStageParseError = exports.isParseStageCommonError = exports.isParseStageError = exports.isParseStageOk = exports.isParseStage = exports.isOk = exports.isLexStageError = exports.isLexStageOk = exports.isLexStage = exports.isError = exports.assertIsParseStageParseError = exports.assertIsParseStageCommonError = exports.assertIsParseStageError = exports.assertIsParseStageOk = exports.assertIsParseStage = exports.assertIsOk = exports.assertIsLexStageError = exports.assertIsLexStageOk = exports.assertIsLexStage = exports.assertIsError = void 0;
const common_1 = require("../common");
const __1 = require("..");
const task_1 = require("./task");
const parser_1 = require("../parser");
function assertIsError(task) {
    if (!isError(task)) {
        throw new common_1.CommonError.InvariantError(`assert failed, expected a different task result kind`, {
            expectedResultKind: common_1.ResultKind.Error,
            actualResultKind: task.resultKind,
        });
    }
}
exports.assertIsError = assertIsError;
function assertIsLexStage(task) {
    if (!isLexStage(task)) {
        throw new common_1.CommonError.InvariantError(`assert failed, expected a different task stage kind`, {
            expected: task_1.TaskStage.Lex,
            actual: task.resultKind,
        });
    }
}
exports.assertIsLexStage = assertIsLexStage;
function assertIsLexStageOk(task) {
    if (!isLexStageOk(task)) {
        throw new common_1.CommonError.InvariantError(`assert failed, expected a different task result kind`, {
            expectedResultKind: common_1.ResultKind.Ok,
            expectedTaskStage: task_1.TaskStage.Lex,
            actualResultKind: task.resultKind,
            acutalTaskStage: task.stage,
        });
    }
}
exports.assertIsLexStageOk = assertIsLexStageOk;
function assertIsLexStageError(task) {
    if (!isLexStageError(task)) {
        throw new common_1.CommonError.InvariantError(`assert failed, expected a different task result kind`, {
            expectedResultKind: common_1.ResultKind.Error,
            expectedTaskStage: task_1.TaskStage.Lex,
            actualResultKind: task.resultKind,
            acutalTaskStage: task.stage,
        });
    }
}
exports.assertIsLexStageError = assertIsLexStageError;
function assertIsOk(task) {
    if (!isOk(task)) {
        throw new common_1.CommonError.InvariantError(`assert failed, expected a different task result kind`, {
            expectedResultKind: common_1.ResultKind.Ok,
            actualResultKind: task.resultKind,
        });
    }
}
exports.assertIsOk = assertIsOk;
function assertIsParseStage(task) {
    if (!isParseStage(task)) {
        throw new common_1.CommonError.InvariantError(`assert failed, expected a different task stage kind`, {
            expectedTaskStage: task_1.TaskStage.Parse,
            acutalTaskStage: task.stage,
        });
    }
}
exports.assertIsParseStage = assertIsParseStage;
function assertIsParseStageOk(task) {
    if (!isParseStageOk(task)) {
        throw new common_1.CommonError.InvariantError(`assert failed, expected a different task result kind`, {
            expectedResultKind: common_1.ResultKind.Ok,
            expectedTaskStage: task_1.TaskStage.Parse,
            actualResultKind: task.resultKind,
            acutalTaskStage: task.stage,
        });
    }
}
exports.assertIsParseStageOk = assertIsParseStageOk;
function assertIsParseStageError(task) {
    if (!isParseStageError(task)) {
        throw new common_1.CommonError.InvariantError(`assert failed, expected a different task result kind`, {
            expectedResultKind: common_1.ResultKind.Error,
            expectedTaskStage: task_1.TaskStage.Parse,
            actualResultKind: task.resultKind,
            acutalTaskStage: task.stage,
        });
    }
}
exports.assertIsParseStageError = assertIsParseStageError;
function assertIsParseStageCommonError(task) {
    if (!isParseStageCommonError(task)) {
        throw new common_1.CommonError.InvariantError(`assert failed, expected a different task result kind`, {
            expectedResultKind: common_1.ResultKind.Error,
            expectedTaskStage: task_1.TaskStage.Parse,
            expectedIsCommonError: true,
            actualResultKind: task.resultKind,
            acutalTaskStage: task.stage,
            actualIsCommonError: isParseStageError(task) ? task.isCommonError : undefined,
        });
    }
}
exports.assertIsParseStageCommonError = assertIsParseStageCommonError;
function assertIsParseStageParseError(task) {
    if (!isParseStageParseError(task)) {
        throw new common_1.CommonError.InvariantError(`assert failed, expected a different task result kind`, {
            expectedResultKind: common_1.ResultKind.Error,
            expectedTaskStage: task_1.TaskStage.Parse,
            expectedIsCommonError: true,
            actualResultKind: task.resultKind,
            acutalTaskStage: task.stage,
            actualIsCommonError: isParseStageError(task) ? task.isCommonError : undefined,
        });
    }
}
exports.assertIsParseStageParseError = assertIsParseStageParseError;
function isError(task) {
    return task.resultKind === common_1.ResultKind.Error;
}
exports.isError = isError;
function isLexStage(task) {
    return task.stage === task_1.TaskStage.Lex;
}
exports.isLexStage = isLexStage;
function isLexStageOk(task) {
    return isLexStage(task) && task.resultKind === common_1.ResultKind.Ok;
}
exports.isLexStageOk = isLexStageOk;
function isLexStageError(task) {
    return isLexStage(task) && task.resultKind === common_1.ResultKind.Error;
}
exports.isLexStageError = isLexStageError;
function isOk(task) {
    return task.resultKind === common_1.ResultKind.Ok;
}
exports.isOk = isOk;
function isParseStage(task) {
    return task.stage === task_1.TaskStage.Parse;
}
exports.isParseStage = isParseStage;
function isParseStageOk(task) {
    return isParseStage(task) && task.resultKind === common_1.ResultKind.Ok;
}
exports.isParseStageOk = isParseStageOk;
function isParseStageError(task) {
    return isParseStage(task) && task.resultKind === common_1.ResultKind.Error;
}
exports.isParseStageError = isParseStageError;
function isParseStageCommonError(task) {
    return isParseStageError(task) && task.isCommonError;
}
exports.isParseStageCommonError = isParseStageCommonError;
function isParseStageParseError(task) {
    return isParseStageError(task) && !task.isCommonError;
}
exports.isParseStageParseError = isParseStageParseError;
function tryLex(settings, text) {
    const triedLex = __1.Lexer.tryLex(settings, text);
    if (common_1.ResultUtils.isError(triedLex)) {
        return createLexTaskError(triedLex.error);
    }
    const state = triedLex.value;
    const errorLineMap = __1.Lexer.errorLineMap(state);
    if (errorLineMap) {
        return createLexTaskError(new __1.Lexer.LexError.LexError(new __1.Lexer.LexError.ErrorLineMapError(errorLineMap, settings.locale)));
    }
    const triedLexerSnapshot = __1.Lexer.trySnapshot(state);
    if (common_1.ResultUtils.isOk(triedLexerSnapshot)) {
        return createLexTaskOk(triedLexerSnapshot.value);
    }
    else {
        return createLexTaskError(triedLexerSnapshot.error);
    }
}
exports.tryLex = tryLex;
function tryParse(settings, lexerSnapshot) {
    return __awaiter(this, void 0, void 0, function* () {
        const triedParse = yield parser_1.ParserUtils.tryParse(settings, lexerSnapshot);
        if (common_1.ResultUtils.isOk(triedParse)) {
            return createParseTaskOk(lexerSnapshot, triedParse.value.root, triedParse.value.state);
        }
        else if (common_1.CommonError.isCommonError(triedParse.error)) {
            return createParseTaskCommonError(lexerSnapshot, triedParse.error);
        }
        else {
            return createParseTaskParseError(lexerSnapshot, triedParse.error);
        }
    });
}
exports.tryParse = tryParse;
function tryLexParse(settings, text) {
    return __awaiter(this, void 0, void 0, function* () {
        const trace = settings.traceManager.entry("LexParse" /* LexParse */, tryLexParse.name, settings.initialCorrelationId);
        const updatedSettings = Object.assign(Object.assign({}, settings), { initialCorrelationId: trace.id });
        const triedLexTask = tryLex(updatedSettings, text);
        if (triedLexTask.resultKind === common_1.ResultKind.Error) {
            return triedLexTask;
        }
        const lexerSnapshot = triedLexTask.lexerSnapshot;
        const result = yield tryParse(updatedSettings, lexerSnapshot);
        trace.exit();
        return result;
    });
}
exports.tryLexParse = tryLexParse;
function createLexTaskOk(lexerSnapshot) {
    return {
        stage: task_1.TaskStage.Lex,
        resultKind: common_1.ResultKind.Ok,
        lexerSnapshot,
    };
}
function createLexTaskError(error) {
    return {
        stage: task_1.TaskStage.Lex,
        resultKind: common_1.ResultKind.Error,
        error,
    };
}
function createParseTaskOk(lexerSnapshot, ast, parseState) {
    return {
        stage: task_1.TaskStage.Parse,
        resultKind: common_1.ResultKind.Ok,
        lexerSnapshot,
        ast,
        parseState,
        nodeIdMapCollection: parseState.contextState.nodeIdMapCollection,
    };
}
function createParseTaskCommonError(lexerSnapshot, error) {
    return {
        stage: task_1.TaskStage.Parse,
        resultKind: common_1.ResultKind.Error,
        isCommonError: true,
        error,
        lexerSnapshot,
    };
}
function createParseTaskParseError(lexerSnapshot, error) {
    const parseState = error.state;
    const contextState = parseState.contextState;
    return {
        stage: task_1.TaskStage.Parse,
        resultKind: common_1.ResultKind.Error,
        isCommonError: false,
        error,
        lexerSnapshot,
        parseState,
        nodeIdMapCollection: contextState.nodeIdMapCollection,
    };
}
//# sourceMappingURL=taskUtils.js.map